/*  GTKtalog.
 *  Copyright (C) 1999-2000  Mathieu VILLEGAS
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>
#ifdef OS_LINUX
#  include <sys/vfs.h>
#endif
#if defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD)
#    include <sys/param.h>
#    include <sys/mount.h>
#endif
#include <gnome.h>
#ifdef USE_GNOMEVFS
#include <libgnomevfs/gnome-vfs-mime-info.h>
//#include <libbonobo.h>
#endif
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "buttonbar.h"
#include "interface.h"
#include "edit.h"
#include "io.h"
#include "folder.h"
#include "compare.h"
#include "progressbar.h"
#include "thread_utils.h"


static GtkWidget *scrolled_window;
static GtkWidget *ctree_window;
static GtkWidget *clist_window;
static GtkWidget *hpaned;
static GtkWidget *vbox_containing_ctree;
#ifdef GNOME1
static GtkWidget *infotext;	/* HY hack: add a text in main window showing file info */
#endif

static GNode *current_displayed_gnode = NULL;
static gboolean current_displayed_gnode_expand_tree;

GtkWidget *clist;
GtkWidget *path_entry;

struct mime_and_string
{
  MIME_EXTENSIONS *m;
  gchar *s;
};

void
ctree_window_add (GtkWidget * ctree)
{
  gtk_container_add (GTK_CONTAINER (ctree_window), ctree);
}

void
traverse_add_dir_to_clist (GNode * gn, gpointer data)
{
  const gchar *temp[6] = { " ", " ", " ", " ", " ", " " };
  FOLDER *racine = data;
  gchar *nothing = " ";
  FILE_DATA *fd = NULL;
  gint i;
  PIX *pix;
  guint32 total_size1 = 0;
  guint32 total_size2 = 0;

  if ((is_dir (gn) == TRUE) || (is_disk (gn) == TRUE)
      || (is_vfs_any (gn) == TRUE))
    {
      fd = get_file_data_from_gnode (gn);

      if (fd->a_parent_node_is_vfs == IS_VFS)
	temp[1] = g_strdup ("-");
      else if (is_vfs_any (gn) == TRUE)
	{
	  temp[1] = get_size (fd->taille);
	}
      else
	{
	  if (my_config->display_dir_size)
	    {
	      folder_statistics (racine, gn, NULL, NULL,
				 NULL, NULL, NULL,
				 NULL, NULL, &total_size1, &total_size2,
				 NULL);
	      if (total_size1 >> 12)	/* 12 = 32 - 20; 32 is for the length of total_size; 20 is for Mb */
		{
		  temp[1] = get_size_mb (total_size1);
		}
	      else
		{
		  temp[1] =
		    get_size ((total_size1 * (1 << 20)) + total_size2);
		}
	    }
	  else
	    temp[1] = g_strdup ("0");
	}
      if (fd->a_parent_node_is_vfs == IS_VFS)
	temp[2] = g_strdup ("-");
      else
	temp[2] =
	  get_time (fd->date, my_config->date_format_type,
		    my_config->date_format);
      temp[3] = folder_get_category_from_file_data (fd);
      if ((!temp[3]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[3] = nothing;
      temp[4] = folder_get_description_from_file_data (fd);
      if ((!temp[4]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[4] = nothing;
      temp[5] = folder_get_informations (fd->node);
      if ((!temp[5]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[5] = nothing;

      i = gtk_clist_append (GTK_CLIST (clist), (gchar **) temp);

      if (is_dir (gn) == TRUE)
	pix = my_config->default_dir_pix;
      else if (is_vfs_any (gn) == TRUE)
	pix = my_config->default_vfs_pix;
      else
	pix = my_config->default_disk_pix;
      gtk_clist_set_pixtext (GTK_CLIST (clist), i, 0,
			     fd->name->str, 3, pix->pixmap, *(pix->mask));
      gtk_clist_set_row_data (GTK_CLIST (clist), i, gn);
      g_free ((gpointer) temp[1]);
      g_free ((gpointer) temp[2]);
    }
}

void
traverse_add_file_to_clist (GNode * gn, gpointer data)
{
  const gchar *temp[6] = { " ", " ", " ", " ", " ", " " };
  gchar *nothing = " ";
  FILE_DATA *fd;
  FILE_DATA *fdl;
  PIX *pix;
  gint i;
  GString *filename;

  if ((is_file (gn) == TRUE) || (is_link (gn) == TRUE)
      || (is_unreadable (gn) == TRUE))
    {
      fd = get_file_data_from_gnode (gn);
      filename = fd->name;

      if (is_link (gn) == TRUE)
	{
	  fdl = get_file_data_from_gnode (g_node_first_child (gn));
	  filename = fdl->name;
	}

      if ((fd->a_parent_node_is_vfs == IS_VFS)
	  || (is_unreadable (gn) == TRUE) || (is_link (gn) == TRUE))
	temp[1] = g_strdup ("-");
      else
	temp[1] = get_size (fd->taille);
      if ((fd->a_parent_node_is_vfs == IS_VFS)
	  || (is_unreadable (gn) == TRUE))
	temp[2] = g_strdup ("-");
      else
	temp[2] =
	  get_time (fd->date, my_config->date_format_type,
		    my_config->date_format);
      temp[3] = folder_get_category_from_file_data (fd);
      if ((!temp[3]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[3] = nothing;
      temp[4] = folder_get_description_from_file_data (fd);
      if ((!temp[4]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[4] = nothing;
      temp[5] = folder_get_informations (fd->node);
      if ((!temp[5]) || (fd->a_parent_node_is_vfs == IS_VFS)
	  || (is_unreadable (gn) == TRUE))
	temp[5] = nothing;

      i = gtk_clist_append (GTK_CLIST (clist), (gchar **) temp);

      pix = get_pix_from_gnode (gn);

      gtk_clist_set_pixtext (GTK_CLIST (clist), i, 0,
			     filename->str, 3, pix->pixmap, *(pix->mask));


      gtk_clist_set_row_data (GTK_CLIST (clist), i, gn);
      g_free ((gpointer) temp[1]);
      g_free ((gpointer) temp[2]);

    }
}

/*****************************************************************************************/


gint
find_viewer (gpointer key, gpointer value, gpointer data)
{
  struct mime_and_string *_find_mime = data;
  MIME_EXTENSIONS *m = value;

  if (m->state == FALSE)
    return (FALSE);

  if (strcmp (m->mime->str, _find_mime->s))
    return (FALSE);
  _find_mime->m = m;
  return (TRUE);
}

gchar *
viewer_command_line (gchar * filename, const gchar * command)
{
  GString *command_line = g_string_new (command);
  gchar *retour = NULL;
  gint pos;
  gchar *p;

  if (strstr (filename, "%f"))
    return (NULL);

  while ((p = strstr (command_line->str, "%f")))
    {
      pos = (p - command_line->str);
      command_line = g_string_erase (command_line, pos, 2);
      command_line = g_string_insert (command_line, pos, filename);
    }
  retour = g_strdup (command_line->str);
  g_string_free (command_line, TRUE);
  return (retour);
}

#ifdef GNOME1
/* HY hacks, use a dialog with text widget to show information,
 * (origally it's displayed via a message box)*/
void
show_file_info (const GtkWindow * parent, const gchar * filename,
		const gchar * info)
{
  GString *caption;
  GtkWidget *infodialog, *infotext, *hbox;
  GtkWidget *vscrollbar, *label;

  /* make sure the dialog is 100-chars in wide */
  caption = g_string_new (filename);
  g_string_sprintf (caption, "%-100s", filename);

  infodialog = gnome_dialog_new (_("File information"),
				 GNOME_STOCK_BUTTON_OK,
				 GNOME_STOCK_BUTTON_CANCEL, NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (infodialog), GTK_WINDOW (parent));
/*    gtk_signal_connect(GTK_OBJECT(infodialog), "destroy", 
        GTK_SIGNAL_FUNC(fileinfo_dialog_closed), &infodialog); */

  label = gtk_label_new (caption->str);
  g_string_free (caption, TRUE);


  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (infodialog)->vbox), label,
		      TRUE, TRUE, 0);
  gtk_widget_show (label);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (infodialog)->vbox), hbox,
		      TRUE, TRUE, 0);

  infotext = gtk_text_new (NULL, NULL);

  vscrollbar = gtk_vscrollbar_new (GTK_TEXT (infotext)->vadj);

  gtk_box_pack_start (GTK_BOX (hbox), infotext, TRUE, TRUE, 0);
  gtk_text_set_editable (GTK_TEXT (infotext), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (infotext), FALSE);

  /* void gtk_text_insert( GtkText *text, GdkFont *font, GdkColor *fore, 
     GdkColor *back, const char *chars, gint length );
   */
/*    gnome_dialog_close_hides(GNOME_DIALOG(infodialog), TRUE); */
  gtk_box_pack_start (GTK_BOX (hbox), vscrollbar, FALSE, FALSE, 0);
  gtk_widget_show (vscrollbar);
  gtk_widget_show (infotext);
  gtk_widget_show (hbox);

  gtk_editable_delete_text (GTK_EDITABLE (infotext), 0, -1);
  gtk_text_insert (GTK_TEXT (infotext), NULL, NULL, NULL, info, -1);
  /*gdk_window_resize(infodialog->window, 320, 240); */
  gnome_dialog_run (GNOME_DIALOG (infodialog));

  gnome_dialog_close (GNOME_DIALOG (infodialog));
}
#endif

void
launch_viewer (GNode * gn, gint row)
{
  FILE_DATA *fd, *tmpfd;
  GString *tmpstr;
  GString *fullname;
  GNode *gntmp;
  gchar *name;
  GdkBitmap *mask;
  GdkPixmap *pixmap;
  gint8 tmp;
  const gchar *info;

  gtk_clist_get_pixtext (GTK_CLIST (clist), row, 0, &name, &tmp,
			 &pixmap, &mask);
  fd = get_file_data_from_gnode (gn);
  if (is_link (gn) == TRUE)
    {

    }
  else if (is_unreadable (gn) == TRUE)
    {

    }
  else if (is_file (gn) == FALSE)
    {
      if ((row <= 0) && (strcmp (name, "..") == 0))
	{
	  gn = gn->parent;
	  fd = get_file_data_from_gnode (gn);

	}
      if (g_node_depth (gn) > 2)
	{
	  tmpstr = g_string_new (fd->name->str);
	  g_string_append_c (tmpstr, '/');
	  gntmp = gn->parent;
	  while (g_node_depth (gntmp) > 2)
	    {
	      g_string_prepend_c (tmpstr, '/');
	      tmpfd = get_file_data_from_gnode (gntmp);
	      g_string_prepend (tmpstr, tmpfd->name->str);
	      gntmp = gntmp->parent;
	    }

	  g_string_prepend (tmpstr, "] /");
	  tmpfd = get_file_data_from_gnode (gntmp);
	  g_string_prepend (tmpstr, tmpfd->name->str);
	  g_string_prepend_c (tmpstr, '[');
	}
      else if (g_node_depth (gn) == 2)
	{
	  tmpstr = g_string_new (fd->name->str);
	}
      else
	{
	  FOLDER *racine = get_catalog_from_gnode (gn);
	  tmpstr = g_string_new (racine->catalog_filename->str);
	}
      gtk_entry_set_text (GTK_ENTRY (path_entry), tmpstr->str);
      g_string_free (tmpstr, TRUE);
      if (is_link (gn) == FALSE)
	{
	  list_folder (gn, TRUE);
	}


    }
  else
    {
      /* Find the program that will browse the file */

      gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      fd = get_file_data_from_gnode (gn);
      if ((fd->a_parent_node_is_vfs == IS_VFS)
	  || (fd->a_parent_node_is_vfs == IS_VFS_EXTENDED))
	{
	  WARNING_DIALOG (_
			  ("Files inside Virtual File Systems cannot be displayed"),
			  main_window);
	  return;
	}
      if ((my_config->dbclick_opens_info)
	  && ((info = folder_get_informations (gn))))
	{
#ifndef GNOME1
	  gnome_dialog_run_and_close (GNOME_DIALOG
				      (gnome_ok_dialog_parented
				       (info, GTK_WINDOW (main_window))));
#else
	  fullname = get_real_path_from_node (gn);
	  show_file_info (GTK_WINDOW (main_window), fullname->str, info);
#endif
	}
      else
	{
	  fullname = get_real_path_from_node (gn);
	  if (!g_file_test (fullname->str, G_FILE_TEST_IS_REGULAR))
	    {
	      ERROR_DIALOG (_
			    ("The file does not exist.\nMaybe it was removed or it is on a CD.\nIn this case, load the CD, mount it and try again"),
			    main_window);
	    }
	  else
	    {
	      gchar **args;
	      gint nb_args = 2;
	      gchar *p;
	      const gchar *mime = folder_get_mime (gn);
	      const gchar *viewer = NULL;
	      gchar *command_line;

	      if (mime)
		{
#ifdef USE_GNOMEVFS
		  viewer = gnome_vfs_mime_get_value (mime, "view");
#else
		  viewer = gnome_mime_get_value (mime, "view");
#endif
		  if (viewer)
		    {
		      command_line =
			viewer_command_line (fullname->str, viewer);
		      if (command_line)
			{
			  p = command_line;
			  while (p[0])
			    {
			      if ((p[0] == ' ') || (p[0] == '\t'))
				nb_args++;
			      p++;
			      if ((p[0] == ' ') || (p[0] == '\t'))
				nb_args--;
			    }
			  args =
			    (gchar **) g_malloc (sizeof (gchar *) * nb_args);
			  p = command_line;
			  nb_args = 1;
			  args[0] = command_line;
			  args[1] = command_line;
			  while (p[0])
			    {
			      if ((p[0] == ' ') || (p[0] == '\t'))
				{
				  nb_args++;
				  p[0] = 0;
				  args[nb_args] = p + 1;
				}
			      p++;
			      if ((p[0] == ' ') || (p[0] == '\t'))
				nb_args--;
			    }
			  args[nb_args + 1] = NULL;

			  gnome_execute_async (NULL, nb_args + 1, args);
			}
		      else
			{
			  ERROR_DIALOG (_
					("The filename contains `%f'. This conflicts with the viewer definition."),
					main_window);

			}
		    }
		  else
		    {
		      if (!compare ("^text/.*", mime, COMPARE_REGEX, TRUE))
			{
			  GtkWidget *dialog, *gl;
#ifndef GNOME1
			  GtkTextBuffer *buffer;
			  gchar *stringbuffer;
#endif
			  dialog =
			    gnome_dialog_new (fd->name->str,
					      GNOME_STOCK_BUTTON_CLOSE, NULL);
#ifdef GNOME1
			  gl = gnome_less_new ();
			  gnome_less_show_file (GNOME_LESS (gl),
						fullname->str);
#else
			  gl = gtk_text_view_new ();
			  buffer =
			    gtk_text_view_get_buffer (GTK_TEXT_VIEW (gl));
			  g_file_get_contents (fullname->str, &stringbuffer,
					       NULL, NULL);
			  gtk_text_buffer_set_text (buffer, stringbuffer, -1);
			  gtk_text_view_set_editable (GTK_TEXT_VIEW (gl),
						      FALSE);
#endif
			  gtk_box_pack_start (GTK_BOX
					      (GNOME_DIALOG (dialog)->vbox),
					      gl, TRUE, TRUE, 0);
			  gnome_dialog_set_parent (GNOME_DIALOG (dialog),
						   GTK_WINDOW (main_window));
			  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE,
						 TRUE, FALSE);
			  gtk_widget_set_usize (dialog, 500, 350);
			  gtk_widget_show (gl);
			  gtk_widget_show (dialog);
			  gnome_dialog_run_and_close (GNOME_DIALOG (dialog));

			}
		      else
			{
			  ERROR_DIALOG (_
					("This file has no viewer, and it cannot be displayed as text."),
					main_window);
			}
		    }
		}
	      else
		{
		  ERROR_DIALOG (_("The viewer could not be found."),
				main_window);

		}

	    }
	  g_string_free (fullname, TRUE);
	}
    }
}


MIME_EXTENSIONS *
find_extension (const gchar * mime)
{
  static struct mime_and_string _find_mime;
  _find_mime.m = NULL;
  _find_mime.s = g_strdup (mime);
  g_tree_traverse (my_config->mime_extensions, find_viewer,
		   G_PRE_ORDER, &_find_mime);
  g_free (_find_mime.s);
  return (_find_mime.m);
}

void
fin_fils (int sig)
{
  wait (NULL);
  signal (SIGCHLD, SIG_DFL);
  signal (SIGCHLD, fin_fils);
}

void
cb_select_all (GtkWidget * w, gpointer data)
{
  if (clist)
    gtk_clist_select_all (GTK_CLIST (clist));
}

void
cb_select_none (GtkWidget * w, gpointer data)
{
  if (clist)
    gtk_clist_unselect_all (GTK_CLIST (clist));
}


static void
selection_made (GtkWidget * clist, gint row, gint column,
		GdkEventButton * event, gpointer data)
{
  GNode *gn;
  FILE_DATA *fd;
  FOLDER *racine;
  gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
  if (gn)
    {
      racine = get_catalog_from_gnode (gn);

      fd = get_file_data_from_gnode (racine->tree);
      gtk_ctree_unselect_recursive (GTK_CTREE (racine->ctree),
				    fd->ctree_node);
      gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      racine->selected_folder = gn;

      if (event)
	{
	  if ((event->button == 1) && (event->type == GDK_2BUTTON_PRESS))
	    launch_viewer (gn, row);
	}
    }
}


static void
unselection_made (GtkWidget * clist, gint row, gint column,
		  GdkEventButton * event, gpointer data)
{
  FOLDER *racine;
  FILE_DATA *fd;

  racine =
    get_catalog_from_gnode (gtk_clist_get_row_data (GTK_CLIST (clist), row));

  fd = get_file_data_from_gnode (racine->tree);
  racine->selected_folder = NULL;
}

gboolean
set_vfs_fd (GNode * gn, gpointer data)
{
  FILE_DATA *fd = get_file_data_from_gnode (gn);
  fd->a_parent_node_is_vfs = IS_VFS;
  return (FALSE);
}

void
change_vfs_to_file (FOLDER * racine, GNode * gn, GNode * parent)
{
  FILE_DATA *fd = gn->data;

  GString *bak_name;		/* name of the file/dir */
  guint32 bak_taille;		/* length of the file/dir */
  time_t bak_date;		/* time_t */
  GString *bak_mime;		/* Mime type of the file */
  guint16 bak_categorie;
  guint16 bak_description;
  GString *bak_information;

  bak_name = g_string_new (fd->name->str);
  bak_taille = fd->taille;
  bak_date = fd->date;
  if (fd->information)
    {
      bak_mime = g_string_new (fd->mime->str);
    }
  else
    {
      bak_mime = NULL;
    }
  bak_categorie = fd->categorie;
  bak_description = fd->description;
  if (fd->information)
    {
      bak_information = g_string_new (fd->information->str);
    }
  else
    {
      bak_information = NULL;
    }
  gtk_clist_freeze (GTK_CLIST (racine->ctree));
  suppress_dir (fd->node, TRUE);
  gn =
    folder_add (racine, parent, bak_name->str,
		bak_information->str, IS_FILE, bak_mime->str,
		bak_taille, bak_date, bak_categorie, bak_description);
  fd = gn->data;
  racine->selected_folder = gn;
  add_gnode_to_ctree (fd->node, racine);
  gtk_clist_thaw (GTK_CLIST (racine->ctree));
  g_string_free (bak_name, TRUE);
  g_string_free (bak_mime, TRUE);
  g_string_free (bak_information, TRUE);
}

void
edit_vfs_mode (GtkWidget * w, FOLDER * racine)
{
  gint returned_value = -1;
  GtkWidget *dialog, *label;
  FILE_DATA *fd;


  if (racine->selected_folder != NULL)
    {

      fd = get_file_data_from_gnode (racine->selected_folder);
      if (fd->type == IS_VFS_EXTENDED)
	{

	  dialog = gnome_dialog_new (_("VFS mode"),
				     GNOME_STOCK_BUTTON_YES,
				     GNOME_STOCK_BUTTON_NO,
				     GNOME_STOCK_BUTTON_CANCEL, NULL);

	  label =
	    gtk_label_new (_
			   ("Convert this virtual file system and keep only the names?\n"
			    "(No will erase the virtual file system and only keep the archive file)"));
	  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label,
			      TRUE, TRUE, 0);
	  gtk_widget_show (label);
	  returned_value = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	  while (gtk_events_pending ())
	    gtk_main_iteration ();
	  switch (returned_value)
	    {
	    case 0:
	      g_node_traverse (fd->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
			       set_vfs_fd, NULL);
	      list_folder (racine->selected_folder->parent, TRUE);
	      my_config->racine->is_modified = TRUE;
	      break;
	    case 1:
	      change_vfs_to_file (racine, fd->node,
				  racine->selected_folder->parent);
	      list_folder (racine->selected_folder->parent, TRUE);
	      my_config->racine->is_modified = TRUE;
	      break;
	    }
	}
      else
	{
	  dialog = gnome_dialog_new (_("VFS mode"),
				     GNOME_STOCK_BUTTON_YES,
				     GNOME_STOCK_BUTTON_CANCEL, NULL);

	  label =
	    gtk_label_new (_
			   ("Erase the virtual file system and only keep the archive file?"));
	  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label,
			      TRUE, TRUE, 0);
	  gtk_widget_show (label);
	  returned_value = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	  if (returned_value == 0)
	    {
	      change_vfs_to_file (racine, fd->node,
				  racine->selected_folder->parent);
	      list_folder (racine->selected_folder->parent, TRUE);
	      my_config->racine->is_modified = TRUE;
	    }
	}
    }

}

GnomeUIInfo clist_pressed_menu_multi_dir[] = {
  GNOMEUIINFO_ITEM_NONE (N_("Delete selected files/dirs"), NULL,
			 delete_folder),
  GNOMEUIINFO_END
};

GnomeUIInfo tree_pressed_menu_dir[] = {
  GNOMEUIINFO_ITEM_NONE (N_("Edit directory"), NULL, edit_folder),
  GNOMEUIINFO_ITEM_NONE (N_("Delete directory"), NULL, delete_folder),
  GNOMEUIINFO_ITEM_NONE (N_("Directory statistics"), NULL, stats_folder),
  GNOMEUIINFO_END
};

GnomeUIInfo tree_pressed_menu_vfs[] = {
  GNOMEUIINFO_ITEM_NONE (N_("Edit VFS"), NULL, edit_folder),
  GNOMEUIINFO_ITEM_NONE (N_("VFS mode"), NULL, edit_vfs_mode),
  GNOMEUIINFO_ITEM_NONE (N_("Delete VFS"), NULL, delete_folder),
  GNOMEUIINFO_END
};

GnomeUIInfo tree_pressed_menu_link[] = {
  GNOMEUIINFO_ITEM_NONE (N_("Edit link"), NULL, edit_folder),
  GNOMEUIINFO_END
};

GnomeUIInfo tree_pressed_menu_disk[] = {
  GNOMEUIINFO_ITEM_NONE (N_("Edit disk"), NULL, edit_folder),
  GNOMEUIINFO_ITEM_NONE (N_("Update disk"), NULL, update_disk),
  GNOMEUIINFO_ITEM_NONE (N_("Delete disk"), NULL, delete_folder),
  GNOMEUIINFO_ITEM_NONE (N_("Disk statistics"), NULL, stats_folder),
  GNOMEUIINFO_END
};


GnomeUIInfo clist_pressed_menu_file[] = {
  GNOMEUIINFO_ITEM_NONE (N_("Edit file"), NULL, edit_folder),
  GNOMEUIINFO_ITEM_NONE (N_("Delete file"), NULL, delete_folder),
  GNOMEUIINFO_END
};

gint
cmp_2file_types (GNode * gn1, GNode * gn2)
{
  if (G_NODE_IS_ROOT (gn1->parent))
    {
      if (G_NODE_IS_ROOT (gn2->parent))
	return (0);
      else
	return (-1);
    }
  if (G_NODE_IS_ROOT (gn2->parent))
    return (+1);

  if (g_node_depth (gn1) < g_node_depth (gn2))
    return (-1);
  if (g_node_depth (gn1) > g_node_depth (gn2))
    return (+1);

  if (is_dir (gn1))
    {
      if (is_dir (gn2))
	return (0);
      else
	return (-1);
    }
  if (is_dir (gn2))
    return (+1);

  if (is_vfs (gn1))
    {
      if (is_vfs (gn2))
	return (0);
      else
	return (-1);
    }
  if (is_vfs (gn2))
    return (+1);

  return (0);
}

gint
clist_cmp_2files (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
{
  GNode *gn1, *gn2;
  FILE_DATA *fd1, *fd2;
  gint cmp;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  gn1 = row1->data;
  gn2 = row2->data;

  if ((cmp = cmp_2file_types (gn1, gn2)))
    return (cmp);

  fd1 = get_file_data_from_gnode (gn1);
  fd2 = get_file_data_from_gnode (gn2);

  cmp = sort_filenames_with_numbers (fd1->name->str, fd2->name->str);
  return (cmp);
}

gint
clist_cmp_2gint (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
{
  GNode *gn1, *gn2;
  FILE_DATA *fd1, *fd2;
  gint cmp;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  gn1 = row1->data;
  gn2 = row2->data;

  if ((cmp = cmp_2file_types (gn1, gn2)))
    return (cmp);


  fd1 = get_file_data_from_gnode (gn1);
  fd2 = get_file_data_from_gnode (gn2);

  if (fd1->taille < fd2->taille)
    return (-1);
  if (fd1->taille > fd2->taille)
    return (+1);


  return (0);
}

gint
clist_cmp_2dates (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
{
  GNode *gn1, *gn2;
  FILE_DATA *fd1, *fd2;
  gint cmp;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  gn1 = row1->data;
  gn2 = row2->data;

  if ((cmp = cmp_2file_types (gn1, gn2)))
    return (cmp);

  fd1 = get_file_data_from_gnode (gn1);
  fd2 = get_file_data_from_gnode (gn2);

  if (fd1->date > fd2->date)
    return (-1);
  if (fd1->date < fd2->date)
    return (+1);


  return (0);
}

static void
sort_list (GtkWidget * wid, gint column, gpointer data)
{
  FOLDER *racine = data;

  racine->selected_folder = NULL;
  gtk_clist_set_sort_column (GTK_CLIST (clist), (gint) column);
  if (column == 0)
    {
      gtk_clist_set_compare_func (GTK_CLIST (clist), clist_cmp_2files);
    }
  else if (column == 1)
    {
      gtk_clist_set_compare_func (GTK_CLIST (clist), clist_cmp_2gint);
    }
  else if (column == 2)
    {
      gtk_clist_set_compare_func (GTK_CLIST (clist), clist_cmp_2dates);
    }
  else
    {
      gtk_clist_set_compare_func (GTK_CLIST (clist), NULL);
    }
  gtk_clist_sort (GTK_CLIST (clist));
  my_config->sort_column = column;
}

void
set_currently_displayed_folder_pixtext (FOLDER * racine, gboolean is_open)
{
  gchar *dir_text;
  FILE_DATA *fd;
  PIX *pix;
  GNode *gn;

  if (racine->currently_displayed_gnode)
    {
      gn = racine->currently_displayed_gnode;
      while (g_node_depth (gn) >= 2)
	{
	  fd = get_file_data_from_gnode (gn);
	  if (is_open)
	    {
	      if (g_node_depth (gn) == 2)
		pix = my_config->default_opened_disk_pix;
	      else if (is_vfs_any (gn) == TRUE)
		pix = my_config->default_opened_vfs_pix;
	      else
		pix = my_config->default_opened_dir_pix;

	    }
	  else
	    {
	      if (g_node_depth (gn) == 2)
		pix = my_config->default_disk_pix;
	      else if (is_vfs_any (gn) == TRUE)
		pix = my_config->default_vfs_pix;
	      else
		pix = my_config->default_dir_pix;
	    }

	  gtk_ctree_node_get_pixtext (GTK_CTREE (racine->ctree),
				      fd->ctree_node, 0, &dir_text, NULL,
				      NULL, NULL);
	  gtk_ctree_node_set_pixtext (GTK_CTREE (racine->ctree),
				      fd->ctree_node, 0,
				      g_strdup (dir_text), 4, pix->pixmap,
				      *(pix->mask));
	  gn = gn->parent;

	}
    }
}

/* Displays files and directories in the right window. */
void
list_folder (GNode * parent, gboolean expand_tree)
{
  const gchar *temp[6] = { " ", " ", " ", " ", " ", " " };
  gchar *nothing = " ";
  FILE_DATA *fd = NULL;
  PIX *pix;
  int i;
  /*  selected_clist_folder = NULL; */

  if (parent == NULL)
    {
      parent = current_displayed_gnode;
      expand_tree = current_displayed_gnode_expand_tree;
    }
  current_displayed_gnode = parent;
  current_displayed_gnode_expand_tree = expand_tree;
  if (current_displayed_gnode == NULL)
    return;
  gtk_clist_freeze (GTK_CLIST (clist));
  gtk_clist_clear (GTK_CLIST (clist));
  if (g_node_depth (parent) > 1)
    {
      fd = get_file_data_from_gnode (parent->parent);

      temp[0] = g_strdup ("..");
      if (fd->a_parent_node_is_vfs == IS_VFS)
	temp[1] = g_strdup ("-");
      else
	temp[1] = g_strdup ("0");
      if (fd->a_parent_node_is_vfs == IS_VFS)
	temp[2] = g_strdup ("-");
      else
	temp[2] =
	  get_time (fd->date, my_config->date_format_type,
		    my_config->date_format);
      temp[3] = folder_get_category_from_file_data (fd);
      if ((!temp[3]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[3] = nothing;
      temp[4] = folder_get_description_from_file_data (fd);
      if ((!temp[4]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[4] = nothing;
      temp[5] = folder_get_informations (fd->node);
      if ((!temp[5]) || (fd->a_parent_node_is_vfs == IS_VFS))
	temp[5] = nothing;


      i = gtk_clist_append (GTK_CLIST (clist), (gchar **) temp);
      gtk_clist_set_selectable (GTK_CLIST (clist), i, FALSE);

      if (g_node_depth (parent) == 2)
	pix = my_config->default_disk_pix;
      else if (is_vfs_any (parent) == TRUE)
	pix = my_config->default_vfs_pix;
      else
	pix = my_config->default_dir_pix;

      gtk_clist_set_pixtext (GTK_CLIST (clist), i, 0,
			     temp[0], 3, pix->pixmap, *(pix->mask));
      gtk_clist_set_row_data (GTK_CLIST (clist), i, parent);
      g_free ((gpointer) temp[0]);
      g_free ((gpointer) temp[1]);
      g_free ((gpointer) temp[2]);
    }

  g_node_children_foreach (parent, G_TRAVERSE_ALL, traverse_add_dir_to_clist,
			   my_config->racine);

  if (g_node_depth (parent) > 1)
    g_node_children_foreach (parent, G_TRAVERSE_ALL,
			     traverse_add_file_to_clist, NULL);


  gtk_signal_connect (GTK_OBJECT (clist), "click_column",
		      GTK_SIGNAL_FUNC (sort_list),
		      get_catalog_from_gnode (parent));
  gtk_clist_thaw (GTK_CLIST (clist));

  set_currently_displayed_folder_pixtext (my_config->racine, FALSE);
  catalog_set_currently_displayed_gnode (parent);
  set_currently_displayed_folder_pixtext (my_config->racine, TRUE);

  sort_list (NULL, my_config->sort_column, my_config->racine);

  if (expand_tree)
    expand_folder (parent);

  gtk_clist_columns_autosize (GTK_CLIST (clist));
}

void
key_pressed (GtkWidget * wid, GdkEventKey * event, gpointer data)
{
  if ((event->keyval == GDK_Delete) || (event->keyval == GDK_KP_Delete))
    {
      if (my_config->racine && my_config->racine->selected_folder)
	delete_folder (wid, my_config->racine);
    }
}

void
clist_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data)
{
  gint row, column;
  gint x, y;
  GNode *gn;
  GtkWidget *list_menu;
  FOLDER *racine;
#ifdef GNOME1
  FILE_DATA *fd;
  const gchar *info;
#endif

  if (event->button == 3 || event->button == 1)
    {
      if ((get_thread_status () == THREAD_SCAN_RUNNING_NO)
	  && (!is_search_running ()))
	{
	  x = event->x;
	  y = event->y;
	  if (gtk_clist_get_selection_info
	      (GTK_CLIST (wid), x, y, &row, &column))
	    {
	      gn = gtk_clist_get_row_data (GTK_CLIST (wid), row);
	      racine = get_catalog_from_gnode (gn);
	      racine->selected_folder = gn;

#ifdef GNOME1
	      fd = get_file_data_from_gnode (gn);
	      if ((fd->a_parent_node_is_vfs == IS_VFS)
		  || (fd->a_parent_node_is_vfs == IS_VFS_EXTENDED))
		{
		  return;
		}
	      gtk_editable_delete_text (GTK_EDITABLE (infotext), 0, -1);
	      if ((info = folder_get_informations (gn)))
		{
		  gtk_text_insert (GTK_TEXT (infotext), NULL, NULL, NULL,
				   info, -1);
		}
#endif

  if (event->button == 3)
  {
	      if (g_list_length ((GTK_CLIST (wid))->selection) > 1)
		{
		  list_menu =
		    gnome_popup_menu_new (clist_pressed_menu_multi_dir);
		}
	      else if (is_dir (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_dir);

		}
	      else if (is_link (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_link);
		}
	      else if (is_file (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (clist_pressed_menu_file);
		}
	      else if (is_unreadable (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (clist_pressed_menu_file);
		}
	      else if (is_vfs_any (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_vfs);
		}
	      else
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_disk);
		}
#ifdef GNOME1
	      gnome_popup_menu_do_popup (list_menu, NULL, NULL, event,
					 racine);
#else
	      gnome_popup_menu_do_popup (list_menu, NULL, NULL, event,
					 racine, NULL);
#endif
  }
	    }
	}
    }
}



GtkWidget *
create_list ()
{

  gchar *titles[6] = { _("Filename"), _("Size"), _("Date"), _("Category"),
    _("Description"), _("Information")
  };

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);


  clist = gtk_clist_new_with_titles (6, titles);

  gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_OUT);
  gtk_container_add (GTK_CONTAINER (scrolled_window), clist);
  gtk_signal_connect (GTK_OBJECT (clist), "button_press_event",
		      GTK_SIGNAL_FUNC (clist_pressed), NULL);
  gtk_signal_connect (GTK_OBJECT (clist), "key_press_event",
		      GTK_SIGNAL_FUNC (key_pressed), NULL);
  gtk_signal_connect (GTK_OBJECT (clist), "select_row",
		      GTK_SIGNAL_FUNC (selection_made), NULL);
  gtk_signal_connect (GTK_OBJECT (clist), "unselect_row",
		      GTK_SIGNAL_FUNC (unselection_made), NULL);
  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_EXTENDED);
  gtk_widget_show (clist);

  return scrolled_window;
}

void
tree_selected(GtkWidget * clist, gint row, gint column,
            GdkEventButton * event, gpointer data)
{
  GNode *gn, *gntmp;
  FOLDER *racine;
  FILE_DATA *fd, *tmpfd;
  GString *tmp;

  if ((event->button == 1))
    {
	{
	  gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
	  racine = get_catalog_from_gnode (gn);
	  if (G_NODE_IS_ROOT (gn) != TRUE)
	    {
	      racine->selected_folder = gn;

	      fd = get_file_data_from_gnode (gn);

	      if (g_node_depth (gn) > 2)
		{
		  tmp = g_string_new (fd->name->str);
		  g_string_append_c (tmp, '/');
		  gntmp = gn->parent;
		  while (g_node_depth (gntmp) > 2)
		    {
		      g_string_prepend_c (tmp, '/');
		      tmpfd = get_file_data_from_gnode (gntmp);
		      g_string_prepend (tmp, tmpfd->name->str);
		      gntmp = gntmp->parent;
		    }

		  g_string_prepend (tmp, "] /");
		  tmpfd = get_file_data_from_gnode (gntmp);
		  g_string_prepend (tmp, tmpfd->name->str);
		  g_string_prepend_c (tmp, '[');
		}
	      else if (g_node_depth (gn) == 2)
		{
		  tmp = g_string_new (fd->name->str);
		  tmp = g_string_prepend_c (tmp, '[');
		  tmp = g_string_append_c (tmp, ']');
		}
	      else
		{
		  tmp = g_string_new (racine->catalog_filename->str);
		}
	      gtk_entry_set_text (GTK_ENTRY (path_entry), tmp->str);
	      g_string_free (tmp, TRUE);
	      if (is_link (gn) == FALSE)
		{
		  list_folder (gn, FALSE);
		}
	    }
	}
    }
}


void
tree_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data)
{
  GtkWidget *list_menu;
  FOLDER *racine = data;
  gint row, column;
  gint x, y;
  GNode *gn;

  if (event->button == 3)
    {
      if ((get_thread_status () == THREAD_SCAN_RUNNING_NO)
	  && (!is_search_running ()))
	{
	  x = event->x;
	  y = event->y;
	  if (gtk_clist_get_selection_info
	      (GTK_CLIST (wid), x, y, &row, &column))
	    {
	      gn = gtk_clist_get_row_data (GTK_CLIST (wid), row);
	      racine = get_catalog_from_gnode (gn);
	      racine->selected_folder = gn;

	      if (is_dir (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_dir);

		}
	      else if (is_link (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_link);
		}
	      else if (is_vfs_any (gn) == TRUE)
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_vfs);
		}
	      else
		{
		  list_menu = gnome_popup_menu_new (tree_pressed_menu_disk);
		}
#ifdef GNOME1
	      gnome_popup_menu_do_popup (list_menu, NULL, NULL, event,
					 racine);
#else
	      gnome_popup_menu_do_popup (list_menu, NULL, NULL, event,
					 racine, NULL);
#endif
	    }
	}
    }

}


void
update_tree (FOLDER * racine)
{
  gtk_ctree_sort_node (GTK_CTREE (racine->ctree), NULL);

  racine->selected_folder = NULL;
  gtk_widget_show_all (racine->ctree);
}

/* GNOME Menu */
GnomeUIInfo recent_files_menu[] = {
  GNOMEUIINFO_END
};

GnomeUIInfo file_menu[] = {
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New catalog"), N_("_New catalog"),
			     cb_new_file,
			     NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (cb_open_cat, NULL),
  {GNOME_APP_UI_ITEM, N_("_Merge..."), NULL, cb_merge_cat, NULL, NULL,
   GNOME_APP_PIXMAP_NONE, NULL,
   GDK_M, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  GNOMEUIINFO_MENU_SAVE_ITEM (cb_fast_save, NULL),
  GNOMEUIINFO_MENU_SAVE_AS_ITEM (cb_save_cat, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_SUBTREE (N_("_Recent files"), recent_files_menu),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (cb_quit_prog, NULL),
  GNOMEUIINFO_END
};

GnomeUIInfo edit_menu[] = {
  {GNOME_APP_UI_ITEM, N_("_Select all"), NULL, cb_select_all, NULL, NULL,
   GNOME_APP_PIXMAP_NONE, NULL,
   GDK_A, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  GNOMEUIINFO_ITEM_NONE (N_("_Unselect all"), NULL, cb_select_none),
  GNOMEUIINFO_SEPARATOR,
  {GNOME_APP_UI_ITEM, N_("_Find file"), NULL, cb_create_search_dialog, NULL,
   NULL, GNOME_APP_PIXMAP_NONE, NULL,
   GDK_F, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  GNOMEUIINFO_END
};

GnomeUIInfo prefs_menu[] = {
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (cb_run_setup, NULL),
  GNOMEUIINFO_END
};

GnomeUIInfo action_menu[] = {
  {GNOME_APP_UI_ITEM, N_("_Edit categories"), NULL, cb_edit_categories, NULL,
   NULL, GNOME_APP_PIXMAP_NONE, NULL,
   GDK_E, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  {GNOME_APP_UI_ITEM, N_("Catalog _information"), NULL, cb_cat_info, NULL,
   NULL, GNOME_APP_PIXMAP_NONE, NULL,
   GDK_I, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  GNOMEUIINFO_SEPARATOR,
  {GNOME_APP_UI_ITEM, N_("Create _Report"), NULL, cb_open_report_dialog, NULL,
   NULL, GNOME_APP_PIXMAP_NONE, NULL,
   GDK_R, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  GNOMEUIINFO_SEPARATOR,
  {GNOME_APP_UI_ITEM, N_("Add a new _CD"), NULL, cb_fast_add_disk, NULL, NULL,
   GNOME_APP_PIXMAP_NONE, NULL,
   GDK_C, (GdkModifierType) GDK_CONTROL_MASK, NULL},
  GNOMEUIINFO_ITEM_NONE (N_("Add a new _directory"), NULL,
			 cb_show_select_dir_win),
  GNOMEUIINFO_END
};

GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_HELP (PACKAGE),
  GNOMEUIINFO_MENU_ABOUT_ITEM (cb_about_gtktalog, NULL),
  GNOMEUIINFO_END
};

GnomeUIInfo menubar[] = {
  GNOMEUIINFO_MENU_FILE_TREE (file_menu),
  GNOMEUIINFO_MENU_EDIT_TREE (edit_menu),
  GNOMEUIINFO_SUBTREE (N_("Catalog"), action_menu),
  GNOMEUIINFO_MENU_SETTINGS_TREE (prefs_menu),
  GNOMEUIINFO_MENU_HELP_TREE (help_menu),
  GNOMEUIINFO_END
};

void
_gnome_add_file_menu (gchar * txt, gpointer data)
{
  GnomeUIInfo sm[] =
    { GNOMEUIINFO_ITEM_DATA (txt, NULL, cb_open_recent, data, NULL),
    GNOMEUIINFO_END
  };
  gchar *path =
    g_strconcat (GNOME_MENU_FILE_PATH, _("_Recent files"), "/", NULL);
  gnome_app_insert_menus (GNOME_APP (main_window), path, sm);
  g_free (path);
}

void
empty_recentfile_menu ()
{
  gchar *path =
    g_strconcat (GNOME_MENU_FILE_PATH, _("_Recent files"), "/", NULL);

  if (my_config->currently_displayed_recent_files)
    gnome_app_remove_menus (GNOME_APP (main_window), path,
			    my_config->currently_displayed_recent_files);

  my_config->currently_displayed_recent_files = 0;
  g_free (path);
}

void
fill_recentfile_menu ()
{
  gchar *path =
    g_strconcat (GNOME_MENU_FILE_PATH, _("_Recent files"), "/", NULL);
  RECENT_FILE *rf;
  GList *gl;

  if (my_config->currently_displayed_recent_files)
    empty_recentfile_menu ();
  gl = my_config->recent_files;
  my_config->currently_displayed_recent_files = 0;
  while (gl)
    {
      rf = gl->data;
      _gnome_add_file_menu (rf->label, rf->fullname);
      my_config->currently_displayed_recent_files++;
      gl = g_list_next (gl);
    }
  g_free (path);
}

GList *
add_file_to_recent_files (GList * recent_files, gint nb_max_recent_files,
			  gchar * _fullname)
{
  RECENT_FILE *rf;
  GList *local_recent_files = recent_files;
  GList *gl;
  gchar *label, *new_label;
  gchar *fullname;
  gchar *p1, *p2;
  gint len;
  gboolean have_underscores;

  fullname = g_strdup (_fullname);
  if (fullname[0] != '/')
    label = g_strdup (fullname);
  else
    {
      label = fullname;
      while (label[0])
	label++;
      while (label[0] != '/')
	label--;
      label++;
      label = g_strdup (label);
    }

  /* if the label contains some underscore chars, duplicate them! */
  len = 1;
  have_underscores = FALSE;
  p1 = label;
  while (p1[0])
    {
      if (p1[0] == '_')
	{
	  len++;
	  have_underscores = TRUE;
	}
      len++;
      p1++;
    }
  if (have_underscores)
    {
      new_label = p2 = (gchar *) g_malloc (sizeof (gchar) * len);
      p1 = label;
      while (p1[0])
	{
	  if (p1[0] == '_')
	    {
	      p2[0] = '_';
	      p2++;
	    }
	  p2[0] = p1[0];
	  p1++;
	  p2++;
	}
      p2[0] = 0;
      g_free (label);
      label = new_label;
    }

  /* Search for existing same entries */
  gl = local_recent_files;
  while (gl)
    {
      rf = gl->data;
      if (strcmp (rf->fullname, fullname) == 0)
	{
	  local_recent_files = g_list_remove_link (local_recent_files, gl);
	  rf = gl->data;
	  g_free (rf->label);
	  g_free (rf->fullname);
	  g_free (rf);
	  g_list_free_1 (gl);
	  gl = local_recent_files;
	}
      else
	{
	  gl = g_list_next (gl);
	}
    }

  /* No more than my_config->nb_max_recent_files entries */
  while (g_list_length (local_recent_files) >= nb_max_recent_files)
    {
      gl = g_list_last (local_recent_files);
      local_recent_files = g_list_remove_link (local_recent_files, gl);
      rf = gl->data;
      g_free (rf->label);
      g_free (rf->fullname);
      g_free (rf);
      g_list_free_1 (gl);
    }

  /* Add the new entry */
  rf = (RECENT_FILE *) g_malloc (sizeof (RECENT_FILE));
  rf->label = label;
  rf->fullname = fullname;
  local_recent_files = g_list_prepend (local_recent_files, rf);
  return (local_recent_files);
}

void
add_file_menu (gchar * fullname)
{
#if defined(__DEBUG__)
  GML_log_point (1, "add_file_menu");
#endif
  empty_recentfile_menu ();
  my_config->recent_files =
    add_file_to_recent_files (my_config->recent_files,
			      my_config->nb_max_recent_files, fullname);
  fill_recentfile_menu ();
}

void
menus_set_sensitive (gboolean sensitive)
{
  gtk_widget_set_sensitive (GTK_WIDGET (file_menu[2].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (file_menu[3].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (file_menu[4].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[0].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[1].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[3].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (action_menu[0].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (action_menu[1].widget), sensitive);
  gtk_widget_set_sensitive (GTK_WIDGET (action_menu[3].widget), sensitive);
  buttons_set_sensitive (sensitive);
}

void
menus_set_all_sensitive (gboolean sensitive)
{
  int i;
  for (i = 0; i < (sizeof (file_menu) / sizeof (GnomeUIInfo)); i++)
    {
      if (file_menu[i].widget)
	gtk_widget_set_sensitive (GTK_WIDGET (file_menu[i].widget),
				  sensitive);
    }
  for (i = 0; i < (sizeof (edit_menu) / sizeof (GnomeUIInfo)); i++)
    {
      if (edit_menu[i].widget)
	gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[i].widget),
				  sensitive);
    }
  for (i = 0; i < (sizeof (action_menu) / sizeof (GnomeUIInfo)); i++)
    {
      if (action_menu[i].widget)
	gtk_widget_set_sensitive (GTK_WIDGET (action_menu[i].widget),
				  sensitive);
    }
  for (i = 0; i < (sizeof (prefs_menu) / sizeof (GnomeUIInfo)); i++)
    {
      if (prefs_menu[i].widget)
	gtk_widget_set_sensitive (GTK_WIDGET (prefs_menu[i].widget),
				  sensitive);
    }
  buttons_set_all_sensitive (sensitive);
}

void
save_ctree_window_size (void)
{
  if (my_config->racine)
    {
      gdk_window_get_size (my_config->racine->ctree->window,
			   &(my_config->ctreewindow_w),
			   &(my_config->ctreewindow_h));
    }
}

void
build_gui (void)
{
  GtkWidget *appbar1;
#ifdef GNOME1
  GtkWidget *vpaned;
  GtkWidget *sclwin;
#endif

/* Main vertical box */
  vbox_containing_ctree = gtk_vbox_new (FALSE, 1);
  gtk_container_border_width (GTK_CONTAINER (vbox_containing_ctree), 1);
  gnome_app_set_contents (GNOME_APP (main_window), vbox_containing_ctree);
  gtk_widget_show (vbox_containing_ctree);

/* Menus and button bar */

  gnome_app_create_menus_with_data (GNOME_APP (main_window), menubar, NULL);
  enable_gnome_button_bar (main_window, my_config->toolbar_type);
  gtk_menu_item_right_justify (GTK_MENU_ITEM (menubar[3].widget));
  menus_set_sensitive (FALSE);
  fill_recentfile_menu ();

/* Status and progress bar */
  appbar1 = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_ref (appbar1);
  gtk_object_set_data_full (GTK_OBJECT (main_window), "appbar1", appbar1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (appbar1);
  gnome_app_set_statusbar (GNOME_APP (main_window), appbar1);
  progress_setGnomeAppBar (appbar1, GNOMEAPPBAR_MAIN, TRUE);

/* Path entry */
  path_entry = gtk_entry_new_with_max_length (300);
  gtk_editable_set_editable (GTK_EDITABLE (path_entry), FALSE);
  gtk_entry_set_text (GTK_ENTRY (path_entry), "");
  gtk_entry_append_text (GTK_ENTRY (path_entry), "");
  gtk_entry_select_region (GTK_ENTRY (path_entry),
			   0, GTK_ENTRY (path_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox_containing_ctree), path_entry, FALSE,
		      TRUE, 0);
  gtk_widget_show (path_entry);


/* Horizontal pan for the tree and file display */
  hpaned = gtk_hpaned_new ();
  gtk_container_add (GTK_CONTAINER (vbox_containing_ctree), hpaned);
#ifdef GNOME1
  gtk_paned_set_handle_size (GTK_PANED (hpaned), 10);
  gtk_paned_set_gutter_size (GTK_PANED (hpaned), 10);
#endif
  gtk_widget_show (hpaned);


#ifdef GNOME1
  /* --- HY hack ------------- */
  vpaned = gtk_vpaned_new ();
  gtk_paned_add2 (GTK_PANED (hpaned), vpaned);
  gtk_paned_set_handle_size (GTK_PANED (vpaned), 10);
  gtk_paned_set_gutter_size (GTK_PANED (vpaned), 10);
  sclwin = gtk_scrolled_window_new (NULL, NULL);

  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sclwin),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  infotext = gtk_text_new (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (sclwin), infotext);

  gtk_widget_show (infotext);
  gtk_widget_set_usize (sclwin, -1, 30);
  gtk_widget_show (sclwin);
  gtk_paned_add2 (GTK_PANED (vpaned), sclwin);

  gtk_widget_show (vpaned);
  /* =============           */

/* Clist window to display the files */
  clist_window = create_list ();
  gtk_widget_set_usize (clist_window, -1, 250);
  gtk_paned_add1 (GTK_PANED (vpaned), clist_window);
  gtk_widget_show (clist_window);
#else
  clist_window = create_list ();
  gtk_paned_add2 (GTK_PANED (hpaned), clist_window);
#endif




/* CTtree window to display the directory tree */
  ctree_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ctree_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  if (my_config->ctreewindow_save_wh)
    gtk_widget_set_usize (ctree_window, my_config->ctreewindow_w,
			  my_config->ctreewindow_h);
  else
#ifdef GNOME1
    gtk_widget_set_usize (ctree_window, 150, 300);
#else
    gtk_widget_set_usize (ctree_window, 150, 150);
#endif

  gtk_widget_show (ctree_window);

  gtk_paned_add1 (GTK_PANED (hpaned), ctree_window);

}

void
stats_folder (GtkWidget * w, FOLDER * racine)
{
  if (racine->selected_folder != NULL)
    {
      show_statistics (racine, racine->selected_folder);
    }
}

void
show_statistics (FOLDER * racine, GNode * gn)
{
  guint32 nb_desc = 0;
  guint32 nb_cat = 0;
  guint32 nb_disks = 0;
  guint32 nb_dirs = 0;
  guint32 nb_dirs_with_vfs = 0;
  guint32 nb_files = 0;
  guint32 nb_files_with_vfs = 0;
  guint32 nb_vfs = 0;
  guint32 total_size = 0;
  gchar tmpstr[20];
  gint y1, y2;

  GtkWidget *dialog;
  GtkWidget *myentry;
  GtkWidget *mylabel;
  GtkWidget *mytable;

  dialog = gnome_dialog_new (_("Statistics"), GNOME_STOCK_BUTTON_CLOSE, NULL);
  if (main_window)
    gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (main_window));

  if (gn == racine->tree)
    {
      folder_statistics (racine, gn, &nb_desc, &nb_cat,
			 &nb_disks, &nb_dirs, &nb_dirs_with_vfs,
			 &nb_files, &nb_files_with_vfs, &total_size, NULL,
			 &nb_vfs);
      y1 = 8;
      y2 = 8;
    }
  else
    {
      folder_statistics (racine, gn, NULL, NULL,
			 NULL, &nb_dirs, &nb_dirs_with_vfs,
			 &nb_files, &nb_files_with_vfs, &total_size, NULL,
			 &nb_vfs);
      y1 = 5;
      y2 = 5;
    }

  mytable = gtk_table_new (y1 + 1, 2, FALSE);
  gtk_widget_show (mytable);

  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), mytable, TRUE,
		      TRUE, 0);

  mylabel = gtk_label_new (_("Total size (Mb)"));
  gtk_widget_show (mylabel);
  gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
  y1--;

  if (gn == racine->tree)
    {
      mylabel = gtk_label_new (_("Nb of descriptions"));
      gtk_widget_show (mylabel);
      gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
      gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
      y1--;
    }

  if (gn == racine->tree)
    {
      mylabel = gtk_label_new (_("Nb of categories"));
      gtk_widget_show (mylabel);
      gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
      gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
      y1--;
    }

  mylabel = gtk_label_new (_("Nb of files including those into VFS"));
  gtk_widget_show (mylabel);
  gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
  y1--;

  mylabel = gtk_label_new (_("Nb of directories including those into VFS"));
  gtk_widget_show (mylabel);
  gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
  y1--;

  mylabel = gtk_label_new (_("Nb of VFS"));
  gtk_widget_show (mylabel);
  gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
  y1--;

  mylabel = gtk_label_new (_("Nb of files"));
  gtk_widget_show (mylabel);
  gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
  y1--;

  mylabel = gtk_label_new (_("Nb of directories"));
  gtk_widget_show (mylabel);
  gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
  y1--;

  if (gn == racine->tree)
    {
      mylabel = gtk_label_new (_("Nb of disks"));
      gtk_widget_show (mylabel);
      gtk_table_attach (GTK_TABLE (mytable), mylabel, 0, 1, y1, y1 + 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
      gtk_label_set_justify (GTK_LABEL (mylabel), GTK_JUSTIFY_RIGHT);
    }

  myentry = gtk_entry_new ();
  gtk_widget_show (myentry);
  gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  snprintf (tmpstr, 19, "%d", total_size);
  gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
  y2--;

  if (gn == racine->tree)
    {
      myentry = gtk_entry_new ();
      gtk_widget_show (myentry);
      gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
      snprintf (tmpstr, 19, "%d", nb_desc);
      gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
      gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
      y2--;
    }

  if (gn == racine->tree)
    {
      myentry = gtk_entry_new ();
      gtk_widget_show (myentry);
      gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
      snprintf (tmpstr, 19, "%d", nb_cat);
      gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
      gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
      y2--;
    }

  myentry = gtk_entry_new ();
  gtk_widget_show (myentry);
  gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  snprintf (tmpstr, 19, "%d", nb_files_with_vfs);
  gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
  y2--;

  myentry = gtk_entry_new ();
  gtk_widget_show (myentry);
  gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  snprintf (tmpstr, 19, "%d", nb_dirs_with_vfs);
  gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
  y2--;

  myentry = gtk_entry_new ();
  gtk_widget_show (myentry);
  gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  snprintf (tmpstr, 19, "%d", nb_vfs);
  gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
  y2--;

  myentry = gtk_entry_new ();
  gtk_widget_show (myentry);
  gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  snprintf (tmpstr, 19, "%d", nb_files);
  gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
  y2--;

  myentry = gtk_entry_new ();
  gtk_widget_show (myentry);
  gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  snprintf (tmpstr, 19, "%d", nb_dirs);
  gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
  y2--;

  if (gn == racine->tree)
    {
      myentry = gtk_entry_new ();
      gtk_widget_show (myentry);
      gtk_table_attach (GTK_TABLE (mytable), myentry, 1, 2, y2, y2 + 1,
			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
      snprintf (tmpstr, 19, "%d", nb_disks);
      gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
      gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
    }
  gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
}
