/*  GTKtalog.
 *  Copyright (C) 1999-2000  Mathieu VILLEGAS
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <gnome.h>
#ifdef USE_GNOMEVFS
#include <libgnomevfs/gnome-vfs-mime-info.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#endif
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "setup.h"
#include "interface.h"
#include "io.h"
#include "buttonbar.h"

#include "config_common.h"

#include "xpm/on.xpm"
#include "xpm/off.xpm"

#include "xpm/setup__Disk_setup.xpm"
#include "xpm/setup__General_options.xpm"
#include "xpm/setup__LoadSave_options.xpm"
#include "xpm/setup__Scan_options.xpm"
#include "xpm/setup__Scan_mime_types.xpm"
#include "xpm/setup__Virtual_File_Systems.xpm"
#include "xpm/setup__Information_plugins.xpm"
#include "xpm/setup__MIME_types_to_skip_during_a_scan.xpm"
#include "xpm/setup__MIME_type_icons.xpm"

/* setup dialog */
static GtkWidget *include_list_box;
static GtkWidget *vfs_include_list_box;
static GtkWidget *ei_include_list_box;
static GtkWidget *ess_include_list_box;
static GtkWidget *gnome_mount_entry;
static GtkWidget *gnome_umount_entry;
static GtkWidget *mount_entry;
static GtkWidget *umount_entry;
static GtkWidget *mount_point_entry;
static GtkWidget *eject_entry;
static GtkWidget *checkbutton_usesudo;
static GtkWidget *checkbutton_useautomount;
static GtkWidget *retry_value;
static GtkWidget *checkbutton_dus;
static GtkWidget *checkbutton_warnMount;
static GtkWidget *checkbutton_eject;
static GtkWidget *checkbutton_read_cd_name;
static GtkWidget *checkbutton_validate_cd_name;
static GtkWidget *compression_level;
static GtkWidget *checkbutton_getinformation;
static GtkWidget *checkbutton_dbclick_on_file;
static GtkWidget *checkbutton_dateislocale;
static GtkWidget *checkbutton_dateisiso8601;
static GtkWidget *checkbutton_dateisusersettings;
static GtkWidget *entry_dateislocale;
static GtkWidget *size_format;
static GtkWidget *test_vfs_mode;
static GtkWidget *toolbar_style_mode;
static GtkWidget *new_search_type_mode;
static GtkWidget *checkbutton_displaydirsize;
static GtkWidget *recentfilesmax;
static GtkWidget *checkbutton_save_cat_mode;
static GtkWidget *checkbutton_save_ctreewin_wh;
static GtkWidget *checkbutton_save_mainwin_xywh;
static GtkWidget *checkbutton_scan_mime_type;
static GtkWidget *checkbutton_usedefaultinfoplugin;
static GtkWidget *checkbutton_autosave_mode;
static GtkWidget *checkbutton_strictalphabeticsort_mode;
static GtkWidget *checkbutton_keepbackup_mode;
static GtkWidget *checkbutton_loadatboot_mode;
static GtkWidget *checkbutton_displaysplash;
static GtkWidget *entry_filetoload;
static GtkWidget *gnome_entry_dwpath;
static GtkWidget *save_every;
static GtkWidget *gnome_default_information_plugin_entry;
static GtkWidget *default_information_plugin_entry;
static GtkWidget *default_information_plugin_args_entry;
static GdkPixmap *on_pix;
static GdkPixmap *off_pix;
static GtkStyle *style;
static GdkBitmap *state_mask;
static GtkWidget *window;
static gint selected_row;
static gint vfs_selected_row;
static gint ei_selected_row;
static gint ess_selected_row;

/* Add type dialog */
static GtkWidget *include_type_entry;
static GtkWidget *include_prog_entry;
static GtkWidget *include_arg_entry;
static GtkWidget *add_type_window;

/* Add vfs type dialog */
static GtkWidget *vfs_include_type_entry;
static GtkWidget *vfs_include_prog_entry;
static GtkWidget *vfs_include_arg_entry;
static GtkWidget *vfs_add_type_window;
static GtkWidget *vfs_separator;
static GtkWidget *vfs_field_date;
static GtkWidget *vfs_format_date;
static GtkWidget *vfs_format_date_entry;
static GtkWidget *vfs_field_size;
static GtkWidget *vfs_field_name;
static GtkWidget *vfs_button_last_date;
static GtkWidget *vfs_button_last_size;
static GtkWidget *vfs_button_last_name;
static GtkWidget *vfs_button_last_nothing;

static GtkWidget *ei_include_type_entry;
static GtkWidget *ei_add_type_window;
static PIX *ei_selecting_pix;
static GtkWidget *ei_pentry;
static GtkWidget *ei_button_add;
static GtkWidget *ei_button_edit;
static GtkWidget *ei_button_del;

static GtkWidget *ess_include_type_entry;
static GtkWidget *ess_add_type_window;
static GtkWidget *ess_button_add;
static GtkWidget *ess_button_del;

static GtkWidget *frame_mountpoint;
static GtkWidget *frame_general;
static GtkWidget *frame_loadsave;
static GtkWidget *frame_scan;
static GtkWidget *frame_scan_general;
static GtkWidget *frame_scan_VFS;
static GtkWidget *frame_scan_plugins;
static GtkWidget *frame_scan_mimenoscan;
static GtkWidget *frame_mime;
static GtkWidget *setup_vbox;
static GtkWidget *current_frame;

static GtkCTreeNode *setup_scan_mime;
static GtkCTreeNode *setup_scan_info;
static GtkCTreeNode *setup_scan_vfs;


MIME_EXTENSIONS *selected_extension;
MIME_VFS_EXTENSIONS *vfs_selected_extension;
gchar *ei_selected_extension;
gchar *ess_selected_extension;
GTree *local_extensions;
GTree *local_vfs_extensions;
GTree *local_ei_extensions;
GTree *local_ess_extensions;

gchar *date_format;
gchar *mount_prog;
gchar *umount_prog;

gchar *default_file_pix_string;
gchar *default_dir_pix_string;
gchar *default_disk_pix_string;
gchar *default_unreadable_pix_string;
gchar *default_vfs_pix_string;
gchar *default_opened_dir_pix_string;
gchar *default_opened_disk_pix_string;
gchar *default_opened_vfs_pix_string;
PIX *local_default_file_pix;
PIX *local_default_dir_pix;
PIX *local_default_disk_pix;
PIX *local_default_unreadable_pix;
PIX *local_default_vfs_pix;
PIX *local_default_opened_dir_pix;
PIX *local_default_opened_disk_pix;
PIX *local_default_opened_vfs_pix;

gboolean
is_mime_valid (gchar * mime)
{
/*FIXME: Is there a better way to do this ?
 */
  GList *tmp;
#ifdef USE_GNOMEVFS
  tmp = gnome_vfs_mime_get_key_list (mime);
#else
  tmp = gnome_mime_get_keys (mime);
#endif
  if (tmp)
    {
      g_list_free (tmp);
      return (TRUE);
    }
  else
    {
      return (FALSE);
    }
}

gboolean
is_mime_default (gchar * mime)
{
  if (strcmp (default_file_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_dir_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_disk_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_unreadable_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_vfs_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_opened_dir_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_opened_disk_pix_string, mime) == 0)
    return (TRUE);
  if (strcmp (default_opened_vfs_pix_string, mime) == 0)
    return (TRUE);
  return (FALSE);
}


/**********************************************************************/
/***** Extension pixmaps **********************************************/
/**********************************************************************/

PIX *
new_pix_from_filename (GtkWidget * w, GtkStyle * style,
		       gchar * gs, CONFIG * my_config)
{
  PIX *pix;
  gchar *tmpstr;
  gchar *error_load_pixmap =
    _("Could not load pixmap %s.\nMaybe not an xpm file?");
  gint n;

  if ((pix = g_tree_lookup (my_config->pix_paths, gs)))
    {
      return (pix);
    }

  if (g_file_test (gs, G_FILE_TEST_IS_REGULAR))
    {
      pix = (PIX *) g_malloc (sizeof (PIX));
      pix->path = g_string_new (gs);
      pix->mask = (GdkBitmap **) g_malloc (sizeof (GdkBitmap **));
      pix->pixmap =
	gdk_pixmap_create_from_xpm (GTK_WIDGET (w)->window, pix->mask,
				    &style->bg[GTK_STATE_NORMAL],
				    pix->path->str);
      if (pix->pixmap)
	{

	  g_tree_insert (my_config->pix_paths, pix->path->str, pix);
#if defined(__DEBUG__)
	  GML_malloc_forget (pix);
	  GML_malloc_forget (pix->path);
	  GML_malloc_forget (pix->mask);
#endif
	  return (pix);
	}
      else
	{
	  g_string_free (pix->path, TRUE);
	  g_free (pix->mask);
	  g_free (pix);
	}
    }
  n = strlen (error_load_pixmap) + strlen (gs);
  tmpstr = (gchar *) g_malloc (sizeof (gchar) * n);
  snprintf (tmpstr, n, error_load_pixmap, gs);
  gnome_dialog_run_and_close (GNOME_DIALOG
			      (gnome_error_dialog_parented
			       (tmpstr, GTK_WINDOW (w))));
  g_free (tmpstr);
  return (NULL);

}

gboolean
is_default_pix (gchar * extension)
{
  if (strcmp (extension, default_file_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_dir_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_disk_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_unreadable_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_vfs_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_opened_dir_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_opened_disk_pix_string) == 0)
    return (TRUE);
  if (strcmp (extension, default_opened_vfs_pix_string) == 0)
    return (TRUE);
  return (FALSE);
}

void
eip_split_default_pix (PIX ** pix, gchar * default_pix_string)
{
  *pix = (PIX *) g_tree_lookup (my_config->mime_icons, default_pix_string);
  g_tree_remove (my_config->mime_icons, default_pix_string);
}

gint
_pix_gc_set (gpointer key, gpointer value, gpointer data)
{
  PIX *pix = value;
  pix->used = GPOINTER_TO_INT (data);
  return (FALSE);
}

gint
_pix_gc_free (gpointer key, gpointer value, gpointer data)
{
  PIX *pix = value;
  GList *node_to_free = *((GList **) data);

  if (pix->used == FALSE)
    {
      node_to_free = g_list_append (node_to_free, key);
      g_free (pix->pixmap);
      g_free (pix->mask);

      g_string_free (pix->path, FALSE);
    }
  *((GList **) data) = node_to_free;

  return (FALSE);
}

void
pix_garbage_collection ()
{
  GList *node_to_free = NULL;
  GList *n;
  g_tree_traverse (my_config->pix_paths, _pix_gc_set, G_IN_ORDER,
		   GINT_TO_POINTER (FALSE));

  my_config->default_file_pix->used = TRUE;
  my_config->default_dir_pix->used = TRUE;
  my_config->default_disk_pix->used = TRUE;
  my_config->default_unreadable_pix->used = TRUE;
  my_config->default_vfs_pix->used = TRUE;
  my_config->default_opened_dir_pix->used = TRUE;
  my_config->default_opened_disk_pix->used = TRUE;
  my_config->default_opened_vfs_pix->used = TRUE;

  g_tree_traverse (my_config->mime_icons, _pix_gc_set, G_IN_ORDER,
		   GINT_TO_POINTER (TRUE));
  g_tree_traverse (my_config->pix_paths, _pix_gc_free, G_IN_ORDER,
		   &node_to_free);
  n = node_to_free;
  while (n)
    {
      g_tree_remove (my_config->pix_paths, n->data);
      g_free (n->data);
      n->data = NULL;
      n = g_list_next (n);
    }
  g_list_free (node_to_free);
}

gint
_ei_free_traverse (gpointer key, gpointer value, gpointer data)
{
  PIX *eip = value;
  gint row;

  row = gtk_clist_find_row_from_data (GTK_CLIST (ei_include_list_box), eip);
  gtk_clist_set_row_data (GTK_CLIST (ei_include_list_box), row, NULL);
  return (FALSE);
}

void
ei_free (GTree * ei)
{
  g_tree_traverse (ei, _ei_free_traverse, G_IN_ORDER, NULL);
  g_tree_destroy (ei);
}

gboolean
ei_extensions_traverse (gpointer key, gpointer value, gpointer data)
{
  PIX *ie = value;
  gint i;
  gchar *tmp[3] = { "", "", "" };

  tmp[1] = key;
  tmp[2] = ie->path->str;

  i = gtk_clist_append (GTK_CLIST (ei_include_list_box), (gchar **) tmp);
  gtk_clist_set_pixtext (GTK_CLIST (ei_include_list_box), i, 0,
			 "", 3, ie->pixmap, *(ie->mask));

  gtk_clist_set_row_data (GTK_CLIST (ei_include_list_box), i, key);
  return (FALSE);
}

gint
pix_tree_printf (gpointer key, gpointer value, gpointer data)
{
  gchar *iconpath = key;
  gchar *mime_list = value;
  FILE *f = data;

  fprintf (f, "AddMimeIcon %s %s\n", iconpath, mime_list);
  g_free (mime_list);
  return (FALSE);
}

gint
pix_make_addicon_tree (gpointer key, gpointer value, gpointer data)
{
  GString *g;
  gchar *mime_list;
  gchar *strtmp;
  GTree *addicons = data;

  g = ((PIX *) value)->path;
  strtmp = g_tree_lookup (addicons, g->str);
  if (strtmp)
    {
      mime_list = g_strconcat (" ", (gchar *) key, strtmp, NULL);
      g_free (strtmp);
    }
  else
    {
      mime_list = g_strconcat (" ", (gchar *) key, NULL);
    }
  g_tree_insert (addicons, g->str, mime_list);
  return (FALSE);
}

void
ei_add_type_ok_clicked (GtkWidget * w, gpointer data)
{
  PIX *e2;
  gchar *old_extension = data;

  char *temp_icon_path;
  char *temp_extension;
  PIX *pix;

  temp_extension =
    gtk_editable_get_chars (GTK_EDITABLE (ei_include_type_entry), 0, -1);

  if (!is_mime_default (temp_extension))
    {
      if (!is_mime_valid (temp_extension))
	{
	  ERROR_DIALOG (_
			("The specified MIME is not valid. Nothing will be done."),
			main_window);
	  gtk_widget_destroy (ei_add_type_window);
	  return;

	}
    }
  e2 = g_tree_lookup (local_ei_extensions, temp_extension);

  if (e2 && (!old_extension))
    {
      gnome_dialog_run_and_close (GNOME_DIALOG
				  (gnome_warning_dialog_parented
				   (_
				    ("This MIME type already exists. Nothing will be done."),
				    GTK_WINDOW (ei_add_type_window))));
    }
  else
    {
      temp_icon_path =
	gnome_pixmap_entry_get_filename (GNOME_PIXMAP_ENTRY (ei_pentry));
      pix =
	new_pix_from_filename (ei_add_type_window, style, temp_icon_path,
			       my_config);
      if (pix)
	{
	  if ((old_extension) || (is_default_pix (temp_extension)))
	    {
	      g_tree_remove (local_ei_extensions, ei_selected_extension);
	      g_free (ei_selected_extension);
	      gtk_clist_remove (GTK_CLIST (ei_include_list_box),
				ei_selected_row);

	    }


	  temp_extension = g_strdup (temp_extension);
	  g_tree_insert (local_ei_extensions, temp_extension, pix);
	  ei_extensions_traverse (temp_extension, pix, NULL);
	  gtk_clist_sort (GTK_CLIST (ei_include_list_box));
	  gtk_clist_columns_autosize (GTK_CLIST (ei_include_list_box));
	  ei_selected_extension = NULL;
	  ei_selected_row = -1;
	}
    }

  gtk_widget_destroy (ei_add_type_window);
}

void
ei_add_type (GtkWidget * w, gpointer data)
{
  GtkWidget *vbox, *hbox, *vbox2, *hbox2;
  GtkWidget *ok, *cancel;
  GtkWidget *frame;
  GtkWidget *label, *guess_button;
  gint ei_add_edit_mode = GPOINTER_TO_INT (data);

  ei_add_type_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_modal (GTK_WINDOW (ei_add_type_window), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (ei_add_type_window),
				GTK_WINDOW (window));

  gtk_window_set_position (GTK_WINDOW (ei_add_type_window),
			   GTK_WIN_POS_MOUSE);
//  gtk_widget_set_usize (GTK_WIDGET (ei_add_type_window), 450, 450);
  gtk_window_set_title (GTK_WINDOW (ei_add_type_window),
			_("MIME type icons"));
  gtk_window_set_wmclass (GTK_WINDOW (ei_add_type_window),
			  _("MIME type icons"), "GTKtalog");
  gtk_window_set_policy (GTK_WINDOW (ei_add_type_window), TRUE, TRUE, TRUE);
  gtk_container_border_width (GTK_CONTAINER (ei_add_type_window), 10);
  gtk_signal_connect (GTK_OBJECT (ei_add_type_window), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy),
		      GTK_OBJECT (ei_add_type_window));

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (ei_add_type_window), vbox);
  gtk_widget_show (vbox);
  /*** extension ************/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("MIME type"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  ei_include_type_entry = gtk_entry_new_with_max_length (80);
  if (ei_add_edit_mode == EDIT_MODE)
    {
      if (ei_selected_extension)
	{
	  gtk_entry_set_text (GTK_ENTRY (ei_include_type_entry),
			      ei_selected_extension);
	  gtk_widget_set_sensitive (GTK_WIDGET (ei_include_type_entry),
				    FALSE);
	}
    }
  gtk_entry_select_region (GTK_ENTRY (ei_include_type_entry), 0,
			   GTK_ENTRY (ei_include_type_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox2), ei_include_type_entry, FALSE, FALSE,
		      0);
  gtk_widget_show (ei_include_type_entry);

  if (!((ei_add_edit_mode == EDIT_MODE) && ei_selected_extension))
    {
      hbox2 = gtk_hbox_new (FALSE, 4);
      gtk_container_add (GTK_CONTAINER (vbox2), hbox2);
      gtk_widget_show (hbox2);

      label = gtk_label_new (_("For example `image/jpeg'"));
      gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, FALSE, 0);
      gtk_widget_show (label);

      guess_button = gtk_button_new_with_label (_("Guess from an extension"));
      gtk_box_pack_start (GTK_BOX (hbox2), guess_button, TRUE, FALSE, 0);

      gtk_signal_connect (GTK_OBJECT (guess_button), "clicked",
			  GTK_SIGNAL_FUNC (extension_to_mime_GUI),
			  ei_include_type_entry);
      gtk_widget_show (guess_button);
    }
  /*** icon ********/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("Icon"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);




  ei_pentry =
    gnome_pixmap_entry_new ("ei_history_id", _("Select a pixmap"), TRUE);
  ei_selecting_pix = NULL;

  if (ei_add_edit_mode == EDIT_MODE)
    {
      if (ei_selected_extension)
	{
	  ei_selecting_pix =
	    g_tree_lookup (local_ei_extensions, ei_selected_extension);

	  gtk_entry_set_text (GTK_ENTRY
			      (gnome_pixmap_entry_gtk_entry
			       (GNOME_PIXMAP_ENTRY (ei_pentry))),
			      ei_selecting_pix->path->str);
	}
    }
  gtk_widget_show (ei_pentry);
  gtk_container_add (GTK_CONTAINER (frame), ei_pentry);


/********************************/

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

#ifdef GNOME1
  ok = gnome_stock_button (GNOME_STOCK_PIXMAP_SAVE);
#else
  ok = gtk_button_new_from_stock (GNOME_STOCK_PIXMAP_SAVE);
#endif
  GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
  gtk_window_set_default (GTK_WINDOW (ei_add_type_window), ok);
  gtk_box_pack_start (GTK_BOX (hbox), ok, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
		      GTK_SIGNAL_FUNC (ei_add_type_ok_clicked),
		      (ei_add_edit_mode ==
		       EDIT_MODE) ? ei_selected_extension : NULL);
  gtk_widget_show (ok);

#ifdef GNOME1
  cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
#else
  cancel = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
#endif
  GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), cancel, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (ei_add_type_window));
  gtk_widget_show (cancel);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  gtk_widget_show (ei_add_type_window);

}

void
ei_include_selection_made (GtkWidget * clist, gint row, gint column,
			   GdkEventButton * event, gpointer data)
{
  gchar *clrow_data;

  ei_selected_row = row;
  clrow_data = gtk_clist_get_row_data (GTK_CLIST (ei_include_list_box), row);
  ei_selected_extension = clrow_data;
  if (is_default_pix (ei_selected_extension))
    {
      gtk_widget_set_sensitive (GTK_WIDGET (ei_button_edit), TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET (ei_button_del), FALSE);
    }
  else
    {
      gtk_widget_set_sensitive (GTK_WIDGET (ei_button_edit), TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET (ei_button_del), TRUE);
    }

}

static gint
_ei_clist_g_strcasecmp (GtkCList * clist, gconstpointer ptr1,
			gconstpointer ptr2)
{
  char *text1 = NULL;
  char *text2 = NULL;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  switch (row1->cell[clist->sort_column].type)
    {
    case GTK_CELL_TEXT:
      text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
      break;
    case GTK_CELL_PIXTEXT:
      text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
      break;
    default:
      break;
    }

  switch (row2->cell[clist->sort_column].type)
    {
    case GTK_CELL_TEXT:
      text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
      break;
    case GTK_CELL_PIXTEXT:
      text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
      break;
    default:
      break;
    }

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;
  if ((is_default_pix (text1)) && (!(is_default_pix (text2))))
    return (-1);
  if ((is_default_pix (text2)) && (!(is_default_pix (text1))))
    return (1);

  return g_strcasecmp (text1, text2);
}

void
ei_include_unselection_made (GtkWidget * clist, gint row, gint column,
			     GdkEventButton * event, gpointer data)
{
  ei_selected_extension = NULL;
  ei_selected_row = -1;
  gtk_widget_set_sensitive (GTK_WIDGET (ei_button_edit), FALSE);
  gtk_widget_set_sensitive (GTK_WIDGET (ei_button_del), FALSE);
}

void
ei_include_edit_button_clicked (GtkWidget * w, gpointer data)
{
  if (ei_selected_extension && (ei_selected_row >= 0))
    ei_add_type (w, GINT_TO_POINTER (EDIT_MODE));
}

void
ei_include_delete_button_clicked (GtkWidget * w, gpointer data)
{
  if (ei_selected_extension && (ei_selected_row >= 0))
    {
      g_tree_remove (local_ei_extensions, ei_selected_extension);
      g_free (ei_selected_extension);
      gtk_clist_remove (GTK_CLIST (ei_include_list_box), ei_selected_row);
      gtk_clist_sort (GTK_CLIST (ei_include_list_box));
      gtk_clist_columns_autosize (GTK_CLIST (ei_include_list_box));
      ei_selected_extension = NULL;
      ei_selected_row = -1;
    }
}

gint
_duplicate_ei_extensions_traverse (gpointer key, gpointer value,
				   gpointer data)
{
  PIX *ei = value;
  g_tree_insert (data, g_strdup (key), ei);

  return (FALSE);
}

GTree *
duplicate_ei_extensions ()
{
  GTree *r;

  r = g_tree_new ((GCompareFunc) strcmp);
  g_tree_traverse (my_config->mime_icons,
		   _duplicate_ei_extensions_traverse, G_IN_ORDER, r);

  return (r);
}

/**********************************************************************/
/***** Extension plugins **********************************************/
/**********************************************************************/

gint
_extensions_free_traverse (gpointer key, gpointer value, gpointer data)
{
  MIME_EXTENSIONS *v = value;
  gint row;

  row = gtk_clist_find_row_from_data (GTK_CLIST (include_list_box), v);
  gtk_clist_set_row_data (GTK_CLIST (include_list_box), row, NULL);
  g_string_free (v->mime, TRUE);
  g_string_free (v->prog, TRUE);
  g_string_free (v->arg, TRUE);
  g_free (v);
  return (FALSE);
}

void
extensions_free (GTree * ext)
{
  g_tree_traverse (ext, _extensions_free_traverse, G_IN_ORDER, NULL);
  g_tree_destroy (ext);
}

gboolean
extensions_traverse (gpointer key, gpointer value, gpointer data)
{
  MIME_EXTENSIONS *m = value;
  gint i;
  gchar *tmp[4] = { "", "", "", "" };

  tmp[1] = m->mime->str;
  tmp[2] = m->prog->str;
  tmp[3] = m->arg->str;
  i = gtk_clist_append (GTK_CLIST (include_list_box), (gchar **) tmp);
  gtk_clist_set_row_data (GTK_CLIST (include_list_box), i, m);
  if (m->state == FALSE)
    gtk_clist_set_pixmap (GTK_CLIST (include_list_box), i, 0,
			  off_pix, state_mask);
  else
    gtk_clist_set_pixmap (GTK_CLIST (include_list_box), i, 0,
			  on_pix, state_mask);
  gtk_clist_unselect_row (GTK_CLIST (include_list_box), i, 0);
  return (FALSE);
}

gint
extensions_tree_printf (gpointer key, gpointer value, gpointer data)
{
  FILE *f = data;
  MIME_EXTENSIONS *m = value;
  gint state = 1;
  if (m->state == FALSE)
    state = 0;

  fprintf (f, "AddMimeInformation '%s' '%s' '%s' %d\n",
	   m->mime->str, m->prog->str, m->arg->str, state);
  return (FALSE);
}

void
add_type_ok_clicked (GtkWidget * w, gpointer data)
{
  MIME_EXTENSIONS *m2;
  gint add_edit_mode = GPOINTER_TO_INT (data);

  char *temp_type;
  char *temp_prog;
  char *temp_arg;

  temp_type =
    gtk_editable_get_chars (GTK_EDITABLE (include_type_entry), 0, -1);

  if (!is_mime_valid (temp_type))
    {
      ERROR_DIALOG (_
		    ("The specified MIME is not valid. Nothing will be done."),
		    main_window);
      g_free (temp_type);
    }
  else
    {

      m2 = g_tree_lookup (local_extensions, temp_type);

      if (m2 && (m2 != selected_extension))
	{
	  gnome_dialog_run_and_close (GNOME_DIALOG
				      (gnome_warning_dialog_parented
				       (_
					("This MIME type already exists. Nothing will be done."),
					GTK_WINDOW (add_type_window))));
	  g_free (temp_type);
	}
      else
	{
	  if (add_edit_mode == EDIT_MODE)
	    {
	      g_string_free (selected_extension->mime, TRUE);
	      g_string_free (selected_extension->prog, TRUE);
	      g_string_free (selected_extension->arg, TRUE);
	      g_tree_remove (local_extensions, selected_extension);
	      g_free (selected_extension);
	      gtk_clist_remove (GTK_CLIST (include_list_box), selected_row);

	    }
	  selected_extension =
	    (MIME_EXTENSIONS *) g_malloc (sizeof (MIME_EXTENSIONS));

	  temp_type =
	    gtk_editable_get_chars (GTK_EDITABLE (include_type_entry), 0, -1);
	  temp_prog =
	    gtk_editable_get_chars (GTK_EDITABLE (include_prog_entry), 0, -1);
	  temp_arg =
	    gtk_editable_get_chars (GTK_EDITABLE (include_arg_entry), 0, -1);
	  selected_extension->mime = g_string_new (temp_type);
	  g_free (temp_type);
	  selected_extension->prog = g_string_new (temp_prog);
	  g_free (temp_prog);
	  selected_extension->arg = g_string_new (temp_arg);
	  g_free (temp_arg);
	  selected_extension->state = TRUE;
	  g_tree_insert (local_extensions,
			 g_strdup (selected_extension->mime->str),
			 selected_extension);

	  extensions_traverse (selected_extension->mime->str,
			       selected_extension, NULL);

	}
    }
  gtk_widget_destroy (add_type_window);
}


void
add_type (GtkWidget * w, gpointer data)
{
  GtkWidget *vbox, *hbox, *vbox2, *hbox2;
  GtkWidget *ok, *cancel;
  GtkWidget *frame;
  GtkWidget *label, *guess_button;
  GtkWidget *gnome_include_prog_entry;
  gint add_edit_mode = GPOINTER_TO_INT (data);

  if ((selected_row == -1) && (add_edit_mode == EDIT_MODE))
    return;

  add_type_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_modal (GTK_WINDOW (add_type_window), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (add_type_window),
				GTK_WINDOW (window));

  gtk_window_set_position (GTK_WINDOW (add_type_window), GTK_WIN_POS_MOUSE);
//  gtk_widget_set_usize (GTK_WIDGET (add_type_window), 300, 295);
  gtk_window_set_title (GTK_WINDOW (add_type_window), _("File type"));
  gtk_window_set_wmclass (GTK_WINDOW (add_type_window), _("File type"),
			  "GTKtalog");
  gtk_window_set_policy (GTK_WINDOW (add_type_window), TRUE, TRUE, TRUE);
  gtk_container_border_width (GTK_CONTAINER (add_type_window), 10);
  gtk_signal_connect (GTK_OBJECT (add_type_window), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy),
		      GTK_OBJECT (add_type_window));

  vbox = gtk_vbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (add_type_window), vbox);
  gtk_widget_show (vbox);
  /***************/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("File Type"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  include_type_entry = gtk_entry_new_with_max_length (80);

  if (selected_extension)
    gtk_entry_set_text (GTK_ENTRY (include_type_entry),
			selected_extension->mime->str);
  gtk_entry_select_region (GTK_ENTRY (include_type_entry), 0,
			   GTK_ENTRY (include_type_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox2), include_type_entry, FALSE, FALSE, 0);
  gtk_widget_show (include_type_entry);

  hbox2 = gtk_hbox_new (FALSE, 4);
  gtk_container_add (GTK_CONTAINER (vbox2), hbox2);
  gtk_widget_show (hbox2);

  label = gtk_label_new (_("For example `image/jpeg'"));
  gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_widget_show (label);

  guess_button = gtk_button_new_with_label (_("Guess from an extension"));
  gtk_box_pack_start (GTK_BOX (hbox2), guess_button, TRUE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (guess_button), "clicked",
		      GTK_SIGNAL_FUNC (extension_to_mime_GUI),
		      include_type_entry);
  gtk_widget_show (guess_button);

  /***********/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame),
		       _("Plugin to get information about files"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  gnome_include_prog_entry =
    gnome_file_entry_new (NULL, _("Choose the mount program"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_include_prog_entry),
			      TRUE);
  include_prog_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gnome_include_prog_entry));

  if (selected_extension)
    gtk_entry_set_text (GTK_ENTRY (include_prog_entry),
			selected_extension->prog->str);
  gtk_entry_select_region (GTK_ENTRY (include_prog_entry), 0,
			   GTK_ENTRY (include_prog_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox2), gnome_include_prog_entry, FALSE, FALSE,
		      0);
  gtk_widget_show (gnome_include_prog_entry);

  /***********/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("Plugin Option"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  include_arg_entry = gtk_entry_new_with_max_length (80);

  if (selected_extension)
    gtk_entry_set_text (GTK_ENTRY (include_arg_entry),
			selected_extension->arg->str);
  gtk_entry_select_region (GTK_ENTRY (include_arg_entry), 0,
			   GTK_ENTRY (include_arg_entry)->text_length);
  gtk_container_add (GTK_CONTAINER (frame), include_arg_entry);
  gtk_widget_show (include_arg_entry);

/********************************/


  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

#ifdef GNOME1
  ok = gnome_stock_button (GNOME_STOCK_PIXMAP_SAVE);
#else
  ok = gtk_button_new_from_stock (GNOME_STOCK_PIXMAP_SAVE);
#endif
  GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
  gtk_window_set_default (GTK_WINDOW (add_type_window), ok);
  gtk_box_pack_start (GTK_BOX (hbox), ok, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
		      GTK_SIGNAL_FUNC (add_type_ok_clicked), data);
  gtk_widget_show (ok);

#ifdef GNOME1
  cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
#else
  cancel = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
#endif
  GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), cancel, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (add_type_window));
  gtk_widget_show (cancel);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  gtk_widget_show (add_type_window);

}

void
include_selection_made (GtkWidget * clist, gint row, gint column,
			GdkEventButton * event, gpointer data)
{
  MIME_EXTENSIONS *clrow_data;

  selected_row = row;
  clrow_data = gtk_clist_get_row_data (GTK_CLIST (include_list_box), row);
  selected_extension = clrow_data;

  if (event != NULL && event->button == 1 && event->type == GDK_2BUTTON_PRESS)
    {
      if (clrow_data->state == FALSE)
	{
	  clrow_data->state = TRUE;
	  gtk_clist_set_pixmap (GTK_CLIST (include_list_box), row, 0,
				on_pix, state_mask);
	}
      else
	{
	  clrow_data->state = FALSE;
	  gtk_clist_set_pixmap (GTK_CLIST (include_list_box), row, 0,
				off_pix, state_mask);
	}
    }
}

void
include_unselection_made (GtkWidget * clist, gint row, gint column,
			  GdkEventButton * event, gpointer data)
{
  selected_extension = NULL;
  selected_row = -1;
}

void
include_edit_button_clicked (GtkWidget * w, gpointer data)
{
  if (selected_extension && (selected_row >= 0))
    add_type (w, GINT_TO_POINTER (EDIT_MODE));
}


void
include_delete_button_clicked (GtkWidget * w, gpointer data)
{
  if (selected_row >= 0)
    {
      g_tree_remove (local_extensions, selected_extension->mime->str);
      g_string_free (selected_extension->mime, TRUE);
      g_string_free (selected_extension->prog, TRUE);
      g_string_free (selected_extension->arg, TRUE);
/* FIXME: Memory leak here. The key is not freed. */
      g_free (selected_extension);
      selected_extension = NULL;
      gtk_clist_remove (GTK_CLIST (include_list_box), selected_row);
      selected_row = -1;
      gtk_clist_sort (GTK_CLIST (ei_include_list_box));
      gtk_clist_columns_autosize (GTK_CLIST (ei_include_list_box));
    }
}

gint
_duplicate_extensions_traverse (gpointer key, gpointer value, gpointer data)
{
  MIME_EXTENSIONS *dup_value;
  MIME_EXTENSIONS *v = value;

  dup_value = (MIME_EXTENSIONS *) g_malloc (sizeof (MIME_EXTENSIONS));
  dup_value->mime = g_string_new (v->mime->str);
  dup_value->prog = g_string_new (v->prog->str);
  dup_value->arg = g_string_new (v->arg->str);
  dup_value->state = v->state;

  g_tree_insert (data, g_strdup (dup_value->mime->str), dup_value);

  return (FALSE);
}

GTree *
duplicate_extensions (GTree * ext)
{
  GTree *r;

  r = g_tree_new ((GCompareFunc) strcmp);
  g_tree_traverse (my_config->mime_extensions, _duplicate_extensions_traverse,
		   G_IN_ORDER, r);

  return (r);
}

/**********************************************************************/
/***** VFS Extensions *************************************************/
/**********************************************************************/

gint
_duplicate_vfs_extensions_traverse (gpointer key, gpointer value,
				    gpointer data)
{
  MIME_VFS_EXTENSIONS *dup_value;
  MIME_VFS_EXTENSIONS *ve = value;

  dup_value = (MIME_VFS_EXTENSIONS *) g_malloc (sizeof (MIME_VFS_EXTENSIONS));
  dup_value->mime = g_string_new (ve->mime->str);
  dup_value->prog = g_string_new (ve->prog->str);
  dup_value->args = g_string_new (ve->args->str);
  dup_value->separator = g_string_new (ve->separator->str);
  dup_value->date = ve->date;
  dup_value->date_format = g_string_new (ve->date_format->str);
  dup_value->size = ve->size;
  dup_value->name = ve->name;
  dup_value->state = ve->state;

  g_tree_insert (data, g_strdup (dup_value->mime->str), dup_value);

  return (FALSE);
}

GTree *
duplicate_vfs_extensions (GTree * vfsext)
{
  GTree *r;
  r = g_tree_new ((GCompareFunc) strcmp);
  g_tree_traverse (my_config->mime_vfs_extensions,
		   _duplicate_vfs_extensions_traverse, G_IN_ORDER, r);

  return (r);
}

gint
_vfs_extensions_free_traverse (gpointer key, gpointer value, gpointer data)
{
  MIME_VFS_EXTENSIONS *v = value;
  gint row;

  row = gtk_clist_find_row_from_data (GTK_CLIST (vfs_include_list_box), v);
  gtk_clist_set_row_data (GTK_CLIST (vfs_include_list_box), row, NULL);
  g_string_free (v->mime, TRUE);
  g_string_free (v->prog, TRUE);
  g_string_free (v->args, TRUE);
  g_string_free (v->separator, TRUE);
  g_string_free (v->date_format, TRUE);
  g_free (v);
  return (FALSE);
}

void
vfs_extensions_free (GTree * vfsext)
{
  g_tree_traverse (vfsext, _vfs_extensions_free_traverse, G_IN_ORDER, NULL);
  g_tree_destroy (vfsext);
}

gboolean
vfs_extensions_traverse (gpointer key, gpointer value, gpointer data)
{
  MIME_VFS_EXTENSIONS *ve = value;
  gint i;
  gchar *tmp[4] = { "", "", "", "" };

  tmp[1] = ve->mime->str;
  tmp[2] = ve->prog->str;
  tmp[3] = ve->args->str;
  i = gtk_clist_append (GTK_CLIST (vfs_include_list_box), (gchar **) tmp);
  gtk_clist_set_row_data (GTK_CLIST (vfs_include_list_box), i, ve);
  if (ve->state == FALSE)
    gtk_clist_set_pixmap (GTK_CLIST (vfs_include_list_box), i, 0,
			  off_pix, state_mask);
  else
    gtk_clist_set_pixmap (GTK_CLIST (vfs_include_list_box), i, 0,
			  on_pix, state_mask);
  gtk_clist_unselect_row (GTK_CLIST (vfs_include_list_box), i, 0);
  return (FALSE);
}

gint
vfs_extensions_tree_printf (gpointer key, gpointer value, gpointer data)
{
  FILE *f = data;
  MIME_VFS_EXTENSIONS *vm = value;
  gint state = 1;
  gchar *lc[] = { "Size", "Date", "Name", "Nothing" };
  gint lcn;

  if (vm->state == FALSE)
    state = 0;

  switch (vm->last_column)
    {
    case VFS_LAST_COLUMN_SIZE:
      lcn = 0;
      break;
    case VFS_LAST_COLUMN_DATE:
      lcn = 1;
      break;
    case VFS_LAST_COLUMN_NAME:
      lcn = 2;
      break;
    default:
      lcn = 3;
    }
  fprintf (f, "AddVfsMime '%s' '%s' '%s' '%s' %d '%s' %d %d %d %s\n",
	   vm->mime->str,
	   vm->prog->str, vm->args->str, vm->separator->str, vm->date,
	   vm->date_format->str, vm->size, vm->name, state, lc[lcn]);
  return (FALSE);
}

void
vfs_add_type_ok_clicked (GtkWidget * w, gpointer data)
{
  MIME_VFS_EXTENSIONS *e2;
  gint vfs_add_edit_mode = GPOINTER_TO_INT (data);

  char *temp_type;
  char *temp_prog;
  char *temp_args;
  char *temp_separator;
  char *temp_date_format;
  gint size, date, name;
  LAST_COLUMN last_column;

  temp_type =
    gtk_editable_get_chars (GTK_EDITABLE (vfs_include_type_entry), 0, -1);

  if (!is_mime_valid (temp_type))
    {
      ERROR_DIALOG (_
		    ("The specified MIME is not valid. Nothing will be done."),
		    main_window);
      g_free (temp_type);
    }
  else
    {


      e2 = g_tree_lookup (local_vfs_extensions, temp_type);

      if (e2 && (e2 != vfs_selected_extension))
	{
	  gnome_dialog_run_and_close (GNOME_DIALOG
				      (gnome_warning_dialog_parented
				       (_
					("This MIME type already exists. Nothing will be done."),
					GTK_WINDOW (vfs_add_type_window))));
	  g_free (temp_type);
	}
      else
	{
	  if (vfs_add_edit_mode == EDIT_MODE)
	    {
	      g_string_free (vfs_selected_extension->mime, TRUE);
	      g_string_free (vfs_selected_extension->prog, TRUE);
	      g_string_free (vfs_selected_extension->args, TRUE);
	      g_string_free (vfs_selected_extension->separator, TRUE);
	      g_string_free (vfs_selected_extension->date_format, TRUE);
	      g_tree_remove (local_vfs_extensions, vfs_selected_extension);
	      g_free (vfs_selected_extension);
	      gtk_clist_remove (GTK_CLIST (vfs_include_list_box),
				vfs_selected_row);

	    }
	  vfs_selected_extension =
	    (MIME_VFS_EXTENSIONS *) g_malloc (sizeof (MIME_VFS_EXTENSIONS));

	  temp_type =
	    gtk_editable_get_chars (GTK_EDITABLE (vfs_include_type_entry), 0,
				    -1);
	  temp_prog =
	    gtk_editable_get_chars (GTK_EDITABLE (vfs_include_prog_entry), 0,
				    -1);
	  temp_args =
	    gtk_editable_get_chars (GTK_EDITABLE (vfs_include_arg_entry), 0,
				    -1);
	  temp_separator =
	    gtk_editable_get_chars (GTK_EDITABLE (vfs_separator), 0, -1);
	  temp_date_format =
	    gtk_editable_get_chars (GTK_EDITABLE
				    ((GTK_COMBO (vfs_format_date))->entry), 0,
				    -1);
	  date =
	    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
					      (vfs_field_date));
	  size =
	    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
					      (vfs_field_size));
	  name =
	    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
					      (vfs_field_name));

	  if (gtk_toggle_button_get_active
	      (GTK_TOGGLE_BUTTON (vfs_button_last_size)))
	    {
	      last_column = VFS_LAST_COLUMN_SIZE;
	    }
	  else
	    if (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (vfs_button_last_date)))
	    {
	      last_column = VFS_LAST_COLUMN_DATE;
	    }
	  else
	    if (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (vfs_button_last_name)))
	    {
	      last_column = VFS_LAST_COLUMN_NAME;
	    }
	  else
	    {
	      last_column = VFS_LAST_COLUMN_NOTHING;
	    }

	  vfs_selected_extension->mime = g_string_new (temp_type);
	  g_free (temp_type);
	  vfs_selected_extension->prog = g_string_new (temp_prog);
	  g_free (temp_prog);
	  vfs_selected_extension->args = g_string_new (temp_args);
	  g_free (temp_args);
	  vfs_selected_extension->separator = g_string_new (temp_separator);
	  g_free (temp_separator);
	  vfs_selected_extension->date_format =
	    g_string_new (temp_date_format);
	  g_free (temp_date_format);
	  vfs_selected_extension->date = date;
	  vfs_selected_extension->size = size;
	  vfs_selected_extension->name = name;
	  vfs_selected_extension->last_column = last_column;
	  g_tree_insert (local_vfs_extensions,
			 g_strdup (vfs_selected_extension->mime->str),
			 vfs_selected_extension);
	  vfs_selected_extension->state = TRUE;
	  vfs_extensions_traverse (vfs_selected_extension->mime->str,
				   vfs_selected_extension, NULL);
	  gtk_clist_sort (GTK_CLIST (vfs_include_list_box));
	  gtk_clist_columns_autosize (GTK_CLIST (vfs_include_list_box));
	}
    }
  gtk_widget_destroy (vfs_add_type_window);
}


void
vfs_add_type (GtkWidget * w, gpointer data)
{
  GtkWidget *vbox, *hbox, *vbox2, *hbox2;
  GtkWidget *ok, *cancel;
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *guess_button;
  GtkWidget *table;
  GtkWidget *gnome_vfs_include_prog_entry;
  GtkObject *spinbutton_adj;
  GList *combo_items = NULL;
#ifndef GNOME1
GtkTextBuffer *stringbuffer;
#endif
  vfs_add_type_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_modal (GTK_WINDOW (vfs_add_type_window), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (vfs_add_type_window),
				GTK_WINDOW (window));

  gtk_window_set_position (GTK_WINDOW (vfs_add_type_window),
			   GTK_WIN_POS_MOUSE);
/*  gtk_widget_set_usize (GTK_WIDGET (vfs_add_type_window), 450, 450);*/
  gtk_window_set_title (GTK_WINDOW (vfs_add_type_window), _("File type"));
  gtk_window_set_wmclass (GTK_WINDOW (vfs_add_type_window), _("File type"),
			  "GTKtalog");
  gtk_window_set_policy (GTK_WINDOW (vfs_add_type_window), TRUE, TRUE, TRUE);
  gtk_container_border_width (GTK_CONTAINER (vfs_add_type_window), 10);
  gtk_signal_connect (GTK_OBJECT (vfs_add_type_window), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy),
		      GTK_OBJECT (vfs_add_type_window));

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (vfs_add_type_window), vbox);
  gtk_widget_show (vbox);
  /*** extension ************/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("File Type"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  vfs_include_type_entry = gtk_entry_new_with_max_length (80);

  if (vfs_selected_extension)
    gtk_entry_set_text (GTK_ENTRY (vfs_include_type_entry),
			vfs_selected_extension->mime->str);
  gtk_entry_select_region (GTK_ENTRY (vfs_include_type_entry), 0,
			   GTK_ENTRY (vfs_include_type_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox2), vfs_include_type_entry, FALSE, FALSE,
		      0);
  gtk_widget_show (vfs_include_type_entry);

  hbox2 = gtk_hbox_new (FALSE, 4);
  gtk_container_add (GTK_CONTAINER (vbox2), hbox2);
  gtk_widget_show (hbox2);

  label = gtk_label_new (_("For example `application/x-compressed-tar'"));
  gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_widget_show (label);

  guess_button = gtk_button_new_with_label (_("Guess from an extension"));
  gtk_box_pack_start (GTK_BOX (hbox2), guess_button, TRUE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (guess_button), "clicked",
		      GTK_SIGNAL_FUNC (extension_to_mime_GUI),
		      vfs_include_type_entry);
  gtk_widget_show (guess_button);


  /*** plugin ********/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame),
		       _
		       ("Plugin to read the files contained into the vfs file"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  gnome_vfs_include_prog_entry =
    gnome_file_entry_new (NULL, _("Choose the mount program"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_vfs_include_prog_entry),
			      TRUE);
  vfs_include_prog_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY
				(gnome_vfs_include_prog_entry));
  if (vfs_selected_extension)
    gtk_entry_set_text (GTK_ENTRY (vfs_include_prog_entry),
			vfs_selected_extension->prog->str);
  gtk_entry_select_region (GTK_ENTRY (vfs_include_prog_entry), 0,
			   GTK_ENTRY (vfs_include_prog_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox2), gnome_vfs_include_prog_entry, FALSE,
		      FALSE, 0);
  gtk_widget_show (gnome_vfs_include_prog_entry);

  /*** plugin options ********/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("Plugin Option"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vfs_include_arg_entry = gtk_entry_new_with_max_length (80);

  if (vfs_selected_extension)
    gtk_entry_set_text (GTK_ENTRY (vfs_include_arg_entry),
			vfs_selected_extension->args->str);
  gtk_entry_select_region (GTK_ENTRY (vfs_include_arg_entry), 0,
			   GTK_ENTRY (vfs_include_arg_entry)->text_length);
  gtk_container_add (GTK_CONTAINER (frame), vfs_include_arg_entry);
  gtk_widget_show (vfs_include_arg_entry);

  /*** options ********/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame),
		       _("Fields in plugin output (first one is 0)"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (TRUE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  table = gtk_table_new (5, 3, FALSE);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (vbox2), table);

  label = gtk_label_new (_("Separator characters"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  label = gtk_label_new (_("Date field"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  label = gtk_label_new (_("Date format"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  label = gtk_label_new (_("Size field"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  label = gtk_label_new (_("Name field"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  vfs_separator = gtk_entry_new ();
  gtk_widget_show (vfs_separator);
  gtk_table_attach (GTK_TABLE (table), vfs_separator, 1, 2, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  if (vfs_selected_extension)
    gtk_entry_set_text (GTK_ENTRY (vfs_separator),
			vfs_selected_extension->separator->str);

  if (vfs_selected_extension)
    spinbutton_adj =
      gtk_adjustment_new (vfs_selected_extension->date, 0, 100, 1, 10, 10);
  else
    spinbutton_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  vfs_field_date =
    gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_widget_show (vfs_field_date);
  gtk_table_attach (GTK_TABLE (table), vfs_field_date, 1, 2, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  vfs_format_date = gtk_combo_new ();
  gtk_widget_show (vfs_format_date);
  gtk_table_attach (GTK_TABLE (table), vfs_format_date, 1, 2, 2, 3,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  if (vfs_selected_extension)
    combo_items =
      g_list_append (combo_items, vfs_selected_extension->date_format->str);
  combo_items = g_list_append (combo_items, "YYYY-MM-DD");
  combo_items = g_list_append (combo_items, "DD-MM-YYYY");
  combo_items = g_list_append (combo_items, "SSSSSSSSSS");
  gtk_combo_set_popdown_strings (GTK_COMBO (vfs_format_date), combo_items);
  g_list_free (combo_items);

  vfs_format_date_entry = GTK_COMBO (vfs_format_date)->entry;
  gtk_widget_show (vfs_format_date_entry);

  if (vfs_selected_extension)
    spinbutton_adj =
      gtk_adjustment_new (vfs_selected_extension->size, 0, 100, 1, 10, 10);
  else
    spinbutton_adj = gtk_adjustment_new (2, 0, 100, 1, 10, 10);
  vfs_field_size =
    gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_widget_show (vfs_field_size);
  gtk_table_attach (GTK_TABLE (table), vfs_field_size, 1, 2, 3, 4,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  if (vfs_selected_extension)
    spinbutton_adj =
      gtk_adjustment_new (vfs_selected_extension->name, 0, 100, 1, 10, 10);
  else
    spinbutton_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  vfs_field_name =
    gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_widget_show (vfs_field_name);
  gtk_table_attach (GTK_TABLE (table), vfs_field_name, 1, 2, 4, 5,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);


  vfs_button_last_nothing =
    gtk_radio_button_new_with_label (NULL, _("No last col."));
  gtk_widget_show (vfs_button_last_nothing);
  gtk_table_attach (GTK_TABLE (table), vfs_button_last_nothing, 2, 3, 0, 1,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  vfs_button_last_date =
    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
						 (vfs_button_last_nothing),
						 _("Last col."));
  gtk_widget_show (vfs_button_last_date);
  gtk_table_attach (GTK_TABLE (table), vfs_button_last_date, 2, 3, 1, 2,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  vfs_button_last_size =
    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
						 (vfs_button_last_nothing),
						 _("Last col."));
  gtk_widget_show (vfs_button_last_size);
  gtk_table_attach (GTK_TABLE (table), vfs_button_last_size, 2, 3, 3, 4,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  vfs_button_last_name =
    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
						 (vfs_button_last_nothing),
						 _("Last col."));
  gtk_widget_show (vfs_button_last_name);
  gtk_table_attach (GTK_TABLE (table), vfs_button_last_name, 2, 3, 4, 5,
		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  if(vfs_selected_extension) {
  switch (vfs_selected_extension->last_column)
    {
    case VFS_LAST_COLUMN_SIZE:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vfs_button_last_size),
				    TRUE);
      break;
    case VFS_LAST_COLUMN_DATE:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vfs_button_last_date),
				    TRUE);
      break;
    case VFS_LAST_COLUMN_NAME:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vfs_button_last_name),
				    TRUE);
      break;
    default:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				    (vfs_button_last_nothing), TRUE);
    }
  }


#ifdef GNOME1
  label = gtk_text_new (NULL, NULL);
  gtk_text_insert (GTK_TEXT (label), NULL, NULL, NULL,
		   _("tar tvzf gtktalog.tar.gz\n"
		     "drwxrwxrwx 501/502           0 2000-08-22 18:33:51 gtktalog/\n"
		     "-rw-r--r-- 501/502       14362 2000-08-22 18:33:49 gtktalog/Makefile.in\n"
		     "drwxr-xr-x 501/502           0 2000-08-22 18:33:49 gtktalog/Docs/\n"
		     "...\n"
		     "Separator characters = ` \\t\'\n"
		     "Date field: 3\n"
		     "Date format: `YYYY-MM-DD'\n"
		     "Size field: 2\n" "Name field: 5\n"), -1);
  gtk_text_set_editable (GTK_TEXT (label), FALSE);
#else
  label = gtk_text_view_new ();
  stringbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(label));
  gtk_text_buffer_set_text (stringbuffer,
		   _("tar tvzf gtktalog.tar.gz\n"
		     "drwxrwxrwx 501/502           0 2000-08-22 18:33:51 gtktalog/\n"
		     "-rw-r--r-- 501/502       14362 2000-08-22 18:33:49 gtktalog/Makefile.in\n"
		     "drwxr-xr-x 501/502           0 2000-08-22 18:33:49 gtktalog/Docs/\n"
		     "...\n"
		     "Separator characters = ` \\t\'\n"
		     "Date field: 3\n"
		     "Date format: `YYYY-MM-DD'\n"
		     "Size field: 2\n" "Name field: 5\n"), -1);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (label), FALSE);
#endif
  gtk_widget_show (label);
  gtk_container_add (GTK_CONTAINER (vbox2), label);



/********************************/

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

#ifdef GNOME1
  ok = gnome_stock_button (GNOME_STOCK_PIXMAP_SAVE);
#else
  ok = gtk_button_new_from_stock (GNOME_STOCK_PIXMAP_SAVE);
#endif
  GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
  gtk_window_set_default (GTK_WINDOW (vfs_add_type_window), ok);
  gtk_box_pack_start (GTK_BOX (hbox), ok, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
		      GTK_SIGNAL_FUNC (vfs_add_type_ok_clicked), data);
  gtk_widget_show (ok);

#ifdef GNOME1
  cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
#else
  cancel = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
#endif
  GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), cancel, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (vfs_add_type_window));
  gtk_widget_show (cancel);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  gtk_widget_show (vfs_add_type_window);

}

void
vfs_include_selection_made (GtkWidget * clist, gint row, gint column,
			    GdkEventButton * event, gpointer data)
{
  MIME_VFS_EXTENSIONS *clrow_data;

  vfs_selected_row = row;
  clrow_data = gtk_clist_get_row_data (GTK_CLIST (vfs_include_list_box), row);
  vfs_selected_extension = clrow_data;

  if (event != NULL && event->button == 1 && event->type == GDK_2BUTTON_PRESS)
    {
      if (clrow_data->state == FALSE)
	{
	  clrow_data->state = TRUE;
	  gtk_clist_set_pixmap (GTK_CLIST (vfs_include_list_box), row, 0,
				on_pix, state_mask);
	}
      else
	{
	  clrow_data->state = FALSE;
	  gtk_clist_set_pixmap (GTK_CLIST (vfs_include_list_box), row, 0,
				off_pix, state_mask);
	}
    }
}

void
vfs_include_unselection_made (GtkWidget * clist, gint row, gint column,
			      GdkEventButton * event, gpointer data)
{
  vfs_selected_extension = NULL;
  vfs_selected_row = -1;
}

void
vfs_include_edit_button_clicked (GtkWidget * w, gpointer data)
{
  if (vfs_selected_extension && (vfs_selected_row >= 0))
    vfs_add_type (w, GINT_TO_POINTER (EDIT_MODE));
}

void
vfs_include_delete_button_clicked (GtkWidget * w, gpointer data)
{
  if (vfs_selected_row >= 0)
    {
      g_tree_remove (local_vfs_extensions, vfs_selected_extension->mime->str);
      g_string_free (vfs_selected_extension->mime, TRUE);
      g_string_free (vfs_selected_extension->prog, TRUE);
      g_string_free (vfs_selected_extension->args, TRUE);
      g_string_free (vfs_selected_extension->separator, TRUE);
      g_string_free (vfs_selected_extension->date_format, TRUE);
/* FIXME: Memory leak here. The key is not freed. */
      g_free (vfs_selected_extension);
      vfs_selected_extension = NULL;
      gtk_clist_remove (GTK_CLIST (vfs_include_list_box), vfs_selected_row);
      vfs_selected_row = -1;
      gtk_clist_sort (GTK_CLIST (vfs_include_list_box));
      gtk_clist_columns_autosize (GTK_CLIST (vfs_include_list_box));
    }
}

/**********************************************************************/
/***** Mimes to skip during a scan ************************************/
/**********************************************************************/

gboolean
ess_extensions_traverse (gpointer key, gpointer value, gpointer data)
{
  gint state = GPOINTER_TO_INT (value);
  gint i;
  gchar *tmp[2] = { "", "" };

  tmp[1] = key;

  i = gtk_clist_append (GTK_CLIST (ess_include_list_box), (gchar **) tmp);
  if (state)
    gtk_clist_set_pixmap (GTK_CLIST (ess_include_list_box), i, 0,
			  on_pix, state_mask);
  else
    gtk_clist_set_pixmap (GTK_CLIST (ess_include_list_box), i, 0,
			  off_pix, state_mask);
  gtk_clist_unselect_row (GTK_CLIST (ess_include_list_box), i, 0);
  gtk_clist_set_row_data (GTK_CLIST (ess_include_list_box), i, key);
  return (FALSE);
}

gint
ess_tree_printf (gpointer key, gpointer value, gpointer data)
{
  gchar *mime = key;
  gint state = GPOINTER_TO_INT (value);
  FILE *f = data;

  fprintf (f, "AddMimeToSkip '%s' %d\n", mime, state);
  return (FALSE);
}

gint
_ess_free_traverse (gpointer key, gpointer value, gpointer data)
{
  PIX *essp = value;
  gint row;

  row = gtk_clist_find_row_from_data (GTK_CLIST (ess_include_list_box), essp);
  gtk_clist_set_row_data (GTK_CLIST (ess_include_list_box), row, NULL);
  return (FALSE);
}

void
ess_free (GTree * ess)
{
  g_tree_traverse (ess, _ess_free_traverse, G_IN_ORDER, NULL);
  g_tree_destroy (ess);
}

void
ess_add_type_ok_clicked (GtkWidget * w, gpointer data)
{
  PIX *e2;

  char *temp_extension;

  temp_extension =
    gtk_editable_get_chars (GTK_EDITABLE (ess_include_type_entry), 0, -1);

  if (!is_mime_valid (temp_extension))
    {
      ERROR_DIALOG (_
		    ("The specified MIME is not valid. Nothing will be done."),
		    main_window);
    }
  else
    {

      e2 = g_tree_lookup (local_ess_extensions, temp_extension);

      if (e2)
	{
	  gnome_dialog_run_and_close (GNOME_DIALOG
				      (gnome_warning_dialog_parented
				       (_
					("This MIME type already exists. Nothing will be done."),
					GTK_WINDOW (ess_add_type_window))));
	}
      else
	{
	  temp_extension = g_strdup (temp_extension);
	  g_tree_insert (local_ess_extensions, temp_extension,
			 GINT_TO_POINTER (1));
	  ess_extensions_traverse (temp_extension, GINT_TO_POINTER (1), NULL);
	  gtk_clist_sort (GTK_CLIST (ess_include_list_box));
	  gtk_clist_columns_autosize (GTK_CLIST (ess_include_list_box));
	  ess_selected_extension = NULL;
	  ess_selected_row = -1;
	}
    }
  gtk_widget_destroy (ess_add_type_window);
}


void
ess_add_type (GtkWidget * w, gpointer data)
{
  GtkWidget *vbox, *hbox, *vbox2, *hbox2;
  GtkWidget *ok, *cancel;
  GtkWidget *frame;
  GtkWidget *label, *guess_button;
  gint ess_add_edit_mode = GPOINTER_TO_INT (data);

  ess_add_type_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_modal (GTK_WINDOW (ess_add_type_window), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (ess_add_type_window),
				GTK_WINDOW (window));

  gtk_window_set_position (GTK_WINDOW (ess_add_type_window),
			   GTK_WIN_POS_MOUSE);
//  gtk_widget_set_usize (GTK_WIDGET (ess_add_type_window), 450, 450);
  gtk_window_set_title (GTK_WINDOW (ess_add_type_window),
			_("MIME types to skip during a scan"));
  gtk_window_set_wmclass (GTK_WINDOW (ess_add_type_window),
			  _("MIME types to skip during a scan"), "GTKtalog");
  gtk_window_set_policy (GTK_WINDOW (ess_add_type_window), TRUE, TRUE, TRUE);
  gtk_container_border_width (GTK_CONTAINER (ess_add_type_window), 10);
  gtk_signal_connect (GTK_OBJECT (ess_add_type_window), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy),
		      GTK_OBJECT (ess_add_type_window));

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (ess_add_type_window), vbox);
  gtk_widget_show (vbox);
  /*** extension ************/
  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
  gtk_frame_set_label (GTK_FRAME (frame), _("MIME type"));
  gtk_frame_set_label_align (GTK_FRAME (frame), 0.0, 0.0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show (frame);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  gtk_widget_show (vbox2);

  ess_include_type_entry = gtk_entry_new_with_max_length (80);
  if (ess_add_edit_mode == EDIT_MODE)
    {
      if (ess_selected_extension)
	{
	  gtk_entry_set_text (GTK_ENTRY (ess_include_type_entry),
			      ess_selected_extension);
	  if (is_default_pix (ess_selected_extension))
	    gtk_editable_set_editable (GTK_EDITABLE (ess_include_type_entry),
				       FALSE);
	}
    }
  gtk_entry_select_region (GTK_ENTRY (ess_include_type_entry), 0,
			   GTK_ENTRY (ess_include_type_entry)->text_length);
  gtk_box_pack_start (GTK_BOX (vbox2), ess_include_type_entry, FALSE, FALSE,
		      0);
  gtk_widget_show (ess_include_type_entry);

  hbox2 = gtk_hbox_new (FALSE, 4);
  gtk_container_add (GTK_CONTAINER (vbox2), hbox2);
  gtk_widget_show (hbox2);

  label = gtk_label_new (_("For example `image/jpeg'"));
  gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, FALSE, 0);
  gtk_widget_show (label);

  guess_button = gtk_button_new_with_label (_("Guess from an extension"));
  gtk_box_pack_start (GTK_BOX (hbox2), guess_button, TRUE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (guess_button), "clicked",
		      GTK_SIGNAL_FUNC (extension_to_mime_GUI),
		      ess_include_type_entry);
  gtk_widget_show (guess_button);


/********************************/

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

#ifdef GNOME1
  ok = gnome_stock_button (GNOME_STOCK_PIXMAP_SAVE);
#else
  ok = gtk_button_new_from_stock (GNOME_STOCK_PIXMAP_SAVE);
#endif
  GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
  gtk_window_set_default (GTK_WINDOW (ess_add_type_window), ok);
  gtk_box_pack_start (GTK_BOX (hbox), ok, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
		      GTK_SIGNAL_FUNC (ess_add_type_ok_clicked), data);
  gtk_widget_show (ok);

#ifdef GNOME1
  cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
#else
  cancel = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
#endif
  GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), cancel, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancel), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (ess_add_type_window));
  gtk_widget_show (cancel);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  gtk_widget_show (ess_add_type_window);

}

void
ess_include_selection_made (GtkWidget * clist, gint row, gint column,
			    GdkEventButton * event, gpointer data)
{
  gchar *clrow_data;
  gint state;
  gpointer *ptrtmp;

  ess_selected_row = row;
  clrow_data = gtk_clist_get_row_data (GTK_CLIST (ess_include_list_box), row);
  ess_selected_extension = clrow_data;
  gtk_widget_set_sensitive (GTK_WIDGET (ess_button_del), TRUE);

  if (event != NULL && event->button == 1 && event->type == GDK_2BUTTON_PRESS)
    {
      ptrtmp = g_tree_lookup (local_ess_extensions, clrow_data);
      state = GPOINTER_TO_INT (ptrtmp);
      if (state)
	{
	  g_tree_insert (local_ess_extensions, clrow_data,
			 GINT_TO_POINTER (0));
	  gtk_clist_set_pixmap (GTK_CLIST (ess_include_list_box), row, 0,
				off_pix, state_mask);
	}
      else
	{
	  g_tree_insert (local_ess_extensions, clrow_data,
			 GINT_TO_POINTER (1));
	  gtk_clist_set_pixmap (GTK_CLIST (ess_include_list_box), row, 0,
				on_pix, state_mask);
	}
    }
}

static gint
_ess_clist_g_strcasecmp (GtkCList * clist, gconstpointer ptr1,
			 gconstpointer ptr2)
{
  char *text1 = NULL;
  char *text2 = NULL;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  switch (row1->cell[clist->sort_column].type)
    {
    case GTK_CELL_TEXT:
      text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
      break;
    case GTK_CELL_PIXTEXT:
      text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
      break;
    default:
      break;
    }

  switch (row2->cell[clist->sort_column].type)
    {
    case GTK_CELL_TEXT:
      text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
      break;
    case GTK_CELL_PIXTEXT:
      text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
      break;
    default:
      break;
    }

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  return g_strcasecmp (text1, text2);
}

void
ess_include_unselection_made (GtkWidget * clist, gint row, gint column,
			      GdkEventButton * event, gpointer data)
{
  ess_selected_extension = NULL;
  ess_selected_row = -1;
  gtk_widget_set_sensitive (GTK_WIDGET (ess_button_del), FALSE);
}

void
ess_include_edit_button_clicked (GtkWidget * w, gpointer data)
{
  if (ess_selected_extension && (ess_selected_row >= 0))
    ess_add_type (w, GINT_TO_POINTER (EDIT_MODE));
}

void
ess_include_delete_button_clicked (GtkWidget * w, gpointer data)
{
  if (ess_selected_row >= 0)
    {
      g_tree_remove (local_ess_extensions, ess_selected_extension);
      g_free (ess_selected_extension);
      gtk_clist_remove (GTK_CLIST (ess_include_list_box), ess_selected_row);
      ess_selected_extension = NULL;
      gtk_clist_remove (GTK_CLIST (ess_include_list_box), ess_selected_row);
      ess_selected_row = -1;
      gtk_clist_sort (GTK_CLIST (ess_include_list_box));
      gtk_clist_columns_autosize (GTK_CLIST (ess_include_list_box));
    }
}

gint
_duplicate_ess_extensions_traverse (gpointer key, gpointer value,
				    gpointer data)
{
  g_tree_insert (data, g_strdup (key), value);

  return (FALSE);
}

GTree *
duplicate_ess_extensions ()
{
  GTree *r;

  r = g_tree_new ((GCompareFunc) strcmp);
  g_tree_traverse (my_config->mime_to_skip_during_a_scan,
		   _duplicate_ess_extensions_traverse, G_IN_ORDER, r);

  return (r);
}



/**********************************************************************/
/***** GUI ************************************************************/
/**********************************************************************/

void
setup_ok_clicked (GtkWidget * w, gpointer data)
{
  GtkWidget *testvfs_widget, *toolbarstyle_widget, *new_search_type_widget;
  GtkWidget *size_format_widget;
  gboolean dateisuserdefined;
  gboolean automount_is_used;
  gboolean sudo_is_used;
  g_string_free (my_config->mount, TRUE);
  g_string_free (my_config->umount, TRUE);
  g_string_free (my_config->eject_prog, TRUE);
  g_string_free (my_config->mount_point, TRUE);

  /* Get Config */
  sudo_is_used = (gboolean) (gtk_toggle_button_get_active
				  (GTK_TOGGLE_BUTTON
				   (checkbutton_usesudo)));
  automount_is_used = (gboolean) (gtk_toggle_button_get_active
				  (GTK_TOGGLE_BUTTON
				   (checkbutton_useautomount)));
  if (!automount_is_used)
    {
      g_free (mount_prog);
      g_free (umount_prog);
      mount_prog = g_strdup (gtk_entry_get_text (GTK_ENTRY (mount_entry)));
      umount_prog = g_strdup (gtk_entry_get_text (GTK_ENTRY (umount_entry)));
    }
  my_config->mount = g_string_new (mount_prog);
  g_free (mount_prog);
  my_config->umount = g_string_new (umount_prog);
  g_free (umount_prog);
  my_config->eject_prog =
    g_string_new (gtk_entry_get_text (GTK_ENTRY (eject_entry)));
  my_config->mount_point =
    g_string_new (gtk_entry_get_text (GTK_ENTRY (mount_point_entry)));
  /* Config Options */
  my_config->use_sudo = sudo_is_used;
  my_config->use_automount = automount_is_used;
  my_config->use_supermount = FALSE;
  if (my_config->use_automount)
    {
      my_config->use_supermount = check_supermount (my_config);
    }

  if ((my_config->use_supermount) || (!(my_config->use_automount)))
    {
      my_config->mount_device =
	get_mount_device (my_config->mount_point->str);
    }
  else
    {
      my_config->mount_device = NULL;
    }

  my_config->mount_retry =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (retry_value));
  my_config->use_du_s =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_dus)));
  my_config->eject_disk =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_eject)));
  my_config->read_cd_name =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_read_cd_name)));
  my_config->validate_cd_name =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_validate_cd_name)));
  my_config->warn_mount =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_warnMount)));
  my_config->compression_level =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (compression_level));
  my_config->minutes_before_autosave =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (save_every));
  if (my_config->file_to_load)
    g_free (my_config->file_to_load);
  my_config->file_to_load =
    g_strdup (gtk_entry_get_text (GTK_ENTRY (entry_filetoload)));
  my_config->load_file_at_boot =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_loadatboot_mode)));
  my_config->autosave =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_autosave_mode)));
  my_config->keep_bakup_file =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_keepbackup_mode)));
  my_config->getInformation =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_getinformation)));
  my_config->dbclick_opens_info =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_dbclick_on_file)));
  my_config->display_splash =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_displaysplash)));

  dateisuserdefined = (gboolean) (gtk_toggle_button_get_active
				  (GTK_TOGGLE_BUTTON
				   (checkbutton_dateisusersettings)));
  my_config->display_dir_size =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_displaydirsize)));

  my_config->strict_alphabetic_sort = !((gboolean)
					(gtk_toggle_button_get_active
					 (GTK_TOGGLE_BUTTON
					  (checkbutton_strictalphabeticsort_mode))));

  my_config->date_format =
    g_strdup (gtk_entry_get_text (GTK_ENTRY (entry_dateislocale)));
  if (dateisuserdefined)
    {
      my_config->date_format_type = DATE_FORMAT_IS_USERDEFINED;
    }
  else
    {
      if (gtk_toggle_button_get_active
	  (GTK_TOGGLE_BUTTON (checkbutton_dateislocale)))
	my_config->date_format_type = DATE_FORMAT_IS_LOCALE;
      else
	my_config->date_format_type = DATE_FORMAT_IS_ISO8601;
    }
  g_free (date_format);

  my_config->save_only_used_categories =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_save_cat_mode)));

  testvfs_widget =
    gtk_menu_get_active (GTK_MENU
			 (gtk_option_menu_get_menu
			  (GTK_OPTION_MENU (test_vfs_mode))));
  my_config->testvfs =
    GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (testvfs_widget)));
  my_config->nb_max_recent_files =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (recentfilesmax));

  extensions_free (my_config->mime_extensions);
  vfs_extensions_free (my_config->mime_vfs_extensions);
  ei_free (my_config->mime_icons);
  ess_free (my_config->mime_to_skip_during_a_scan);

  my_config->mime_extensions = local_extensions;
  my_config->mime_vfs_extensions = local_vfs_extensions;
  my_config->mime_icons = local_ei_extensions;
  my_config->mime_to_skip_during_a_scan = local_ess_extensions;

  eip_split_default_pix (&(my_config->default_file_pix),
			 default_file_pix_string);
  eip_split_default_pix (&(my_config->default_dir_pix),
			 default_dir_pix_string);
  eip_split_default_pix (&(my_config->default_disk_pix),
			 default_disk_pix_string);
  eip_split_default_pix (&(my_config->default_unreadable_pix),
			 default_unreadable_pix_string);
  eip_split_default_pix (&(my_config->default_vfs_pix),
			 default_vfs_pix_string);
  eip_split_default_pix (&(my_config->default_opened_dir_pix),
			 default_opened_dir_pix_string);
  eip_split_default_pix (&(my_config->default_opened_disk_pix),
			 default_opened_disk_pix_string);
  eip_split_default_pix (&(my_config->default_opened_vfs_pix),
			 default_opened_vfs_pix_string);
  pix_garbage_collection ();
  if (my_config->eject_prog)
    {
      if (my_config->eject_prog->len <= 0)
	{
	  my_config->eject_disk = FALSE;
	}
    }
  else
    {
      my_config->eject_disk = FALSE;
    }

  g_string_free (my_config->default_information_plugin->prog, TRUE);
  my_config->default_information_plugin->prog =
    g_string_new (gtk_entry_get_text
		  (GTK_ENTRY (default_information_plugin_entry)));
  g_string_free (my_config->default_information_plugin->arg, TRUE);
  my_config->default_information_plugin->arg =
    g_string_new (gtk_entry_get_text
		  (GTK_ENTRY (default_information_plugin_args_entry)));

  my_config->scan_mime_types =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_scan_mime_type)));

  my_config->use_default_information_plugin =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_usedefaultinfoplugin)));

  my_config->ctreewindow_save_wh =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_save_ctreewin_wh)));

  my_config->mainwindow_save_xywh =
    (gboolean) (gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON (checkbutton_save_mainwin_xywh)));

  toolbarstyle_widget =
    gtk_menu_get_active (GTK_MENU
			 (gtk_option_menu_get_menu
			  (GTK_OPTION_MENU (toolbar_style_mode))));
  my_config->toolbar_type =
    GPOINTER_TO_INT (gtk_object_get_user_data
		     (GTK_OBJECT (toolbarstyle_widget)));

  update_toolbar_style ();

  size_format_widget =
    gtk_menu_get_active (GTK_MENU
			 (gtk_option_menu_get_menu
			  (GTK_OPTION_MENU (size_format))));
  my_config->size_format =
    GPOINTER_TO_INT (gtk_object_get_user_data
		     (GTK_OBJECT (size_format_widget)));

  new_search_type_widget =
    gtk_menu_get_active (GTK_MENU
			 (gtk_option_menu_get_menu
			  (GTK_OPTION_MENU (new_search_type_mode))));
  my_config->new_search_type =
    GPOINTER_TO_INT (gtk_object_get_user_data
		     (GTK_OBJECT (new_search_type_widget)));

  if (my_config->default_working_path)
    g_free (my_config->default_working_path);
  /* Don't use gnome_file_entry_get_full_path! (breaks `.' as path) */
  my_config->default_working_path
    =
    g_strdup (gtk_entry_get_text
	      (GTK_ENTRY
	       (gnome_file_entry_gtk_entry
		(GNOME_FILE_ENTRY (gnome_entry_dwpath)))));

  save_myconfig ();
  g_free (default_file_pix_string);
  g_free (default_dir_pix_string);
  g_free (default_disk_pix_string);
  g_free (default_unreadable_pix_string);
  g_free (default_vfs_pix_string);
  g_free (default_opened_dir_pix_string);
  g_free (default_opened_disk_pix_string);
  g_free (default_opened_vfs_pix_string);
  gtk_widget_destroy (window);
  list_folder (NULL, FALSE);
  if (my_config->racine && my_config->racine->ctree)
    gtk_ctree_sort_recursive (GTK_CTREE (my_config->racine->ctree), NULL);
}

void
save_myconfig ()
{
  GString *conf_file;
  FILE *f;
  RECENT_FILE *rf;
  GList *gl;
  GTree *addicons;

  conf_file = g_string_new (g_get_home_dir ());
  if (conf_file->len < 1)
    {
      gnome_dialog_run_and_close (GNOME_DIALOG
				  (gnome_warning_dialog_parented
				   (_
				    ("Home directory not found. Setup will not be saved."),
				    GTK_WINDOW (main_window))));
    }
  else
    {
      conf_file = g_string_append (conf_file, USER_CONFIG_DIR);
      if (!g_file_test (conf_file->str, G_FILE_TEST_IS_REGULAR))
	{
	  mkdir (conf_file->str, 493);
	}
      conf_file = g_string_append (conf_file, "/" USER_CONFIG_FILE);

      if (!(f = fopen (conf_file->str, "w+")))
	{
	  ERROR_DIALOG (_("Can't save to config file"), main_window);
	}
      else
	{
	  /* TO TRANSLATORS: Those strings used to be translated.
	   * However, many bugs could happen here, depending on the translation.
	   * As this is a config file that nobody will try to read, I think that it does
	   * not matter if we do not translate it any more.
	   * Yves Mettier <ymettier@libertysurf.fr>
	   * 20001221
	   */
	  fprintf (f, "# Sample config file for gtktalog\n\n");
	  fprintf (f, "# AddMimeIcon icon_path mime [mime [mime [...]]]\n\n");
	  addicons = g_tree_new ((GCompareFunc) strcmp);
	  g_tree_traverse (my_config->mime_icons, pix_make_addicon_tree,
			   G_PRE_ORDER, addicons);
	  g_tree_traverse (addicons, pix_tree_printf, G_PRE_ORDER, f);
	  g_tree_destroy (addicons);
	  fprintf (f, "\n# AddMimeInformation mime plugin arguments state\n");
	  g_tree_traverse (my_config->mime_extensions, extensions_tree_printf,
			   G_PRE_ORDER, f);
	  fprintf (f,
		   "\n# AddVfsMime vfs_mime plugin arguments separator date_field date_format size_field name_field state last_column\n");
	  g_tree_traverse (my_config->mime_vfs_extensions,
			   vfs_extensions_tree_printf, G_PRE_ORDER, f);

	  fprintf (f, "\n# DefaultFileIcon icon_path\n");
	  fprintf (f, "DefaultFileIcon %s\n",
		   my_config->default_file_pix->path->str);
	  fprintf (f, "# DefaultDiskIcon icon_path\n");
	  fprintf (f, "DefaultDiskIcon %s\n",
		   my_config->default_disk_pix->path->str);
	  fprintf (f, "# DefaultDirIcon icon_path\n");
	  fprintf (f, "DefaultDirIcon %s\n",
		   my_config->default_dir_pix->path->str);
	  fprintf (f, "# DefaultUnreadableIcon icon_path\n");
	  fprintf (f, "DefaultUnreadableIcon %s\n",
		   my_config->default_unreadable_pix->path->str);
	  fprintf (f, "# DefaultVfsIcon icon_path\n");
	  fprintf (f, "DefaultVfsIcon %s\n",
		   my_config->default_vfs_pix->path->str);
	  fprintf (f, "# DefaultOpenedDiskIcon icon_path\n");
	  fprintf (f, "DefaultOpenedDiskIcon %s\n",
		   my_config->default_opened_disk_pix->path->str);
	  fprintf (f, "# DefaultOpenedDirIcon icon_path\n");
	  fprintf (f, "DefaultOpenedDirIcon %s\n",
		   my_config->default_opened_dir_pix->path->str);
	  fprintf (f, "# DefaultOpenedVfsIcon icon_path\n");
	  fprintf (f, "DefaultOpenedVfsIcon %s\n",
		   my_config->default_opened_vfs_pix->path->str);
	  fprintf (f, "# MountProgram path\n");
	  fprintf (f, "MountProgram '%s'\n", my_config->mount->str);
	  fprintf (f, "# UmountProgram path\n");
	  fprintf (f, "UmountProgram '%s'\n", my_config->umount->str);
	  fprintf (f, "# EjectProgram path\n");
	  fprintf (f, "EjectProgram '%s'\n", my_config->eject_prog->str);
	  fprintf (f, "# MountPoint path\n");
	  fprintf (f, "MountPoint '%s'\n", my_config->mount_point->str);

	  /* Options */
	  fprintf (f, "\n# Options \n");
	  fprintf (f, "# Only use True or False for the following\n");
	  fprintf (f, "# Use sudo?\n");
	  fprintf (f, "UseSudo %s\n",
		   (my_config->use_sudo ? "True" : "False"));
	  fprintf (f, "# Use automount?\n");
	  fprintf (f, "UseAutomount %s\n",
		   (my_config->use_automount ? "True" : "False"));
	  fprintf (f, "MountRetry %d\n", (my_config->mount_retry));
	  fprintf (f, "# Use du -s to get the size of the disk\n");
	  fprintf (f, "OptionDus %s\n",
		   (my_config->use_du_s ? "True" : "False"));
	  fprintf (f, "# Warn if a disk is mounted\n");
	  fprintf (f, "OptionWarnMount %s\n",
		   (my_config->warn_mount ? "True" : "False"));
	  fprintf (f, "# Eject the disk after a fast scan\n");
	  fprintf (f, "OptionEject %s\n",
		   (my_config->eject_disk ? "True" : "False"));
	  fprintf (f, "# Read the CD name from CD\n");
	  fprintf (f, "ReadCDName %s\n",
		   (my_config->read_cd_name ? "True" : "False"));
	  fprintf (f, "# Prompt to validate the CD name\n");
	  fprintf (f, "ValidateCDName %s\n",
		   (my_config->validate_cd_name ? "True" : "False"));
	  fprintf (f, "# Compression level for saving files\n");
	  fprintf (f, "CompressionLevel %d\n",
		   (my_config->compression_level));

	  fprintf (f, "# File to load\n");
	  if (my_config->file_to_load)
	    {
	      fprintf (f, "FileToLoad '%s'\n", (my_config->file_to_load));
	    }
	  else
	    {
	      fprintf (f, "FileToLoad ''\n");
	    }

	  fprintf (f, "# Load file at boot?\n");
	  fprintf (f, "LoadFileAtBoot %s\n",
		   (my_config->load_file_at_boot ? "True" : "False"));

	  fprintf (f, "# Autosave?\n");
	  fprintf (f, "Autosave %s\n",
		   (my_config->autosave ? "True" : "False"));

	  fprintf (f, "# Keep Backup file?\n");
	  fprintf (f, "KeepBackupFile %s\n",
		   (my_config->keep_bakup_file ? "True" : "False"));

	  fprintf (f, "# Save every X minutes\n");
	  fprintf (f, "MinutesBeforeAutosave %d\n",
		   my_config->minutes_before_autosave);

	  fprintf (f,
		   "# Previous search mode for the extended search filename\n");
	  fprintf (f, "PreviousSearchMode %d\n",
		   (my_config->previous_search_mode));

	  fprintf (f,
		   "# Scan VFS (Virtual File System) Normal/Extended/Ignored\n");
	  switch (my_config->testvfs)
	    {
	    case TEST_VFS_NO:
	      fprintf (f, "VFSMode Ignored\n");
	      break;
	    case TEST_VFS_YES:
	      fprintf (f, "VFSMode Normal\n");
	      break;
	    case TEST_VFS_EXTENDED:
	      fprintf (f, "VFSMode Extended\n");
	      break;
	    }
	  fprintf (f, "# Retrieve information about files\n");
	  fprintf (f, "ReadInformation %s\n",
		   (my_config->getInformation ? "True" : "False"));
	  fprintf (f, "# Get Mime type from files\n");
	  fprintf (f, "ScanMimeTypes %s\n",
		   (my_config->scan_mime_types ? "True" : "False"));

	  fprintf (f, "\n# AddMimeToSkip mime state\n");
	  g_tree_traverse (my_config->mime_to_skip_during_a_scan,
			   ess_tree_printf, G_PRE_ORDER, f);



	  fprintf (f, "\n# Use default information plugin\n");
	  fprintf (f, "UseDefaultMimePlugin %s\n",
		   (my_config->use_default_information_plugin ? "True" :
		    "False"));
	  fprintf (f, "\n");
	  fprintf (f, "# Default information plugin\n");
	  fprintf (f, "# DefaultMimeInformation 'prog' 'args'\n");
	  fprintf (f, "DefaultMimeInformation '%s' '%s'\n",
		   my_config->default_information_plugin->prog->str,
		   my_config->default_information_plugin->arg->str);
	  fprintf (f, "# Double click on a file to see information\n");
	  fprintf (f, "DbClickFileForInfo %s\n",
		   (my_config->dbclick_opens_info ? "True" : "False"));
	  fprintf (f, "# Strict alphabetic sort\n");
	  fprintf (f, "StrictAlphabeticSort %s\n",
		   (my_config->strict_alphabetic_sort ? "True" : "False"));
	  fprintf (f,
		   "# DateFormat Locale or ISO8601 or \"format (man strftime)\"\n");
	  if (my_config->date_format_type == DATE_FORMAT_IS_USERDEFINED)
	    {
	      fprintf (f, "DateFormat \"%s\"\n", my_config->date_format);
	    }
	  else if (my_config->date_format_type == DATE_FORMAT_IS_ISO8601)
	    {
	      fprintf (f, "DateFormat ISO8601\n");
	    }
	  else
	    {
	      fprintf (f, "DateFormat Locale\n");
	    }
	  fprintf (f, "# Display directories sizes?\n");
	  fprintf (f, "DisplayDirSize %s\n",
		   (my_config->display_dir_size ? "True" : "False"));


	  fprintf (f, "\n# Recent files\n");
	  fprintf (f, "# Nb max of recent files\n");
	  fprintf (f, "RecentFilesMax %d\n", my_config->nb_max_recent_files);
	  fprintf (f, "\n");
	  fprintf (f, "# Save only used categories\n");
	  fprintf (f, "SaveOnlyUsedCategories %s\n",
		   (my_config->save_only_used_categories ? "True" : "False"));
	  fprintf (f, "\n");
	  fprintf (f, "# Save main window position and size\n");
	  fprintf (f, "# MainWindowPosition X Y W H use\n");
	  fprintf (f, "MainWindowPosition %d %d %d %d %s\n",
		   my_config->mainwindow_x, my_config->mainwindow_y,
		   my_config->mainwindow_w, my_config->mainwindow_h,
		   (my_config->mainwindow_save_xywh ? "True" : "False"));
	  fprintf (f, "\n");
	  fprintf (f, "# Save tree pane size\n");
	  fprintf (f, "# TreeWindowSize W H use\n");
	  fprintf (f, "TreeWindowSize %d %d %s\n",
		   my_config->ctreewindow_w,
		   my_config->ctreewindow_h,
		   (my_config->ctreewindow_save_wh ? "True" : "False"));
	  fprintf (f, "\n");
	  gl = my_config->recent_files;
	  /* Search for existing same entries */
	  while (gl)
	    {
	      char *fullname;
	      int i, j;
	      rf = gl->data;
	      for (i = 0, j = 0; rf->fullname[i]; i++)
		{
		  if (rf->fullname[i] == '\'')
		    j++;
		  j++;
		}
	      fullname = g_malloc ((j + 1) * sizeof (char));
	      for (i = 0, j = 0; rf->fullname[i]; i++)
		{
		  if (rf->fullname[i] == '\'')
		    fullname[j++] = '\\';
		  fullname[j++] = rf->fullname[i];
		}
	      fullname[j] = '\0';
	      fprintf (f, "RecentFile '%s'\n", fullname);
	      g_free (fullname);
	      gl = g_list_next (gl);
	    }
	  fprintf (f, "\n# Toolbar style\n");
	  fprintf (f, "# ToolbarStyle [ICONS|TEXT|BOTH]\n");
	  if (my_config->toolbar_type == GTK_TOOLBAR_ICONS)
	    {
	      fprintf (f, "ToolbarStyle Icons\n");
	    }
	  else if (my_config->toolbar_type == GTK_TOOLBAR_TEXT)
	    {
	      fprintf (f, "ToolbarStyle Text\n");
	    }
	  else
	    {
	      fprintf (f, "ToolbarStyle Both\n");
	    }

	  fprintf (f, "\n# Size format\n");
	  fprintf (f,
		   "# SizeFormat [Bytes|KiloBytes|MegaBytes|GigaBytes|HumanReadable|Bytes_Human]\n");
	  switch (my_config->size_format)
	    {
	    case SIZE_FORMAT_B:
	      fprintf (f, "SizeFormat Bytes\n");
	      break;
	    case SIZE_FORMAT_KB:
	      fprintf (f, "SizeFormat KiloBytes\n");
	      break;
	    case SIZE_FORMAT_MB:
	      fprintf (f, "SizeFormat MegaBytes\n");
	      break;
	    case SIZE_FORMAT_GB:
	      fprintf (f, "SizeFormat GigaBytes\n");
	      break;
	    case SIZE_FORMAT_HUMAN:
	      fprintf (f, "SizeFormat HumanReadable\n");
	      break;
	    case SIZE_FORMAT_B_HUMAN:
	      fprintf (f, "SizeFormat Bytes_Human\n");
	      break;
	    }

	  fprintf (f, "\n# New search type\n");
	  fprintf (f, "# NewSearchType {0,1,2}\n");
	  fprintf (f,
		   "# where 0==simple, 1==extended, 2==last used(simple), 3==last used(extended)\n");
	  fprintf (f, "NewSearchType %d\n", my_config->new_search_type);

	  fprintf (f, "\n# Default working directory\n");
	  fprintf (f, "# DefaultWorkingDirectory path\n");
	  fprintf (f, "DefaultWorkingDirectory %s\n",
		   my_config->default_working_path);

	  fprintf (f, "\n");
	  fprintf (f, "# Display the splash screen?\n");
	  fprintf (f, "DisplaySplash %s\n",
		   (my_config->display_splash ? "True" : "False"));
	  fprintf (f, "\n");

	  fclose (f);
	}
      g_string_free (conf_file, TRUE);
    }
}

static void
cancel_clicked (GtkWidget * w, gpointer data)
{
  g_free (mount_prog);
  g_free (umount_prog);
  extensions_free (local_extensions);
  vfs_extensions_free (local_vfs_extensions);
  ei_free (local_ei_extensions);
  ess_free (local_ess_extensions);
  gtk_widget_destroy (window);
  pix_garbage_collection ();
  g_free (default_file_pix_string);
  g_free (default_dir_pix_string);
  g_free (default_disk_pix_string);
  g_free (default_unreadable_pix_string);
  g_free (default_vfs_pix_string);
  g_free (default_opened_dir_pix_string);
  g_free (default_opened_disk_pix_string);
  g_free (default_opened_vfs_pix_string);
  return;
}

static void
destroy_window (GtkWidget * w, GdkEventKey * event, gpointer data)
{
  cancel_clicked (w, data);
}

static void
switch_entry_date (GtkWidget * w, gpointer data)
{
  GtkWidget *w2 = data;
  gtk_widget_set_sensitive (GTK_WIDGET (w2),
			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							  (w)));
}

static void
switch_entry_file (GtkWidget * w, gpointer data)
{
  GtkWidget *w2 = data;
  gtk_widget_set_sensitive (GTK_WIDGET (w2),
			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							  (w)));
}

static void
switch_info_options (GtkWidget * w, gpointer data)
{
  gboolean b;

  b =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				  (checkbutton_getinformation));
  gtk_widget_set_sensitive (GTK_WIDGET (checkbutton_usedefaultinfoplugin), b);

  gtk_widget_set_sensitive (GTK_WIDGET (frame_scan_plugins), b);

  if (b)
    b =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				    (checkbutton_usedefaultinfoplugin));
  gtk_widget_set_sensitive (GTK_WIDGET
			    (gnome_default_information_plugin_entry), b);
  gtk_widget_set_sensitive (GTK_WIDGET
			    (default_information_plugin_args_entry), b);


}

static void
switch_defaultinfo_options (GtkWidget * w, gpointer data)
{
  gboolean b;

  b =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				  (checkbutton_usedefaultinfoplugin));
  gtk_widget_set_sensitive (GTK_WIDGET
			    (gnome_default_information_plugin_entry), b);
  gtk_widget_set_sensitive (GTK_WIDGET
			    (default_information_plugin_args_entry), b);
}

static void
switch_vfs_options (GtkWidget * w, gpointer data)
{
  GtkWidget *testvfs_widget =
    gtk_menu_get_active (GTK_MENU
			 (gtk_option_menu_get_menu
			  (GTK_OPTION_MENU (test_vfs_mode))));
  gint option =
    GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (testvfs_widget)));

  gboolean b;

  b = (option != TEST_VFS_NO);
  gtk_widget_set_sensitive (GTK_WIDGET (frame_scan_VFS), b);
}

static void
switch_mime_options (GtkWidget * w, gpointer data)
{
  gboolean b;

  b =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				  (checkbutton_scan_mime_type));
  gtk_widget_set_sensitive (GTK_WIDGET (frame_scan_mimenoscan), b);
}

static void
automount_clicked (GtkWidget * w, gpointer data)
{
  gboolean ua;
  ua = (gboolean) (gtk_toggle_button_get_active
		   (GTK_TOGGLE_BUTTON (checkbutton_useautomount)));
  gtk_widget_set_sensitive (GTK_WIDGET (gnome_mount_entry), !ua);
  gtk_widget_set_sensitive (GTK_WIDGET (gnome_umount_entry), !ua);
  gtk_widget_set_sensitive (GTK_WIDGET (retry_value), !ua);
}

static gint
_clist_g_strcasecmp (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
{
  char *text1 = NULL;
  char *text2 = NULL;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  switch (row1->cell[clist->sort_column].type)
    {
    case GTK_CELL_TEXT:
      text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
      break;
    case GTK_CELL_PIXTEXT:
      text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
      break;
    default:
      break;
    }

  switch (row2->cell[clist->sort_column].type)
    {
    case GTK_CELL_TEXT:
      text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
      break;
    case GTK_CELL_PIXTEXT:
      text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
      break;
    default:
      break;
    }

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  return g_strcasecmp (text1, text2);
}

void
extension_to_mime_GUI (GtkWidget * w, gpointer data)
{
  GtkWidget *dialog, *label, *entry;
  GtkWidget *mime_entry = data;
  gchar *extension, *e;
  const gchar *mime;
  gint pos = 0;

  int button;
  dialog =
    gnome_dialog_new (_("Convert extension to MIME"),
		      GNOME_STOCK_BUTTON_APPLY, GNOME_STOCK_BUTTON_CANCEL,
		      NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (main_window));
  label = gtk_label_new (_("Enter an extension"));
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE,
		      TRUE, 0);
  gtk_widget_show (label);

  entry = gtk_entry_new ();
  gtk_editable_set_editable (GTK_EDITABLE (entry), TRUE);
  gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), entry, TRUE,
		      TRUE, 0);

  gtk_widget_show (entry);
  gnome_dialog_editable_enters (GNOME_DIALOG (dialog), GTK_EDITABLE (entry));
  button = gnome_dialog_run (GNOME_DIALOG (dialog));

  if (button == 0)
    {
      extension = g_strconcat (".", e =
			       gtk_editable_get_chars (GTK_EDITABLE (entry),
						       0, -1), NULL);
#ifdef USE_GNOMEVFS
      mime = gnome_vfs_mime_type_from_name_or_default (extension, NULL);
#else
      mime = gnome_mime_type_or_default (extension, NULL);
#endif
      g_free (e);
      g_free (extension);
      if (mime)
	{
	  gtk_editable_delete_text (GTK_EDITABLE (mime_entry), 0, -1);
	  gtk_editable_insert_text (GTK_EDITABLE (mime_entry), mime,
				    strlen (mime), &pos);
	}
    }
  gnome_dialog_close (GNOME_DIALOG (dialog));
}

void
frame_display (GtkWidget * wid, GdkEventButton * event, gpointer data)
{
  GtkWidget *frame;
  gint row, column;
  gint x, y;
  if ((event->button == 1))
    {
      x = event->x;
      y = event->y;
      if (gtk_clist_get_selection_info (GTK_CLIST (wid), x, y, &row, &column))
	{
	  frame = gtk_clist_get_row_data (GTK_CLIST (wid), row);
	  if (current_frame)
	    gtk_widget_hide (current_frame);
	  if (frame)
	    gtk_widget_show (GTK_WIDGET (frame));
	  current_frame = frame;
	}
    }
}

void
run_setup ()
{
  GtkWidget *setup_tree;
  GtkWidget *tree_hbox, *tree_vbox, *hpaned;
  GtkCTreeNode *setup_node, *setup_node2;
  GtkWidget *ok;
  GtkWidget *cancel;
  GtkWidget *align;
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *main_box;
  GtkWidget *bbox;
  GtkWidget *add_button;
  GtkWidget *delete_button;
  GtkWidget *edit_button;
  GtkWidget *retry_label;
  GtkWidget *scrolled_window;
  GtkWidget *compression_level_label;
  GtkAdjustment *compression_level_adj;
  GtkAdjustment *mount_retry_adj;
  GtkWidget *size_format_label;
  GtkWidget *size_format_menu;
  GtkWidget *size_format_menu_item;
  GtkWidget *toolbar_style_label;
  GtkWidget *toolbar_style_mode_menuitem;
  GtkWidget *toolbar_style_mode_menu;
  GtkWidget *new_search_type_label;
  GtkWidget *new_search_type_mode_menuitem;
  GtkWidget *new_search_type_mode_menu;
  GtkWidget *test_vfs_mode_label;
  GtkWidget *test_vfs_mode_menuitem;
  GtkWidget *test_vfs_mode_menu;
  GtkWidget *label_dateislocale;
  GtkWidget *recentfilesmax_label;
  GtkAdjustment *recentfilesmax_adj;
  GtkWidget *save_every_label;
  GtkAdjustment *save_every_adj;
  GtkWidget *table;
  GdkPixmap *pix;
  GdkBitmap *pix_mask;
  GtkWidget *box_in_frame;
  GtkWidget *gnome_entry_filetoload;
  GtkWidget *gnome_mount_point_entry;
  GtkWidget *gnome_eject_entry;
  gboolean load_file_at_boot;
  gchar *file_to_load;
  gchar *titles[4] = { _("State"),
    _("File type"),
    _("Plugin"),
    _("Options")
  };
  gchar *treetitles[2] = { NULL, NULL };
  gchar *eititles[3] = { _("Icon"), _("MIME type"), _("Path") };
  gchar *esstitles[2] = { _("State"), _("MIME type") };
  gboolean dateislocale;
  GtkAccelGroup *accel_group;

/**********************************************************************/
/* setup initializations **********************************************/
/**********************************************************************/

  default_file_pix_string = g_strconcat ("[", _("Default file"), "]", NULL);
  default_dir_pix_string = g_strconcat ("[", _("Default dir"), "]", NULL);
  default_disk_pix_string = g_strconcat ("[", _("Default disk"), "]", NULL);
  default_unreadable_pix_string =
    g_strconcat ("[", _("Default unreadable"), "]", NULL);
  default_vfs_pix_string = g_strconcat ("[", _("Default vfs"), "]", NULL);
  default_opened_dir_pix_string =
    g_strconcat ("[", _("Default current dir"), "]", NULL);
  default_opened_disk_pix_string =
    g_strconcat ("[", _("Default current disk"), "]", NULL);
  default_opened_vfs_pix_string =
    g_strconcat ("[", _("Default current vfs"), "]", NULL);

  local_extensions = duplicate_extensions (my_config->mime_extensions);
  local_vfs_extensions =
    duplicate_vfs_extensions (my_config->mime_vfs_extensions);
  local_ei_extensions = duplicate_ei_extensions ();
  local_ess_extensions = duplicate_ess_extensions ();

  local_default_file_pix = my_config->default_file_pix;
  g_tree_insert (local_ei_extensions, g_strdup (default_file_pix_string),
		 local_default_file_pix);
  local_default_dir_pix = my_config->default_dir_pix;
  g_tree_insert (local_ei_extensions, g_strdup (default_dir_pix_string),
		 local_default_dir_pix);
  local_default_disk_pix = my_config->default_disk_pix;
  g_tree_insert (local_ei_extensions, g_strdup (default_disk_pix_string),
		 local_default_disk_pix);
  local_default_unreadable_pix = my_config->default_unreadable_pix;
  g_tree_insert (local_ei_extensions,
		 g_strdup (default_unreadable_pix_string),
		 local_default_unreadable_pix);
  local_default_vfs_pix = my_config->default_vfs_pix;
  g_tree_insert (local_ei_extensions, g_strdup (default_vfs_pix_string),
		 local_default_vfs_pix);
  local_default_opened_dir_pix = my_config->default_opened_dir_pix;
  g_tree_insert (local_ei_extensions,
		 g_strdup (default_opened_dir_pix_string),
		 local_default_opened_dir_pix);
  local_default_opened_disk_pix = my_config->default_opened_disk_pix;
  g_tree_insert (local_ei_extensions,
		 g_strdup (default_opened_disk_pix_string),
		 local_default_opened_disk_pix);
  local_default_opened_vfs_pix = my_config->default_opened_vfs_pix;
  g_tree_insert (local_ei_extensions,
		 g_strdup (default_opened_vfs_pix_string),
		 local_default_opened_vfs_pix);

  mount_prog = g_strdup (my_config->mount->str);
  umount_prog = g_strdup (my_config->umount->str);

/**********************************************************************/
/* GUI initializations ************************************************/
/**********************************************************************/

  accel_group = gtk_accel_group_new ();

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (window),
				GTK_WINDOW (main_window));

  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
  gtk_window_set_title (GTK_WINDOW (window), _("Setup"));
  gtk_window_set_wmclass (GTK_WINDOW (window), _("Setup"), "GTKtalog");
  gtk_signal_connect (GTK_OBJECT (window),
		      "delete_event", G_CALLBACK(destroy_window), NULL);
  gtk_widget_realize (window);



/**********************************************************************/
/* pixmap initializations *********************************************/
/**********************************************************************/

  style = gtk_widget_get_style (window);
  on_pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(state_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) on_xpm);
  style = gtk_widget_get_style (window);
  off_pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(state_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) off_xpm);
  gtk_container_set_border_width (GTK_CONTAINER (window), 2);


/**********************************************************************/
/* main boxes and the tree ********************************************/
/**********************************************************************/

  tree_hbox = gtk_hbox_new (FALSE, 4);	/* main box */
  gtk_container_add (GTK_CONTAINER (window), tree_hbox);
  gtk_widget_show (tree_hbox);

  hpaned = gtk_hpaned_new ();
  gtk_container_add (GTK_CONTAINER (tree_hbox), hpaned);
#ifdef GNOME1
  gtk_paned_set_handle_size (GTK_PANED (hpaned), 10);
  gtk_paned_set_gutter_size (GTK_PANED (hpaned), 10);
#endif
  gtk_widget_show (hpaned);

  tree_vbox = gtk_vbox_new (FALSE, 4);	/* main right box */
  gtk_paned_add1 (GTK_PANED (hpaned), tree_vbox);
  gtk_widget_show (tree_vbox);

  setup_tree = gtk_ctree_new (1, 0);	/* tree */
  gtk_clist_columns_autosize (GTK_CLIST (&(GTK_CTREE (setup_tree)->clist)));

  gtk_box_pack_start (GTK_BOX (tree_vbox), setup_tree, TRUE, TRUE, 0);

  gtk_widget_show (setup_tree);
  gtk_signal_connect (GTK_OBJECT (setup_tree), "button_press_event",
		      GTK_SIGNAL_FUNC (frame_display), NULL);

  tree_vbox = gtk_vbox_new (FALSE, 4);	/* main right box */
  gtk_paned_add2 (GTK_PANED (hpaned), tree_vbox);
  gtk_widget_show (tree_vbox);

  setup_vbox = gtk_vbox_new (FALSE, 4);	/* right top box */
  gtk_box_pack_start (GTK_BOX (tree_vbox), setup_vbox, TRUE, TRUE, 0);
  gtk_widget_show (setup_vbox);


/**********************************************************************/
/* Frame for Mount options ********************************************/
/**********************************************************************/

  frame_mountpoint = gtk_frame_new (_("Mount options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_mountpoint), 3);
  gtk_widget_show (frame_mountpoint);

  treetitles[0] = g_strdup (_("Disk setup"));
  style = gtk_widget_get_style (window);
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__Disk_setup);
  setup_node =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), NULL, NULL, treetitles, 3,
			   pix, pix_mask, pix, pix_mask, FALSE, TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_node, frame_mountpoint);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_mountpoint), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (main_box), 6);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);

  /* begin frame */
  frame = gtk_frame_new (_("Mount method"));
  gtk_container_set_border_width (GTK_CONTAINER (frame), 3);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 0);

  table = gtk_table_new (4, 2, FALSE);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);

  /* Option to know if automount is used */
  checkbutton_useautomount =
    gtk_check_button_new_with_label (_("Use automount (or supermount)"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_useautomount),
				my_config->use_automount);
  gtk_signal_connect (GTK_OBJECT (checkbutton_useautomount), "clicked",
		      GTK_SIGNAL_FUNC (automount_clicked), NULL);
  gtk_widget_show (checkbutton_useautomount);
  gtk_table_attach (GTK_TABLE (table), checkbutton_useautomount, 0, 2, 0, 1,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);

  /* Option to know if sudo is used */
  checkbutton_usesudo =
    gtk_check_button_new_with_label (_("Use sudo"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_usesudo),
				my_config->use_sudo);
  gtk_widget_show (checkbutton_usesudo);
  gtk_table_attach (GTK_TABLE (table), checkbutton_usesudo, 0, 2, 1, 2,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);

  /* For slow/laptop cdroms */
  retry_label = gtk_label_new (_("Number of mount retries:"));
  gtk_misc_set_alignment (GTK_MISC (retry_label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), retry_label, 0, 1, 2, 3,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_show (retry_label);

  align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
  mount_retry_adj =
    (GtkAdjustment *) gtk_adjustment_new (my_config->mount_retry,
					  0, 5, 1, 1, 5);
  retry_value = gtk_spin_button_new (GTK_ADJUSTMENT (mount_retry_adj), 1, 0);
  gtk_container_add (GTK_CONTAINER (align), retry_value);
  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 2, 3,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_show (retry_value);
  gtk_widget_show (align);

  /* Mount program */
  label = gtk_label_new (_("Mount program:"));
  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_show (label);

  gnome_mount_entry =
    gnome_file_entry_new (NULL, _("Choose the mount program"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_mount_entry), TRUE);
  mount_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gnome_mount_entry));
  gtk_entry_set_text (GTK_ENTRY (mount_entry), my_config->mount->str);
  gtk_table_attach (GTK_TABLE (table), gnome_mount_entry, 1, 2, 3, 4,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_show (gnome_mount_entry);

  /* Umount program */
  label = gtk_label_new (_("Umount program:"));
  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_show (label);

  gnome_umount_entry =
    gnome_file_entry_new (NULL, _("Choose the umount program"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_umount_entry), TRUE);
  umount_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gnome_umount_entry));
  gtk_entry_set_text (GTK_ENTRY (umount_entry), my_config->umount->str);
  gtk_table_attach (GTK_TABLE (table), gnome_umount_entry, 1, 2, 4, 5,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_show (gnome_umount_entry);

#ifdef OS_PPC_DARWIN
  /* to mount, unmout and aject CD, just use the Finder in MacOS X */
  gtk_widget_set_sensitive (GTK_WIDGET (table), FALSE);
#endif

  /* Cdrom options */
  frame = gtk_frame_new (_("Cdrom options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 0);

  table = gtk_table_new (2, 2, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
#ifdef OS_PPC_DARWIN
  gtk_widget_set_sensitive (GTK_WIDGET (table), FALSE);
#endif
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);

  label = gtk_label_new (_("Mount point:"));
  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
#ifdef OS_PPC_DARWIN
  gtk_widget_set_sensitive (GTK_WIDGET (label), FALSE);
#endif
  gtk_widget_show (label);

  gnome_mount_point_entry =
    gnome_file_entry_new (NULL, _("Choose the CD mount point"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_mount_point_entry),
			      TRUE);
  mount_point_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gnome_mount_point_entry));
  gtk_entry_set_text (GTK_ENTRY (mount_point_entry),
		      my_config->mount_point->str);
  gtk_table_attach (GTK_TABLE (table), gnome_mount_point_entry, 1, 2, 0, 1,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
#ifdef OS_PPC_DARWIN
  gtk_widget_set_sensitive (GTK_WIDGET (gnome_mount_point_entry), FALSE);
#endif
  gtk_widget_show (gnome_mount_point_entry);

  /* Eject Program */
  label = gtk_label_new (_("Eject Program:"));
  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
#ifdef OS_PPC_DARWIN
  gtk_widget_set_sensitive (GTK_WIDGET (label), FALSE);
#endif
  gtk_widget_show (label);

  gnome_eject_entry =
    gnome_file_entry_new (NULL, _("Choose the eject program"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_eject_entry), TRUE);
  eject_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gnome_eject_entry));
  gtk_entry_set_text (GTK_ENTRY (eject_entry), my_config->eject_prog->str);
  gtk_table_attach (GTK_TABLE (table), gnome_eject_entry, 1, 2, 1, 2,
		    (GTK_FILL | GTK_SHRINK), (GtkAttachOptions) (0), 0, 0);
#ifdef OS_PPC_DARWIN
  gtk_widget_set_sensitive (GTK_WIDGET (gnome_eject_entry), FALSE);
#endif
  gtk_widget_show (gnome_eject_entry);

/**********************************************************************/
/* Frame for General Options ******************************************/
/**********************************************************************/

  frame_general = gtk_frame_new (_("General Options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_general), 6);
  gtk_widget_show (frame_general);

  treetitles[0] = g_strdup (_("General Options"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__General_options);
  setup_node =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), NULL, NULL, treetitles, 3,
			   pix, pix_mask, pix, pix_mask, FALSE, TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_node, frame_general);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_general), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (main_box), 6);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);


  /* Option to use du -s or statfs */
  checkbutton_dus =
    gtk_check_button_new_with_label (_("Prefer better but slower disk size evaluation?"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_dus),
				my_config->use_du_s);
  gtk_widget_show (checkbutton_dus);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_dus, FALSE, FALSE, 0);

  /* Option to warn if the cd is mounted */
  checkbutton_warnMount =
    gtk_check_button_new_with_label (_("Warn if the CD is mounted"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_warnMount),
				my_config->warn_mount);
  gtk_widget_show (checkbutton_warnMount);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_warnMount, FALSE, FALSE,
		      0);

  /* Option to eject the CD */
  checkbutton_eject =
    gtk_check_button_new_with_label (_("Eject the CD after fast scan"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_eject),
				my_config->eject_disk);
#ifdef OS_PPC_DARWIN
  /* to eject the CD, just use the Finder in MacOS X */
  gtk_widget_set_sensitive (GTK_WIDGET (checkbutton_eject), FALSE);
#endif
  gtk_widget_show (checkbutton_eject);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_eject, FALSE, FALSE, 0);

  /* Read the CD name on the CD */
  checkbutton_read_cd_name =
    gtk_check_button_new_with_label (_("Read the CD name on the CD"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_read_cd_name),
				my_config->read_cd_name);
  gtk_widget_show (checkbutton_read_cd_name);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_read_cd_name, FALSE,
		      FALSE, 0);

  /* Prompt to validate the CD name */
  checkbutton_validate_cd_name =
    gtk_check_button_new_with_label (_("Prompt to validate the CD name"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_validate_cd_name),
				my_config->validate_cd_name);
  gtk_widget_show (checkbutton_validate_cd_name);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_validate_cd_name, FALSE,
		      FALSE, 0);

  /* Option to know what to do when there is a db_click on a file */
  checkbutton_dbclick_on_file =
    gtk_check_button_new_with_label (_
				     ("Double click on a file to see information"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_dbclick_on_file),
				my_config->dbclick_opens_info);
  gtk_widget_show (checkbutton_dbclick_on_file);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_dbclick_on_file, FALSE,
		      FALSE, 0);

  /* Option to know whether to display files with strict alphabetic sort or not */
  checkbutton_strictalphabeticsort_mode =
    gtk_check_button_new_with_label (_("Smart files sorting"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_strictalphabeticsort_mode),
				!my_config->strict_alphabetic_sort);
  gtk_widget_show (checkbutton_strictalphabeticsort_mode);
  gtk_box_pack_start (GTK_BOX (main_box),
		      checkbutton_strictalphabeticsort_mode, FALSE, FALSE, 0);

  /* Option about the date format */
  checkbutton_dateislocale =
    gtk_radio_button_new_with_label (NULL, _
				     ("Date format gets from locale settings"));

  if (my_config->date_format_type != DATE_FORMAT_IS_LOCALE)
    dateislocale = FALSE;
  else
    dateislocale = TRUE;
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_dateislocale), dateislocale);
  gtk_widget_show (checkbutton_dateislocale);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_dateislocale, FALSE,
		      FALSE, 0);

  checkbutton_dateisiso8601 =
    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
						 (checkbutton_dateislocale),
						 _
						 ("Date format is ISO 8601"));
  if (my_config->date_format_type != DATE_FORMAT_IS_ISO8601)
    dateislocale = FALSE;
  else
    dateislocale = TRUE;
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_dateisiso8601), dateislocale);
  gtk_widget_show (checkbutton_dateisiso8601);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_dateisiso8601, FALSE,
		      FALSE, 0);

  checkbutton_dateisusersettings =
    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
						 (checkbutton_dateislocale),
						 _
						 ("Date format user defined"));
  if (my_config->date_format_type != DATE_FORMAT_IS_USERDEFINED)
    dateislocale = FALSE;
  else
    dateislocale = TRUE;
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_dateisusersettings),
				dateislocale);
  gtk_widget_show (checkbutton_dateisusersettings);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_dateisusersettings,
		      FALSE, FALSE, 0);

  entry_dateislocale = gtk_entry_new_with_max_length (255);

  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  label_dateislocale = gtk_label_new (_("Date format:"));
  gtk_widget_show (label_dateislocale);
  gtk_box_pack_start (GTK_BOX (hbox), label_dateislocale, FALSE, FALSE, 0);

  if (my_config->date_format)
    {
      date_format = g_strdup (my_config->date_format);
    }
  else
    date_format = g_strdup ("");

  gtk_entry_set_text (GTK_ENTRY (entry_dateislocale), date_format);


  gtk_editable_set_editable (GTK_EDITABLE (entry_dateislocale), TRUE);
  gtk_widget_set_sensitive (GTK_WIDGET (entry_dateislocale), dateislocale);
  gtk_box_pack_start (GTK_BOX (hbox), entry_dateislocale, FALSE, FALSE, 0);
  gtk_widget_show (entry_dateislocale);
  gtk_signal_connect (GTK_OBJECT (checkbutton_dateisusersettings), "clicked",
		      GTK_SIGNAL_FUNC (switch_entry_date),
		      entry_dateislocale);

  /* Option to know whether to display the dir size or not */
  checkbutton_displaydirsize =
    gtk_check_button_new_with_label (_("Display directory sizes"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_displaydirsize),
				my_config->display_dir_size);
  gtk_widget_show (checkbutton_displaydirsize);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_displaydirsize, FALSE,
		      FALSE, 0);

  /* Option for size unit */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  size_format_label = gtk_label_new (_("Size format: "));
  gtk_widget_show (size_format_label);
  gtk_box_pack_start (GTK_BOX (hbox), size_format_label, FALSE, FALSE, 0);

  size_format = gtk_option_menu_new ();
  gtk_widget_show (size_format);
  gtk_box_pack_start (GTK_BOX (hbox), size_format, FALSE, FALSE, 0);
  size_format_menu = gtk_menu_new ();
  gtk_widget_show (size_format_menu);
  size_format_menu_item = gtk_menu_item_new_with_label (_("Bytes"));
  gtk_object_set_user_data (GTK_OBJECT (size_format_menu_item),
			    GINT_TO_POINTER (SIZE_FORMAT_B));
  gtk_widget_show (size_format_menu_item);
  gtk_menu_append (GTK_MENU (size_format_menu), size_format_menu_item);
  size_format_menu_item = gtk_menu_item_new_with_label (_("KiloBytes"));
  gtk_object_set_user_data (GTK_OBJECT (size_format_menu_item),
			    GINT_TO_POINTER (SIZE_FORMAT_KB));
  gtk_widget_show (size_format_menu_item);
  gtk_menu_append (GTK_MENU (size_format_menu), size_format_menu_item);
  size_format_menu_item = gtk_menu_item_new_with_label (_("MegaBytes"));
  gtk_object_set_user_data (GTK_OBJECT (size_format_menu_item),
			    GINT_TO_POINTER (SIZE_FORMAT_MB));
  gtk_widget_show (size_format_menu_item);
  gtk_menu_append (GTK_MENU (size_format_menu), size_format_menu_item);
  size_format_menu_item = gtk_menu_item_new_with_label (_("GigaBytes"));
  gtk_object_set_user_data (GTK_OBJECT (size_format_menu_item),
			    GINT_TO_POINTER (SIZE_FORMAT_GB));
  gtk_widget_show (size_format_menu_item);
  gtk_menu_append (GTK_MENU (size_format_menu), size_format_menu_item);
  size_format_menu_item = gtk_menu_item_new_with_label (_("Human readable"));
  gtk_object_set_user_data (GTK_OBJECT (size_format_menu_item),
			    GINT_TO_POINTER (SIZE_FORMAT_HUMAN));
  gtk_widget_show (size_format_menu_item);
  gtk_menu_append (GTK_MENU (size_format_menu), size_format_menu_item);
  size_format_menu_item =
    gtk_menu_item_new_with_label (_("Bytes & Human readable"));
  gtk_object_set_user_data (GTK_OBJECT (size_format_menu_item),
			    GINT_TO_POINTER (SIZE_FORMAT_B_HUMAN));
  gtk_widget_show (size_format_menu_item);
  gtk_menu_append (GTK_MENU (size_format_menu), size_format_menu_item);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (size_format), size_format_menu);
  switch (my_config->size_format)
    {
    case SIZE_FORMAT_B:
      gtk_option_menu_set_history (GTK_OPTION_MENU (size_format), 0);
      break;
    case SIZE_FORMAT_KB:
      gtk_option_menu_set_history (GTK_OPTION_MENU (size_format), 1);
      break;
    case SIZE_FORMAT_MB:
      gtk_option_menu_set_history (GTK_OPTION_MENU (size_format), 2);
      break;
    case SIZE_FORMAT_GB:
      gtk_option_menu_set_history (GTK_OPTION_MENU (size_format), 3);
      break;
    case SIZE_FORMAT_HUMAN:
      gtk_option_menu_set_history (GTK_OPTION_MENU (size_format), 4);
      break;
    case SIZE_FORMAT_B_HUMAN:
      gtk_option_menu_set_history (GTK_OPTION_MENU (size_format), 5);
      break;
    }

  /* Option for the number of recent files */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  recentfilesmax_label = gtk_label_new (_("Max number of recent files:"));
  gtk_widget_show (recentfilesmax_label);
  gtk_box_pack_start (GTK_BOX (hbox), recentfilesmax_label, FALSE, FALSE, 0);

  recentfilesmax_adj =
    (GtkAdjustment *) gtk_adjustment_new (my_config->nb_max_recent_files, 1,
					  50, 1, 1, 50);
  recentfilesmax =
    gtk_spin_button_new (GTK_ADJUSTMENT (recentfilesmax_adj), 1, 0);
  gtk_widget_show (recentfilesmax);
  gtk_box_pack_start (GTK_BOX (hbox), recentfilesmax, FALSE, FALSE, 0);

  /* Option to know if we save the dimensions of the tree window */
  checkbutton_save_ctreewin_wh =
    gtk_check_button_new_with_label (_("Save the left tree window size"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_save_ctreewin_wh),
				my_config->ctreewindow_save_wh);
  gtk_widget_show (checkbutton_save_ctreewin_wh);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_save_ctreewin_wh,
		      FALSE, FALSE, 0);

  /* Option to know if we save the dimensions and position of the main window */
  checkbutton_save_mainwin_xywh =
    gtk_check_button_new_with_label (_
				     ("Save the main window position and size"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_save_mainwin_xywh),
				my_config->mainwindow_save_xywh);
  gtk_widget_show (checkbutton_save_mainwin_xywh);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_save_mainwin_xywh,
		      FALSE, FALSE, 0);

  /* Option for toolbar style */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  toolbar_style_label = gtk_label_new (_("Toolbar style: "));
  gtk_widget_show (toolbar_style_label);
  gtk_box_pack_start (GTK_BOX (hbox), toolbar_style_label, FALSE, FALSE, 0);

  toolbar_style_mode = gtk_option_menu_new ();
  gtk_widget_show (toolbar_style_mode);
  gtk_box_pack_start (GTK_BOX (hbox), toolbar_style_mode, FALSE, FALSE, 0);
  toolbar_style_mode_menu = gtk_menu_new ();
  gtk_widget_show (toolbar_style_mode_menu);
  toolbar_style_mode_menuitem =
    gtk_menu_item_new_with_label (_("Text & Icons"));
  gtk_object_set_user_data (GTK_OBJECT (toolbar_style_mode_menuitem),
			    GINT_TO_POINTER (GTK_TOOLBAR_BOTH));
  gtk_widget_show (toolbar_style_mode_menuitem);
  gtk_menu_append (GTK_MENU (toolbar_style_mode_menu),
		   toolbar_style_mode_menuitem);
  toolbar_style_mode_menuitem = gtk_menu_item_new_with_label (_("Text"));
  gtk_object_set_user_data (GTK_OBJECT (toolbar_style_mode_menuitem),
			    GINT_TO_POINTER (GTK_TOOLBAR_TEXT));
  gtk_widget_show (toolbar_style_mode_menuitem);
  gtk_menu_append (GTK_MENU (toolbar_style_mode_menu),
		   toolbar_style_mode_menuitem);
  toolbar_style_mode_menuitem = gtk_menu_item_new_with_label (_("Icons"));
  gtk_object_set_user_data (GTK_OBJECT (toolbar_style_mode_menuitem),
			    GINT_TO_POINTER (GTK_TOOLBAR_ICONS));
  gtk_widget_show (toolbar_style_mode_menuitem);
  gtk_menu_append (GTK_MENU (toolbar_style_mode_menu),
		   toolbar_style_mode_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (toolbar_style_mode),
			    toolbar_style_mode_menu);
  switch (my_config->toolbar_type)
    {
    case GTK_TOOLBAR_BOTH:
      gtk_option_menu_set_history (GTK_OPTION_MENU (toolbar_style_mode), 0);
      break;
    case GTK_TOOLBAR_TEXT:
      gtk_option_menu_set_history (GTK_OPTION_MENU (toolbar_style_mode), 1);
      break;
    case GTK_TOOLBAR_ICONS:
      gtk_option_menu_set_history (GTK_OPTION_MENU (toolbar_style_mode), 2);
      break;
    }

  /* Option for new search type */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  new_search_type_label = gtk_label_new (_("New search type: "));
  gtk_widget_show (new_search_type_label);
  gtk_box_pack_start (GTK_BOX (hbox), new_search_type_label, FALSE, FALSE, 0);

  new_search_type_mode = gtk_option_menu_new ();
  gtk_widget_show (new_search_type_mode);
  gtk_box_pack_start (GTK_BOX (hbox), new_search_type_mode, FALSE, FALSE, 0);
  new_search_type_mode_menu = gtk_menu_new ();
  gtk_widget_show (new_search_type_mode_menu);
  new_search_type_mode_menuitem = gtk_menu_item_new_with_label (_("Simple"));
  gtk_object_set_user_data (GTK_OBJECT (new_search_type_mode_menuitem),
			    GINT_TO_POINTER (SEARCH_TYPE_SIMPLE));
  gtk_widget_show (new_search_type_mode_menuitem);
  gtk_menu_append (GTK_MENU (new_search_type_mode_menu),
		   new_search_type_mode_menuitem);
  new_search_type_mode_menuitem =
    gtk_menu_item_new_with_label (_("Extended"));
  gtk_object_set_user_data (GTK_OBJECT (new_search_type_mode_menuitem),
			    GINT_TO_POINTER (SEARCH_TYPE_EXTENDED));
  gtk_widget_show (new_search_type_mode_menuitem);
  gtk_menu_append (GTK_MENU (new_search_type_mode_menu),
		   new_search_type_mode_menuitem);
  new_search_type_mode_menuitem =
    gtk_menu_item_new_with_label (_("Last used"));
  gtk_object_set_user_data (GTK_OBJECT (new_search_type_mode_menuitem),
			    GINT_TO_POINTER (SEARCH_TYPE_LAST_SIMPLE));
  gtk_widget_show (new_search_type_mode_menuitem);
  gtk_menu_append (GTK_MENU (new_search_type_mode_menu),
		   new_search_type_mode_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (new_search_type_mode),
			    new_search_type_mode_menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (new_search_type_mode),
			       my_config->new_search_type);

  /* Option to know whether to display the splash screen or not */
  checkbutton_displaysplash =
    gtk_check_button_new_with_label (_("Display the splash screen"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_displaysplash),
				my_config->display_splash);
  gtk_widget_show (checkbutton_displaysplash);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_displaysplash, FALSE,
		      FALSE, 0);


/**********************************************************************/
/* Frame for Load&save Options ****************************************/
/**********************************************************************/

  frame_loadsave = gtk_frame_new (_("Load&save Options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_loadsave), 6);
  gtk_widget_show (frame_loadsave);

  treetitles[0] = g_strdup (_("Load&save Options"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__LoadSave_options);
  setup_node =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), NULL, NULL, treetitles, 3,
			   pix, pix_mask, pix, pix_mask, FALSE, TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_node, frame_loadsave);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_loadsave), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 3);
  gtk_container_set_border_width (GTK_CONTAINER (main_box), 6);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);

  /* Common frame */
  frame = gtk_frame_new (_("Common options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame), 3);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 0);

  box_in_frame = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (box_in_frame), 6);
  gtk_widget_show (box_in_frame);
  gtk_container_add (GTK_CONTAINER (frame), box_in_frame);

  label = gtk_label_new (_("Default working path"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_box_pack_start (GTK_BOX (box_in_frame), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (box_in_frame), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  gnome_entry_dwpath
    = gnome_file_entry_new (NULL, _("Select default working path"));

  gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (gnome_entry_dwpath),
				     my_config->default_working_path);
  gtk_entry_set_text (GTK_ENTRY
		      (gnome_file_entry_gtk_entry
		       (GNOME_FILE_ENTRY (gnome_entry_dwpath))),
		      my_config->default_working_path);
  gnome_file_entry_set_directory (GNOME_FILE_ENTRY (gnome_entry_dwpath),
				  TRUE);
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_entry_dwpath), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox), gnome_entry_dwpath, FALSE, FALSE, 0);
  gtk_widget_show (gnome_entry_dwpath);

  /* Load frame */
  frame = gtk_frame_new (_("Load options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame), 3);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 0);

  box_in_frame = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (box_in_frame), 6);
  gtk_widget_show (box_in_frame);
  gtk_container_add (GTK_CONTAINER (frame), box_in_frame);


  /* Option to know if we load the filename at boot or not. */
  if (my_config->load_file_at_boot)
    load_file_at_boot = FALSE;
  else
    load_file_at_boot = TRUE;
  checkbutton_loadatboot_mode =
    gtk_check_button_new_with_label (_("Load file at boot"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_loadatboot_mode),
				my_config->load_file_at_boot);
  gtk_widget_show (checkbutton_loadatboot_mode);
  gtk_box_pack_start (GTK_BOX (box_in_frame), checkbutton_loadatboot_mode,
		      FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (box_in_frame), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  if (my_config->file_to_load)
    {
      file_to_load = g_strdup (my_config->file_to_load);
    }
  else
    file_to_load = g_strdup ("");
  gnome_entry_filetoload = gnome_file_entry_new (NULL, _("Select a file"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY (gnome_entry_filetoload),
			      TRUE);
  entry_filetoload =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gnome_entry_filetoload));
  gtk_entry_set_text (GTK_ENTRY (entry_filetoload), file_to_load);

  gtk_editable_set_editable (GTK_EDITABLE (entry_filetoload), TRUE);
  gtk_widget_set_sensitive (GTK_WIDGET (gnome_entry_filetoload),
			    !load_file_at_boot);
  gtk_box_pack_start (GTK_BOX (hbox), gnome_entry_filetoload, FALSE, FALSE,
		      0);
  gtk_widget_show (gnome_entry_filetoload);
  gtk_signal_connect (GTK_OBJECT (checkbutton_loadatboot_mode), "clicked",
		      GTK_SIGNAL_FUNC (switch_entry_file),
		      gnome_entry_filetoload);

  /* Save frame */
  frame = gtk_frame_new (_("Save options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame), 3);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 0);

  box_in_frame = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (box_in_frame), 6);
  gtk_widget_show (box_in_frame);
  gtk_container_add (GTK_CONTAINER (frame), box_in_frame);


  /* Option to know if autosave should be enabled or not. */
  checkbutton_autosave_mode = gtk_check_button_new_with_label (_("Autosave"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_autosave_mode),
				my_config->autosave);
  gtk_widget_show (checkbutton_autosave_mode);
  gtk_box_pack_start (GTK_BOX (box_in_frame), checkbutton_autosave_mode,
		      FALSE, FALSE, 0);

  /* Option for 'save every' level */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (box_in_frame), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  save_every_label =
    gtk_label_new (_("Time between autosaves (in minutes):"));
  gtk_widget_show (save_every_label);
  gtk_box_pack_start (GTK_BOX (hbox), save_every_label, FALSE, FALSE, 0);

  save_every_adj =
    (GtkAdjustment *) gtk_adjustment_new (my_config->minutes_before_autosave,
					  1, 60, 1, 1, 60);
  save_every = gtk_spin_button_new (GTK_ADJUSTMENT (save_every_adj), 1, 0);
  gtk_widget_show (save_every);
  gtk_box_pack_start (GTK_BOX (hbox), save_every, FALSE, FALSE, 0);

  gtk_widget_set_sensitive (GTK_WIDGET (save_every), my_config->autosave);
  gtk_signal_connect (GTK_OBJECT (checkbutton_autosave_mode), "clicked",
		      GTK_SIGNAL_FUNC (switch_entry_file), save_every);

  /* Option for compression level */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (box_in_frame), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  compression_level_label = gtk_label_new (_("Compression level:"));
  gtk_widget_show (compression_level_label);
  gtk_box_pack_start (GTK_BOX (hbox), compression_level_label, FALSE, FALSE,
		      0);

  compression_level_adj =
    (GtkAdjustment *) gtk_adjustment_new (my_config->compression_level, 0, 9,
					  1, 1, 9);
  compression_level =
    gtk_spin_button_new (GTK_ADJUSTMENT (compression_level_adj), 1, 0);
  gtk_widget_show (compression_level);
  gtk_box_pack_start (GTK_BOX (hbox), compression_level, FALSE, FALSE, 0);

  /* Option to know if we save all the categories on only the used ones */
  checkbutton_save_cat_mode =
    gtk_check_button_new_with_label (_("Save only used categories"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_save_cat_mode),
				my_config->save_only_used_categories);
  gtk_widget_show (checkbutton_save_cat_mode);
  gtk_box_pack_start (GTK_BOX (box_in_frame), checkbutton_save_cat_mode,
		      FALSE, FALSE, 0);

  /* Option to know if we keep a backup of the file. */
  checkbutton_keepbackup_mode =
    gtk_check_button_new_with_label (_("Keep a backup file"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_keepbackup_mode),
				my_config->keep_bakup_file);
  gtk_widget_show (checkbutton_keepbackup_mode);
  gtk_box_pack_start (GTK_BOX (box_in_frame), checkbutton_keepbackup_mode,
		      FALSE, FALSE, 0);


/**********************************************************************/
/* Frame for Scan Options *********************************************/
/**********************************************************************/

  frame_scan = gtk_frame_new (_("Scan Options"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_scan), 6);
  gtk_widget_show (frame_scan);

  treetitles[0] = g_strdup (_("Scan Options"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__Scan_options);
  setup_node =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), NULL, NULL, treetitles, 3,
			   pix, pix_mask, pix, pix_mask, FALSE, TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree), setup_node, NULL);


/**********************************************************************/
/* Frame for Scan mime types ******************************************/
/**********************************************************************/

  frame_scan_general = gtk_frame_new (_("Scan mime types"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_scan_general), 6);
  gtk_widget_show (frame_scan_general);

  treetitles[0] = g_strdup (_("Scan mime types"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__Scan_mime_types);
  setup_node2 =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), setup_node, NULL,
			   treetitles, 3, pix, pix_mask, pix, pix_mask, FALSE,
			   TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_node2, frame_scan_general);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_scan_general), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (main_box), 6);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);

  /* Option to know if we scan mime types or not */
  checkbutton_scan_mime_type =
    gtk_check_button_new_with_label (_("Try to get the MIME"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_scan_mime_type),
				my_config->scan_mime_types);
  gtk_widget_show (checkbutton_scan_mime_type);
  gtk_signal_connect (GTK_OBJECT (checkbutton_scan_mime_type),
		      "clicked", GTK_SIGNAL_FUNC (switch_mime_options), NULL);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_scan_mime_type, FALSE,
		      TRUE, 0);


  /* Option to scan vfs */
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

  test_vfs_mode_label = gtk_label_new (_("Virtual File System scan mode: "));
  gtk_widget_show (test_vfs_mode_label);
  gtk_box_pack_start (GTK_BOX (hbox), test_vfs_mode_label, FALSE, TRUE, 0);

  test_vfs_mode = gtk_option_menu_new ();
  gtk_widget_show (test_vfs_mode);
  gtk_box_pack_start (GTK_BOX (hbox), test_vfs_mode, FALSE, FALSE, 0);
  test_vfs_mode_menu = gtk_menu_new ();
  gtk_widget_show (test_vfs_mode_menu);
  test_vfs_mode_menuitem =
    gtk_menu_item_new_with_label (_("Do not scan VFS"));
  gtk_object_set_user_data (GTK_OBJECT (test_vfs_mode_menuitem),
			    GINT_TO_POINTER (TEST_VFS_NO));
  gtk_widget_show (test_vfs_mode_menuitem);
  gtk_menu_append (GTK_MENU (test_vfs_mode_menu), test_vfs_mode_menuitem);
  test_vfs_mode_menuitem = gtk_menu_item_new_with_label (_("Scan VFS"));
  gtk_object_set_user_data (GTK_OBJECT (test_vfs_mode_menuitem),
			    GINT_TO_POINTER (TEST_VFS_YES));
  gtk_widget_show (test_vfs_mode_menuitem);
  gtk_menu_append (GTK_MENU (test_vfs_mode_menu), test_vfs_mode_menuitem);
  test_vfs_mode_menuitem = gtk_menu_item_new_with_label (_("Full scan VFS"));
  gtk_object_set_user_data (GTK_OBJECT (test_vfs_mode_menuitem),
			    GINT_TO_POINTER (TEST_VFS_EXTENDED));
  gtk_widget_show (test_vfs_mode_menuitem);
  gtk_menu_append (GTK_MENU (test_vfs_mode_menu), test_vfs_mode_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (test_vfs_mode),
			    test_vfs_mode_menu);
  switch (my_config->testvfs)
    {
    case TEST_VFS_NO:
      gtk_option_menu_set_history (GTK_OPTION_MENU (test_vfs_mode), 0);
      break;
    case TEST_VFS_YES:
      gtk_option_menu_set_history (GTK_OPTION_MENU (test_vfs_mode), 1);
      break;
    case TEST_VFS_EXTENDED:
      gtk_option_menu_set_history (GTK_OPTION_MENU (test_vfs_mode), 2);
      break;
    }
  gtk_signal_connect (GTK_OBJECT (test_vfs_mode_menu), "selection-done",
		      GTK_SIGNAL_FUNC (switch_vfs_options), NULL);


  /* Option to read the information about files */
  checkbutton_getinformation =
    gtk_check_button_new_with_label (_("Retrieve information about files"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_getinformation),
				my_config->getInformation);
  gtk_signal_connect (GTK_OBJECT (checkbutton_getinformation), "clicked",
		      GTK_SIGNAL_FUNC (switch_info_options), NULL);

  gtk_widget_show (checkbutton_getinformation);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_getinformation, FALSE,
		      TRUE, 0);


  /* The default information plugin: use it or not? */
  checkbutton_usedefaultinfoplugin =
    gtk_check_button_new_with_label (_("Use the default information plugin"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				(checkbutton_usedefaultinfoplugin),
				my_config->use_default_information_plugin);
  gtk_widget_show (checkbutton_usedefaultinfoplugin);
  gtk_signal_connect (GTK_OBJECT (checkbutton_usedefaultinfoplugin),
		      "clicked", GTK_SIGNAL_FUNC (switch_defaultinfo_options),
		      NULL);
  gtk_box_pack_start (GTK_BOX (main_box), checkbutton_usedefaultinfoplugin,
		      FALSE, FALSE, 0);

  /* The default information plugin: program */
  table = gtk_table_new (2, 2, FALSE);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
  label = gtk_label_new (_("Program:"));
  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);
  gtk_widget_show (label);

  gnome_default_information_plugin_entry =
    gnome_file_entry_new (NULL, _("Select the default plugin"));
  gnome_file_entry_set_modal (GNOME_FILE_ENTRY
			      (gnome_default_information_plugin_entry), TRUE);
  default_information_plugin_entry =
    gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY
				(gnome_default_information_plugin_entry));
  gtk_entry_set_text (GTK_ENTRY (default_information_plugin_entry),
		      my_config->default_information_plugin->prog->str);
  gtk_table_attach_defaults (GTK_TABLE (table),
			     gnome_default_information_plugin_entry, 1, 2, 0,
			     1);
  gtk_widget_show (gnome_default_information_plugin_entry);

  /* The default information plugin: Args */
  label = gtk_label_new (_("Args:"));
  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		    GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);

  default_information_plugin_args_entry = gtk_entry_new_with_max_length (200);
  gtk_entry_set_text (GTK_ENTRY (default_information_plugin_args_entry),
		      my_config->default_information_plugin->arg->str);
  gtk_table_attach_defaults (GTK_TABLE (table),
			     default_information_plugin_args_entry,
			     1, 2, 1, 2);
  gtk_widget_show (label);
  gtk_widget_show (default_information_plugin_args_entry);

  gtk_box_pack_start (GTK_BOX (main_box), table, FALSE, FALSE, 0);
  gtk_widget_show (table);


/**********************************************************************/
/* Frame for Virtual File Systems *************************************/
/**********************************************************************/

  frame_scan_VFS = gtk_frame_new (_("Virtual File Systems"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_scan_VFS), 6);
  gtk_widget_show (frame_scan_VFS);

  treetitles[0] = g_strdup (_("Virtual File Systems"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__Virtual_File_Systems);
  setup_scan_vfs =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), setup_node, NULL,
			   treetitles, 3, pix, pix_mask, pix, pix_mask, FALSE,
			   TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_scan_vfs, frame_scan_VFS);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_scan_VFS), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);

  /* The vfs plugin list */
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  gtk_box_pack_start (GTK_BOX (main_box), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (scrolled_window);

  /* setup box to associate vfs plugins to mimes */
  vfs_include_list_box = gtk_clist_new_with_titles (4, titles);
  gtk_signal_connect (GTK_OBJECT (vfs_include_list_box),
		      "select_row",
		      GTK_SIGNAL_FUNC (vfs_include_selection_made), NULL);
  gtk_signal_connect (GTK_OBJECT (vfs_include_list_box), "unselect_row",
		      GTK_SIGNAL_FUNC (vfs_include_unselection_made), NULL);

  gtk_clist_set_shadow_type (GTK_CLIST (vfs_include_list_box),
			     GTK_SHADOW_OUT);
/*
  gtk_clist_set_column_width (GTK_CLIST (vfs_include_list_box), 0, 25);
  gtk_clist_set_column_width (GTK_CLIST (vfs_include_list_box), 1, 200);
  gtk_clist_set_column_width (GTK_CLIST (vfs_include_list_box), 2, 150);
*/
  gtk_clist_set_column_justification (GTK_CLIST
				      (vfs_include_list_box), 0,
				      GTK_JUSTIFY_CENTER);
/*  gtk_clist_set_column_auto_resize (GTK_CLIST (vfs_include_list_box), 3,
				    TRUE);
*/
  gtk_clist_set_compare_func (GTK_CLIST (vfs_include_list_box),
			      _clist_g_strcasecmp);
  gtk_clist_set_sort_column (GTK_CLIST (vfs_include_list_box), 1);
  gtk_clist_set_sort_type (GTK_CLIST (vfs_include_list_box),
			   GTK_SORT_ASCENDING);

  /* fill the clist */
  g_tree_traverse (local_vfs_extensions, vfs_extensions_traverse, G_PRE_ORDER,
		   NULL);

  /* show the clist */
  gtk_container_add (GTK_CONTAINER (scrolled_window), vfs_include_list_box);
  gtk_widget_show (vfs_include_list_box);
  gtk_clist_sort (GTK_CLIST (vfs_include_list_box));
  gtk_clist_columns_autosize (GTK_CLIST (vfs_include_list_box));

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

  add_button = gtk_button_new_with_label (_("Add"));
  GTK_WIDGET_SET_FLAGS (add_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), add_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
		      GTK_SIGNAL_FUNC (vfs_add_type),
		      GINT_TO_POINTER (ADD_MODE));
  gtk_widget_show (add_button);

  edit_button = gtk_button_new_with_label (_("Edit"));
  GTK_WIDGET_SET_FLAGS (edit_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), edit_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (edit_button), "clicked",
		      GTK_SIGNAL_FUNC (vfs_include_edit_button_clicked),
		      NULL);
  gtk_widget_show (edit_button);

  delete_button = gtk_button_new_with_label (_("Delete"));
  GTK_WIDGET_SET_FLAGS (delete_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), delete_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
		      GTK_SIGNAL_FUNC (vfs_include_delete_button_clicked),
		      NULL);
  gtk_widget_show (delete_button);

  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

/**********************************************************************/
/* Frame for Information plugins **************************************/
/**********************************************************************/

  frame_scan_plugins = gtk_frame_new (_("Information plugins"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_scan_plugins), 6);
  gtk_widget_show (frame_scan_plugins);

  treetitles[0] = g_strdup (_("Information plugins"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__Information_plugins);
  setup_scan_info =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), setup_node, NULL,
			   treetitles, 3, pix, pix_mask, pix, pix_mask, FALSE,
			   TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_scan_info, frame_scan_plugins);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_scan_plugins), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);



  /* The info plugin list */
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  gtk_box_pack_start (GTK_BOX (main_box), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (scrolled_window);

  /* setup box to associate programs to file extensions */
  include_list_box = gtk_clist_new_with_titles (4, titles);
  gtk_signal_connect (GTK_OBJECT (include_list_box),
		      "select_row", GTK_SIGNAL_FUNC (include_selection_made),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (include_list_box),
		      "unselect_row",
		      GTK_SIGNAL_FUNC (include_unselection_made), NULL);

  gtk_clist_set_shadow_type (GTK_CLIST (include_list_box), GTK_SHADOW_OUT);
/*
  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 0, 25);
  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 1, 200);
  gtk_clist_set_column_width (GTK_CLIST (include_list_box), 2, 150);
  gtk_clist_set_column_justification (GTK_CLIST
				      (include_list_box), 0,
				      GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_auto_resize (GTK_CLIST (include_list_box), 3, TRUE);
*/

  /* fill the clist */
  g_tree_traverse (local_extensions, extensions_traverse, G_PRE_ORDER, NULL);

  /* show the clist */
  gtk_container_add (GTK_CONTAINER (scrolled_window), include_list_box);
  gtk_widget_show (include_list_box);
  gtk_clist_columns_autosize (GTK_CLIST (include_list_box));

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

  add_button = gtk_button_new_with_label (_("Add"));
  GTK_WIDGET_SET_FLAGS (add_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), add_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
		      GTK_SIGNAL_FUNC (add_type), GINT_TO_POINTER (ADD_MODE));
  gtk_widget_show (add_button);

  edit_button = gtk_button_new_with_label (_("Edit"));
  GTK_WIDGET_SET_FLAGS (edit_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), edit_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (edit_button), "clicked",
		      GTK_SIGNAL_FUNC (include_edit_button_clicked), NULL);
  gtk_widget_show (edit_button);

  delete_button = gtk_button_new_with_label (_("Delete"));
  GTK_WIDGET_SET_FLAGS (delete_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), delete_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
		      GTK_SIGNAL_FUNC (include_delete_button_clicked), NULL);
  gtk_widget_show (delete_button);

  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

/**********************************************************************/
/* Frame for MIME types to skip during a scan *************************/
/**********************************************************************/

  frame_scan_mimenoscan =
    gtk_frame_new (_("MIME types to skip during a scan"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_scan_mimenoscan), 6);
  gtk_widget_show (frame_scan_mimenoscan);

  treetitles[0] = g_strdup (_("MIME types to skip during a scan"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **)
				  setup__MIME_types_to_skip_during_a_scan);
  setup_scan_mime =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), setup_node, NULL,
			   treetitles, 3, pix, pix_mask, pix, pix_mask, FALSE,
			   TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_scan_mime, frame_scan_mimenoscan);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_scan_mimenoscan), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_widget_show (main_box);



  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  gtk_box_pack_start (GTK_BOX (main_box), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (scrolled_window);

  /* setup box to associate icons to file extensions */
  ess_include_list_box = gtk_clist_new_with_titles (2, esstitles);
  gtk_clist_set_shadow_type (GTK_CLIST (ess_include_list_box),
			     GTK_SHADOW_OUT);
/*  gtk_clist_set_column_width (GTK_CLIST (ess_include_list_box), 0, 25);
*/
  gtk_clist_columns_autosize (GTK_CLIST (ess_include_list_box));
  gtk_clist_set_column_justification (GTK_CLIST
				      (ess_include_list_box), 0,
				      GTK_JUSTIFY_CENTER);
/*  gtk_clist_set_column_auto_resize (GTK_CLIST (ess_include_list_box), 1,
				    TRUE);
*/
  gtk_clist_set_compare_func (GTK_CLIST (ess_include_list_box),
			      _ess_clist_g_strcasecmp);
  gtk_clist_set_sort_column (GTK_CLIST (ess_include_list_box), 1);
  gtk_clist_set_sort_type (GTK_CLIST (ess_include_list_box),
			   GTK_SORT_ASCENDING);
  /* fill the clist */
  g_tree_traverse (local_ess_extensions, ess_extensions_traverse, G_PRE_ORDER,
		   NULL);

  gtk_signal_connect (GTK_OBJECT (ess_include_list_box),
		      "select_row",
		      GTK_SIGNAL_FUNC (ess_include_selection_made), NULL);
  gtk_signal_connect (GTK_OBJECT (ess_include_list_box), "unselect_row",
		      GTK_SIGNAL_FUNC (ess_include_unselection_made), NULL);

  /* show the clist */
  gtk_container_add (GTK_CONTAINER (scrolled_window), ess_include_list_box);
  gtk_widget_show (ess_include_list_box);
  gtk_clist_sort (GTK_CLIST (ess_include_list_box));
  gtk_clist_columns_autosize (GTK_CLIST (ess_include_list_box));

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

  ess_button_add = gtk_button_new_with_label (_("Add"));
  GTK_WIDGET_SET_FLAGS (ess_button_add, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), ess_button_add, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (ess_button_add), "clicked",
		      GTK_SIGNAL_FUNC (ess_add_type),
		      GINT_TO_POINTER (ADD_MODE));

  gtk_widget_show (ess_button_add);

  ess_button_del = gtk_button_new_with_label (_("Delete"));
  GTK_WIDGET_SET_FLAGS (ess_button_del, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), ess_button_del, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (ess_button_del), "clicked",
		      GTK_SIGNAL_FUNC (ess_include_delete_button_clicked),
		      NULL);

  gtk_widget_set_sensitive (GTK_WIDGET (ess_button_del), FALSE);
  gtk_widget_show (ess_button_del);

  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

/**********************************************************************/
/* Frame for Mime type icons ******************************************/
/**********************************************************************/

  frame_mime = gtk_frame_new (_("Mime type icons"));
  gtk_container_set_border_width (GTK_CONTAINER (frame_mime), 6);
  gtk_widget_show (frame_mime);

  treetitles[0] = g_strdup (_("MIME type icons"));
  pix =
    gdk_pixmap_create_from_xpm_d (window->window,
				  &(pix_mask),
				  &(style)->bg[GTK_STATE_NORMAL],
				  (gchar **) setup__MIME_type_icons);
  setup_node =
    gtk_ctree_insert_node (GTK_CTREE (setup_tree), NULL, NULL, treetitles, 3,
			   pix, pix_mask, pix, pix_mask, FALSE, TRUE);
  g_free (treetitles[0]);
  gtk_ctree_node_set_row_data (GTK_CTREE (setup_tree),
			       setup_node, frame_mime);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (frame_mime), scrolled_window);
  gtk_widget_show (scrolled_window);

  main_box = gtk_vbox_new (FALSE, 2);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), main_box);
  gtk_clist_columns_autosize (GTK_CLIST (&(GTK_CTREE (setup_tree)->clist)));

  gtk_widget_set_usize (GTK_WIDGET (setup_tree),
			GTK_WIDGET (setup_tree)->requisition.width,
			2.8 * (GTK_WIDGET (setup_tree)->requisition.height));
  gtk_widget_show (main_box);



  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  gtk_box_pack_start (GTK_BOX (main_box), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (scrolled_window);

  /* setup box to associate icons to file extensions */
  ei_include_list_box = gtk_clist_new_with_titles (3, eititles);
  gtk_clist_set_shadow_type (GTK_CLIST (ei_include_list_box), GTK_SHADOW_OUT);
/*
  gtk_clist_set_column_width (GTK_CLIST (ei_include_list_box), 0, 25);
  gtk_clist_set_column_width (GTK_CLIST (ei_include_list_box), 1, 200);
*/
  gtk_clist_set_column_justification (GTK_CLIST
				      (ei_include_list_box), 0,
				      GTK_JUSTIFY_CENTER);
/*  gtk_clist_set_column_auto_resize (GTK_CLIST (ei_include_list_box), 2, TRUE);
*/
  gtk_clist_set_compare_func (GTK_CLIST (ei_include_list_box),
			      _ei_clist_g_strcasecmp);
  gtk_clist_set_sort_column (GTK_CLIST (ei_include_list_box), 1);
  gtk_clist_set_sort_type (GTK_CLIST (ei_include_list_box),
			   GTK_SORT_ASCENDING);
  /* fill the clist */
  g_tree_traverse (local_ei_extensions, ei_extensions_traverse, G_PRE_ORDER,
		   NULL);

  gtk_signal_connect (GTK_OBJECT (ei_include_list_box),
		      "select_row",
		      GTK_SIGNAL_FUNC (ei_include_selection_made), NULL);
  gtk_signal_connect (GTK_OBJECT (ei_include_list_box), "unselect_row",
		      GTK_SIGNAL_FUNC (ei_include_unselection_made), NULL);

  /* show the clist */
  gtk_container_add (GTK_CONTAINER (scrolled_window), ei_include_list_box);
  gtk_widget_show (ei_include_list_box);
  gtk_clist_sort (GTK_CLIST (ei_include_list_box));
  gtk_clist_columns_autosize (GTK_CLIST (ei_include_list_box));

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);

  ei_button_add = gtk_button_new_with_label (_("Add"));
  GTK_WIDGET_SET_FLAGS (ei_button_add, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), ei_button_add, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (ei_button_add), "clicked",
		      GTK_SIGNAL_FUNC (ei_add_type),
		      GINT_TO_POINTER (ADD_MODE));

  gtk_widget_show (ei_button_add);

  ei_button_edit = gtk_button_new_with_label (_("Edit"));
  GTK_WIDGET_SET_FLAGS (ei_button_edit, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), ei_button_edit, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (ei_button_edit), "clicked",
		      GTK_SIGNAL_FUNC (ei_include_edit_button_clicked), NULL);

  gtk_widget_set_sensitive (GTK_WIDGET (ei_button_edit), FALSE);
  gtk_widget_show (ei_button_edit);

  ei_button_del = gtk_button_new_with_label (_("Delete"));
  GTK_WIDGET_SET_FLAGS (ei_button_del, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (hbox), ei_button_del, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (ei_button_del), "clicked",
		      GTK_SIGNAL_FUNC (ei_include_delete_button_clicked),
		      NULL);

  gtk_widget_set_sensitive (GTK_WIDGET (ei_button_del), FALSE);
  gtk_widget_show (ei_button_del);

  gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

/**********************************************************************/
/* Bottom buttons *****************************************************/
/**********************************************************************/

  bbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox), 5);
  gtk_box_pack_start (GTK_BOX (tree_vbox), bbox, FALSE, FALSE, 0);
  gtk_widget_show (bbox);

#ifdef GNOME1
  ok = gnome_stock_button (GNOME_STOCK_PIXMAP_SAVE);
#else
  ok = gtk_button_new_from_stock (GNOME_STOCK_PIXMAP_SAVE);
#endif
  gtk_object_set_user_data (GTK_OBJECT (ok), window);
  GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (bbox), ok, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
		      GTK_SIGNAL_FUNC (setup_ok_clicked), NULL);
  gtk_widget_show (ok);

#ifdef GNOME1
  cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
#else
  cancel = gtk_button_new_from_stock (GNOME_STOCK_BUTTON_CANCEL);
#endif
  GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (bbox), cancel, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (cancel), "clicked", G_CALLBACK(cancel_clicked), NULL);
  gtk_widget_show (cancel);

  gtk_widget_show (window);
  gtk_widget_add_accelerator (cancel, "clicked", accel_group,
			      GDK_Escape, 0, GTK_ACCEL_VISIBLE);

  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);


  selected_row = -1;
  selected_extension = NULL;

  vfs_selected_row = -1;
  vfs_selected_extension = NULL;

  ei_selected_row = -1;
  ei_selected_extension = NULL;

  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_mountpoint, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_general, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_loadsave, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_scan, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_scan_general, TRUE, TRUE,
		      0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_scan_VFS, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_scan_plugins, TRUE, TRUE,
		      0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_scan_mimenoscan, TRUE, TRUE,
		      0);
  gtk_box_pack_start (GTK_BOX (setup_vbox), frame_mime, TRUE, TRUE, 0);
  gtk_widget_hide (GTK_WIDGET (frame_general));
  gtk_widget_hide (GTK_WIDGET (frame_loadsave));
  gtk_widget_hide (GTK_WIDGET (frame_scan));
  gtk_widget_hide (GTK_WIDGET (frame_scan_general));
  gtk_widget_hide (GTK_WIDGET (frame_scan_VFS));
  gtk_widget_hide (GTK_WIDGET (frame_scan_plugins));
  gtk_widget_hide (GTK_WIDGET (frame_scan_mimenoscan));
  gtk_widget_hide (GTK_WIDGET (frame_mime));
  switch_info_options (checkbutton_getinformation, NULL);
  switch_mime_options (NULL, NULL);
  switch_vfs_options (NULL, NULL);
  current_frame = frame_mountpoint;
}
